@@ -36,7 +36,7 @@
 ** Added --single-transaction option 06/06/2002 by Peter Zaitsev
 */
 
-#define DUMP_VERSION "9.08"
+#define DUMP_VERSION "9.11"
 
 #include <my_global.h>
 #include <my_sys.h>
@@ -72,13 +72,13 @@ static char *add_load_option(char *ptr, const char *object,
 
 static char *field_escape(char *to,const char *from,uint length);
 static my_bool  verbose=0,tFlag=0,cFlag=0,dFlag=0,quick=0, extended_insert = 0,
-		lock_tables=0,ignore_errors=0,flush_logs=0,replace=0,
-		ignore=0,opt_drop=0,opt_keywords=0,opt_lock=0,opt_compress=0,
-                opt_delayed=0,create_options=0,opt_quoted=0,opt_databases=0,
-	        opt_alldbs=0,opt_create_db=0,opt_first_slave=0,
-                opt_autocommit=0,opt_master_data,opt_disable_keys=0,opt_xml=0,
-	        opt_delete_master_logs=0, tty_password=0,
-		opt_single_transaction=0;
+  lock_tables=0,ignore_errors=0,flush_logs=0,replace=0,
+  ignore=0,opt_drop=0,opt_keywords=0,opt_lock=0,opt_compress=0,
+  opt_delayed=0,create_options=0,opt_quoted=0,opt_databases=0,
+  opt_alldbs=0,opt_create_db=0,opt_first_slave=0,
+  opt_autocommit=0,opt_master_data,opt_disable_keys=0,opt_xml=0,
+  opt_delete_master_logs=0, tty_password=0,
+  opt_single_transaction=0, opt_comments= 0;
 static MYSQL  mysql_connection,*sock=0;
 static char  insert_pat[12 * 1024],*opt_password=0,*current_user=0,
              *current_host=0,*path=0,*fields_terminated=0,
@@ -131,7 +131,7 @@ static struct my_option my_long_options[] =
    (gptr*) &opt_delayed, (gptr*) &opt_delayed, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0,
    0, 0},
   {"delete-master-logs", OPT_DELETE_MASTER_LOGS,
-   "Delete logs on master after backup. This will automagically enable --first-slave.",
+   "Delete logs on master after backup. This automatically enables --first-slave.",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"disable-keys", 'K',
    "'/*!40000 ALTER TABLE tb_name DISABLE KEYS */; and '/*!40000 ALTER TABLE tb_name ENABLE KEYS */; will be put in the output.", (gptr*) &opt_disable_keys,
@@ -154,7 +154,10 @@ static struct my_option my_long_options[] =
   {"first-slave", 'x', "Locks all tables across all databases.",
    (gptr*) &opt_first_slave, (gptr*) &opt_first_slave, 0, GET_BOOL, NO_ARG,
    0, 0, 0, 0, 0, 0},
-  {"flush-logs", 'F', "Flush logs file in server before starting dump.",
+  {"flush-logs", 'F', "Flush logs file in server before starting dump. "
+    "Note that if you dump many databases at once (using the option "
+    "--databases= or --all-databases), the logs will be flushed for "
+    "each database dumped.",
    (gptr*) &flush_logs, (gptr*) &flush_logs, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0,
    0, 0},
   {"force", 'f', "Continue even if we get an sql-error.",
@@ -170,7 +173,7 @@ static struct my_option my_long_options[] =
   {"lock-tables", 'l', "Lock all tables for read.", (gptr*) &lock_tables,
    (gptr*) &lock_tables, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"master-data", OPT_MASTER_DATA,
-   "This will cause the master position and filename to be appended to your output. This will automagically enable --first-slave.",
+   "This causes the master position and filename to be appended to your output. This automatically enables --first-slave.",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"no-autocommit", OPT_AUTOCOMMIT,
    "Wrap tables with autocommit/commit statements.",
@@ -179,7 +182,7 @@ static struct my_option my_long_options[] =
   {"single-transaction", OPT_TRANSACTION,
    "Dump all tables in single transaction to get consistent snapshot. Mutually exclusive with --lock-tables.",
    (gptr*) &opt_single_transaction, (gptr*) &opt_single_transaction, 0, GET_BOOL, NO_ARG,
-   0, 0, 0, 0, 0, 0},   
+   0, 0, 0, 0, 0, 0},
   {"no-create-db", 'n',
    "'CREATE DATABASE /*!32312 IF NOT EXISTS*/ db_name;' will not be put in the output. The above line will be added otherwise, if --databases or --all-databases option was given.}",
    (gptr*) &opt_create_db, (gptr*) &opt_create_db, 0, GET_BOOL, NO_ARG, 0, 0,
@@ -236,12 +239,15 @@ static struct my_option my_long_options[] =
    NO_ARG, 0, 0, 0, 0, 0, 0},
   {"max_allowed_packet", OPT_MAX_ALLOWED_PACKET, "",
     (gptr*) &max_allowed_packet, (gptr*) &max_allowed_packet, 0,
-    GET_ULONG, REQUIRED_ARG, 24*1024*1024, 4096, 1024*1024L*1024L,
-    MALLOC_OVERHEAD, 1024, 0},
+    GET_ULONG, REQUIRED_ARG, 24*1024*1024, 4096, 
+   (longlong) 2L*1024L*1024L*1024L, MALLOC_OVERHEAD, 1024, 0},
   {"net_buffer_length", OPT_NET_BUFFER_LENGTH, "",
     (gptr*) &net_buffer_length, (gptr*) &net_buffer_length, 0,
     GET_ULONG, REQUIRED_ARG, 1024*1024L-1025, 4096, 16*1024L*1024L,
     MALLOC_OVERHEAD-1024, 1024, 0},
+  {"comments", 'i', "Write additional information.",
+   (gptr*) &opt_comments, (gptr*) &opt_comments, 0, GET_BOOL, NO_ARG,
+   1, 0, 0, 0, 0, 0},
   {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
@@ -257,8 +263,10 @@ static int dump_all_tables_in_db(char *db);
 static int init_dumping(char *);
 static int dump_databases(char **);
 static int dump_all_databases();
-static char *quote_name(char *name, char *buff);
+static char *quote_name(const char *name, char *buff, my_bool force);
 static void print_quoted_xml(FILE *output, char *fname, char *str, uint len);
+static const char *check_if_ignore_table(const char *table_name);
+
 
 static void print_version(void)
 {
@@ -302,12 +310,12 @@ static void write_header(FILE *sql_file, char *db_name)
     fprintf(sql_file,"<?xml version=\"1.0\"?>\n");
     fprintf(sql_file,"<mysqldump>\n");
   }
-  else
+  else if (opt_comments)
   {
     fprintf(sql_file, "-- MySQL dump %s\n--\n", DUMP_VERSION);
     fprintf(sql_file, "-- Host: %s    Database: %s\n",
 	    current_host ? current_host : "localhost", db_name ? db_name : "");
-    fputs("---------------------------------------------------------\n",
+    fputs("-- ------------------------------------------------------\n",
 	  sql_file);
     fprintf(sql_file, "-- Server version\t%s\n",
 	    mysql_get_server_info(&mysql_connection));
@@ -542,33 +550,60 @@ static my_bool test_if_special_chars(const char *str)
   return 0;
 } /* test_if_special_chars */
 
-static char *quote_name(char *name, char *buff)
+
+static char *quote_name(const char *name, char *buff, my_bool force)
 {
-  char *end;
-  if (!opt_quoted && !test_if_special_chars(name))
-    return name;
-  buff[0]=QUOTE_CHAR;
-  end=strmov(buff+1,name);
-  end[0]=QUOTE_CHAR;
-  end[1]=0;
+  char *to= buff;
+  if (!force && !opt_quoted && !test_if_special_chars(name))
+    return (char*) name;
+  *to++= QUOTE_CHAR;
+  while (*name)
+  {
+    if (*name == QUOTE_CHAR)
+      *to= QUOTE_CHAR;
+    *to++= *name++;
+  }
+  to[0]=QUOTE_CHAR;
+  to[1]=0;
   return buff;
 } /* quote_name */
 
 
+
+static char *quote_for_like(const char *name, char *buff)
+{
+  char *to= buff;
+  *to++= '\'';
+  while (*name)
+  {
+    if (*name == '\'' || *name == '_' || *name == '\\' || *name == '%')
+      *to++= '\\';
+    *to++= *name++;
+  }
+  to[0]= '\'';
+  to[1]= 0;
+  return buff;
+}
+
+
 /*
-** getStructure -- retrievs database structure, prints out corresponding
-**       CREATE statement and fills out insert_pat.
-** Return values:  number of fields in table, 0 if error
+  getStructure -- retrievs database structure, prints out corresponding
+  CREATE statement and fills out insert_pat.
+
+  RETURN
+    number of fields in table, 0 if error
 */
+
 static uint getTableStructure(char *table, char* db)
 {
   MYSQL_RES  *tableRes;
   MYSQL_ROW  row;
   my_bool    init=0;
   uint       numFields;
-  char	     *strpos, *table_name;
+  char	     *strpos, *result_table, *opt_quoted_table;
   const char *delayed;
-  char	     name_buff[NAME_LEN+3],table_buff[NAME_LEN+3];
+  char	     name_buff[NAME_LEN+3],table_buff[NAME_LEN*2+3];
+  char	     table_buff2[NAME_LEN*2+3];
   FILE       *sql_file = md_result_file;
   DBUG_ENTER("getTableStructure");
 
@@ -577,8 +612,10 @@ static uint getTableStructure(char *table, char* db)
   if (verbose)
     fprintf(stderr, "-- Retrieving table structure for table %s...\n", table);
 
-  sprintf(insert_pat,"SET OPTION SQL_QUOTE_SHOW_CREATE=%d", (opt_quoted || opt_keywords));
-  table_name=quote_name(table,table_buff);
+  sprintf(insert_pat,"SET OPTION SQL_QUOTE_SHOW_CREATE=%d",
+	  (opt_quoted || opt_keywords));
+  result_table=     quote_name(table, table_buff, 1);
+  opt_quoted_table= quote_name(table, table_buff2, 0);
   if (!mysql_query(sock,insert_pat))
   {
     /* using SHOW CREATE statement */
@@ -587,11 +624,11 @@ static uint getTableStructure(char *table, char* db)
       /* Make an sql-file, if path was given iow. option -T was given */
       char buff[20+FN_REFLEN];
 
-      sprintf(buff,"show create table `%s`",table);
+      sprintf(buff,"show create table %s", result_table);
       if (mysql_query(sock, buff))
       {
-        fprintf(stderr, "%s: Can't get CREATE TABLE for table '%s' (%s)\n",
-		      my_progname, table, mysql_error(sock));
+        fprintf(stderr, "%s: Can't get CREATE TABLE for table %s (%s)\n",
+		      my_progname, result_table, mysql_error(sock));
         safe_exit(EX_MYSQLERR);
         DBUG_RETURN(0);
       }
@@ -609,11 +646,11 @@ static uint getTableStructure(char *table, char* db)
         }
         write_header(sql_file, db);
       }
-      if (!opt_xml)
-	fprintf(sql_file, "\n--\n-- Table structure for table '%s'\n--\n\n",
-		table);
+      if (!opt_xml && opt_comments)
+	fprintf(sql_file, "\n--\n-- Table structure for table %s\n--\n\n",
+		result_table);
       if (opt_drop)
-        fprintf(sql_file, "DROP TABLE IF EXISTS %s;\n",table_name);
+        fprintf(sql_file, "DROP TABLE IF EXISTS %s;\n", opt_quoted_table);
 
       tableRes=mysql_store_result(sock);
       row=mysql_fetch_row(tableRes);
@@ -621,11 +658,11 @@ static uint getTableStructure(char *table, char* db)
 	fprintf(sql_file, "%s;\n", row[1]);
       mysql_free_result(tableRes);
     }
-    sprintf(insert_pat,"show fields from %s",table_name);
+    sprintf(insert_pat,"show fields from %s", result_table);
     if (mysql_query(sock,insert_pat) || !(tableRes=mysql_store_result(sock)))
     {
-      fprintf(stderr, "%s: Can't get info about table: '%s'\nerror: %s\n",
-		    my_progname, table, mysql_error(sock));
+      fprintf(stderr, "%s: Can't get info about table: %s\nerror: %s\n",
+	      my_progname, result_table, mysql_error(sock));
       if (path)
 	my_fclose(sql_file, MYF(MY_WME));
       safe_exit(EX_MYSQLERR);
@@ -633,10 +670,11 @@ static uint getTableStructure(char *table, char* db)
     }
 
     if (cFlag)
-      sprintf(insert_pat, "INSERT %sINTO %s (", delayed, table_name);
+      sprintf(insert_pat, "INSERT %sINTO %s (", delayed, opt_quoted_table);
     else
     {
-      sprintf(insert_pat, "INSERT %sINTO %s VALUES ", delayed, table_name);
+      sprintf(insert_pat, "INSERT %sINTO %s VALUES ", delayed,
+	      opt_quoted_table);
       if (!extended_insert)
         strcat(insert_pat,"(");
     }
@@ -651,7 +689,7 @@ static uint getTableStructure(char *table, char* db)
       }
       init=1;
       if (cFlag)
-        strpos=strmov(strpos,quote_name(row[SHOW_FIELDNAME],name_buff));
+        strpos=strmov(strpos,quote_name(row[SHOW_FIELDNAME], name_buff, 0));
     }
     numFields = (uint) mysql_num_rows(tableRes);
     mysql_free_result(tableRes);
@@ -661,11 +699,11 @@ static uint getTableStructure(char *table, char* db)
   /*  fprintf(stderr, "%s: Can't set SQL_QUOTE_SHOW_CREATE option (%s)\n",
       my_progname, mysql_error(sock)); */
 
-    sprintf(insert_pat,"show fields from %s",table_name);
+    sprintf(insert_pat,"show fields from %s", result_table);
     if (mysql_query(sock,insert_pat) || !(tableRes=mysql_store_result(sock)))
     {
-      fprintf(stderr, "%s: Can't get info about table: '%s'\nerror: %s\n",
-		    my_progname, table, mysql_error(sock));
+      fprintf(stderr, "%s: Can't get info about table: %s\nerror: %s\n",
+		    my_progname, result_table, mysql_error(sock));
       safe_exit(EX_MYSQLERR);
       DBUG_RETURN(0);
     }
@@ -686,18 +724,18 @@ static uint getTableStructure(char *table, char* db)
         }
         write_header(sql_file, db);
       }
-      if (!opt_xml)
-	fprintf(sql_file, "\n--\n-- Table structure for table '%s'\n--\n\n",
-		table);
+      if (!opt_xml && opt_comments)
+	fprintf(sql_file, "\n--\n-- Table structure for table %s\n--\n\n",
+		result_table);
       if (opt_drop)
-        fprintf(sql_file, "DROP TABLE IF EXISTS %s;\n",table_name);
-      fprintf(sql_file, "CREATE TABLE %s (\n", table_name);
+        fprintf(sql_file, "DROP TABLE IF EXISTS %s;\n",result_table);
+      fprintf(sql_file, "CREATE TABLE %s (\n", result_table);
     }
     if (cFlag)
-      sprintf(insert_pat, "INSERT %sINTO %s (", delayed, table_name);
+      sprintf(insert_pat, "INSERT %sINTO %s (", delayed, result_table);
     else
     {
-      sprintf(insert_pat, "INSERT %sINTO %s VALUES ", delayed, table_name);
+      sprintf(insert_pat, "INSERT %sINTO %s VALUES ", delayed, result_table);
       if (!extended_insert)
         strcat(insert_pat,"(");
     }
@@ -715,15 +753,17 @@ static uint getTableStructure(char *table, char* db)
       }
       init=1;
       if (cFlag)
-        strpos=strmov(strpos,quote_name(row[SHOW_FIELDNAME],name_buff));
+        strpos=strmov(strpos,quote_name(row[SHOW_FIELDNAME], name_buff, 0));
       if (!tFlag)
       {
         if (opt_keywords)
-	  fprintf(sql_file, "  %s.%s %s", table_name,
-	  quote_name(row[SHOW_FIELDNAME],name_buff), row[SHOW_TYPE]);
+	  fprintf(sql_file, "  %s.%s %s", result_table,
+		  quote_name(row[SHOW_FIELDNAME],name_buff, 0),
+		  row[SHOW_TYPE]);
         else
 	  fprintf(sql_file, "  %s %s", quote_name(row[SHOW_FIELDNAME],
-						  name_buff), row[SHOW_TYPE]);
+						  name_buff, 0),
+		  row[SHOW_TYPE]);
         if (row[SHOW_DEFAULT])
         {
 	  fputs(" DEFAULT ", sql_file);
@@ -742,11 +782,11 @@ static uint getTableStructure(char *table, char* db)
       /* Make an sql-file, if path was given iow. option -T was given */
       char buff[20+FN_REFLEN];
       uint keynr,primary_key;
-      sprintf(buff,"show keys from %s",table_name);
+      sprintf(buff,"show keys from %s", result_table);
       if (mysql_query(sock, buff))
       {
-        fprintf(stderr, "%s: Can't get keys for table '%s' (%s)\n",
-		my_progname, table, mysql_error(sock));
+        fprintf(stderr, "%s: Can't get keys for table %s (%s)\n",
+		my_progname, result_table, mysql_error(sock));
         if (path)
 	  my_fclose(sql_file, MYF(MY_WME));
         safe_exit(EX_MYSQLERR);
@@ -783,15 +823,15 @@ static uint getTableStructure(char *table, char* db)
 	    putc(')', sql_file);
 	  if (atoi(row[1]))       /* Test if duplicate key */
 	    /* Duplicate allowed */
-	    fprintf(sql_file, ",\n  KEY %s (",quote_name(row[2],name_buff));
+	    fprintf(sql_file, ",\n  KEY %s (",quote_name(row[2],name_buff,0));
 	  else if (keynr == primary_key)
 	    fputs(",\n  PRIMARY KEY (",sql_file); /* First UNIQUE is primary */
 	  else
-	    fprintf(sql_file, ",\n  UNIQUE %s (",quote_name(row[2],name_buff));
+	    fprintf(sql_file, ",\n  UNIQUE %s (",quote_name(row[2],name_buff,0));
         }
         else
 	  putc(',', sql_file);
-        fputs(quote_name(row[4],name_buff), sql_file);
+        fputs(quote_name(row[4], name_buff, 0), sql_file);
         if (row[7])
 	  fprintf(sql_file, " (%s)",row[7]);      /* Sub key */
       }
@@ -802,23 +842,25 @@ static uint getTableStructure(char *table, char* db)
       /* Get MySQL specific create options */
       if (create_options)
       {
-        sprintf(buff,"show table status like '%s'",table);
+	char show_name_buff[FN_REFLEN];
+        sprintf(buff,"show table status like %s",
+		quote_for_like(table, show_name_buff));
         if (mysql_query(sock, buff))
         {
 	  if (mysql_errno(sock) != ER_PARSE_ERROR)
 	  {					/* If old MySQL version */
 	    if (verbose)
 	      fprintf(stderr,
-		      "-- Warning: Couldn't get status information for table '%s' (%s)\n",
-		      table,mysql_error(sock));
+		      "-- Warning: Couldn't get status information for table %s (%s)\n",
+		      result_table,mysql_error(sock));
 	  }
         }
         else if (!(tableRes=mysql_store_result(sock)) ||
 		 !(row=mysql_fetch_row(tableRes)))
         {
 	  fprintf(stderr,
-		  "Error: Couldn't read status information for table '%s' (%s)\n",
-		  table,mysql_error(sock));
+		  "Error: Couldn't read status information for table %s (%s)\n",
+		  result_table,mysql_error(sock));
         }
         else
         {
@@ -896,16 +938,45 @@ static char *field_escape(char *to,const char *from,uint length)
 } /* field_escape */
 
 
+static char *alloc_query_str(ulong size)
+{
+  char *query;
+
+  if (!(query= (char*) my_malloc(size, MYF(MY_WME))))
+  {
+    ignore_errors= 0;   			/* Fatal error */
+    safe_exit(EX_MYSQLERR);			/* Force exit */
+  }
+  return query;
+}
+
 /*
 ** dumpTable saves database contents as a series of INSERT statements.
 */
 static void dumpTable(uint numFields, char *table)
 {
-  char query[QUERY_LENGTH], *end, buff[256],table_buff[NAME_LEN+3];
+  char query_buf[QUERY_LENGTH], *end, buff[256],table_buff[NAME_LEN+3];
+  char *result_table, table_buff2[NAME_LEN*2+3], *opt_quoted_table;
+  char *query= query_buf;
   MYSQL_RES	*res;
   MYSQL_FIELD	*field;
   MYSQL_ROW	row;
   ulong		rownr, row_break, total_length, init_length;
+  const char    *table_type;
+  int error= 0;
+
+  result_table= quote_name(table,table_buff, 1);
+  opt_quoted_table= quote_name(table, table_buff2, 0);
+
+  /* Check table type */
+  if ((table_type= check_if_ignore_table(table)))
+  {
+    if (verbose)
+      fprintf(stderr,
+	      "-- Skipping data for table '%s' because it's of type %s\n",
+	      table, table_type);
+    return;
+  }
 
   if (verbose)
     fprintf(stderr, "-- Sending SELECT query...\n");
@@ -935,11 +1006,14 @@ static void dumpTable(uint numFields, char *table)
     end= add_load_option(end, lines_terminated, " LINES TERMINATED BY");
     *end= '\0';
 
-    sprintf(buff," FROM %s",quote_name(table,table_buff));
+    sprintf(buff," FROM %s", result_table);
     end= strmov(end,buff);
     if (where)
-      end= strxmov(end, " WHERE ",where,NullS);
-    if (mysql_query(sock, query))
+    {
+      query= alloc_query_str((ulong) (strlen(where) + (end - query) + 10));
+      end= strxmov(query, query_buf, " WHERE ", where, NullS);
+    }
+    if (mysql_real_query(sock, query, (uint) (end - query)))
     {
       DBerror(sock, "when executing 'SELECT INTO OUTFILE'");
       return;
@@ -947,23 +1021,25 @@ static void dumpTable(uint numFields, char *table)
   }
   else
   {
-    if (!opt_xml)
-      fprintf(md_result_file,"\n--\n-- Dumping data for table '%s'\n--\n",
-	      table);
+    if (!opt_xml && opt_comments)
+      fprintf(md_result_file,"\n--\n-- Dumping data for table %s\n--\n",
+	      result_table);
     sprintf(query, "SELECT /*!40001 SQL_NO_CACHE */ * FROM %s",
-	    quote_name(table,table_buff));
+	    result_table);
     if (where)
     {
-      if (!opt_xml)
+      if (!opt_xml && opt_comments)
 	fprintf(md_result_file,"-- WHERE:  %s\n",where);
-      strxmov(strend(query), " WHERE ",where,NullS);
+      query= alloc_query_str((ulong) (strlen(where) + strlen(query) + 10));
+      strxmov(query, query_buf, " WHERE ", where, NullS);
     }
     if (!opt_xml)
       fputs("\n", md_result_file);
     if (mysql_query(sock, query))
     {
       DBerror(sock, "when retrieving data from server");
-      return;
+      error= EX_CONSCHECK;
+      goto err;
     }
     if (quick)
       res=mysql_use_result(sock);
@@ -972,24 +1048,24 @@ static void dumpTable(uint numFields, char *table)
     if (!res)
     {
       DBerror(sock, "when retrieving data from server");
-      return;
+      error= EX_CONSCHECK;
+      goto err;
     }
     if (verbose)
       fprintf(stderr, "-- Retrieving rows...\n");
     if (mysql_num_fields(res) != numFields)
     {
-      fprintf(stderr,"%s: Error in field count for table: '%s' !  Aborting.\n",
-	      my_progname,table);
-      safe_exit(EX_CONSCHECK);
-      return;
+      fprintf(stderr,"%s: Error in field count for table: %s !  Aborting.\n",
+	      my_progname, result_table);
+      error= EX_CONSCHECK;
+      goto err;
     }
 
     if (opt_disable_keys)
-      fprintf(md_result_file,"/*!40000 ALTER TABLE %s DISABLE KEYS */;\n",
-	      quote_name(table, table_buff));
+      fprintf(md_result_file, "\n/*!40000 ALTER TABLE %s DISABLE KEYS */;\n",
+	      opt_quoted_table);
     if (opt_lock)
-      fprintf(md_result_file,"LOCK TABLES %s WRITE;\n",
-	      quote_name(table,table_buff));
+      fprintf(md_result_file,"LOCK TABLES %s WRITE;\n", opt_quoted_table);
 
     total_length=net_buffer_length;		/* Force row break */
     row_break=0;
@@ -1017,11 +1093,11 @@ static void dumpTable(uint numFields, char *table)
       {
 	if (!(field = mysql_fetch_field(res)))
 	{
-	  sprintf(query,"%s: Not enough fields from table '%s'! Aborting.\n",
-		  my_progname,table);
+	  sprintf(query,"%s: Not enough fields from table %s! Aborting.\n",
+		  my_progname, result_table);
 	  fputs(query,stderr);
-	  safe_exit(EX_CONSCHECK);
-	  return;
+	  error= EX_CONSCHECK;
+	  goto err;
 	}
 	if (extended_insert)
 	{
@@ -1040,7 +1116,8 @@ static void dumpTable(uint numFields, char *table)
 		if (dynstr_realloc(&extended_row,length * 2+2))
 		{
 		  fputs("Aborting dump (out of memory)",stderr);
-		  safe_exit(EX_EOM);
+		  error= EX_EOM;
+		  goto err;
 		}
 		dynstr_append(&extended_row,"\'");
 		extended_row.length +=
@@ -1051,10 +1128,22 @@ static void dumpTable(uint numFields, char *table)
 	      }
 	      else
 	      {
-		/* change any strings ("inf","nan",..) into NULL */
+		/* change any strings ("inf", "-inf", "nan") into NULL */
 		char *ptr = row[i];
-		dynstr_append(&extended_row,
-			      (!isalpha(*ptr)) ? ptr : "NULL");
+		if (isalpha(*ptr) || (*ptr == '-' && isalpha(ptr[1])))
+		  dynstr_append(&extended_row, "NULL");
+		else
+		{
+		  if (field->type == FIELD_TYPE_DECIMAL)
+		  {
+		    /* add " signs around */
+		    dynstr_append(&extended_row, "\'");
+		    dynstr_append(&extended_row, ptr);
+		    dynstr_append(&extended_row, "\'");
+		  }
+		  else
+		    dynstr_append(&extended_row, ptr);
+		}
 	      }
 	    }
 	    else
@@ -1063,7 +1152,8 @@ static void dumpTable(uint numFields, char *table)
 	  else if (dynstr_append(&extended_row,"NULL"))
 	  {
 	    fputs("Aborting dump (out of memory)",stderr);
-	    safe_exit(EX_EOM);
+	    error= EX_EOM;
+	    goto err;
 	  }
 	}
 	else
@@ -1082,13 +1172,25 @@ static void dumpTable(uint numFields, char *table)
 	    }
 	    else
 	    {
-	      /* change any strings ("inf","nan",..) into NULL */
+	      /* change any strings ("inf", "-inf", "nan") into NULL */
 	      char *ptr = row[i];
 	      if (opt_xml)
 		fprintf(md_result_file, "\t\t<field name=\"%s\">%s</field>\n",
 			field->name,!isalpha(*ptr) ?ptr: "NULL");
+	      else if (isalpha(*ptr) || (*ptr == '-' && isalpha(ptr[1])))
+	        fputs("NULL", md_result_file);
 	      else
-		fputs((!isalpha(*ptr)) ? ptr : "NULL", md_result_file);
+	      {
+		if (field->type == FIELD_TYPE_DECIMAL)
+		{
+		  /* add " signs around */
+		  fputs("\'", md_result_file);
+		  fputs(ptr, md_result_file);
+		  fputs("\'", md_result_file);
+		}
+		else
+		  fputs(ptr, md_result_file);
+	      }
 	    }
 	  }
 	  else
@@ -1142,25 +1244,34 @@ static void dumpTable(uint numFields, char *table)
     fflush(md_result_file);
     if (mysql_errno(sock))
     {
-      sprintf(query,"%s: Error %d: %s when dumping table '%s' at row: %ld\n",
+      sprintf(query,"%s: Error %d: %s when dumping table %s at row: %ld\n",
 	      my_progname,
 	      mysql_errno(sock),
 	      mysql_error(sock),
-	      table,
+	      result_table,
 	      rownr);
       fputs(query,stderr);
-      safe_exit(EX_CONSCHECK);
-      return;
+      error= EX_CONSCHECK;
+      goto err;
     }
     if (opt_lock)
       fputs("UNLOCK TABLES;\n", md_result_file);
     if (opt_disable_keys)
       fprintf(md_result_file,"/*!40000 ALTER TABLE %s ENABLE KEYS */;\n",
-	      quote_name(table,table_buff));
+	      opt_quoted_table);
     if (opt_autocommit)
       fprintf(md_result_file, "commit;\n");
     mysql_free_result(res);
-  }
+    if (query != query_buf)
+      my_free(query, MYF(MY_ALLOW_ZERO_PTR));
+  } 
+  return;
+
+err:
+  if (query != query_buf)
+    my_free(query, MYF(MY_ALLOW_ZERO_PTR));
+  safe_exit(error);
+  return;
 } /* dumpTable */
 
 
@@ -1255,10 +1366,14 @@ static int init_dumping(char *database)
   {
     if (opt_databases || opt_alldbs)
     {
-      fprintf(md_result_file,"\n--\n-- Current Database: %s\n--\n", database);
+      /* length of table name * 2 (if name contain quotas), 2 quotas and 0 */
+      char quoted_database_buf[64*2+3];
+      char *qdatabase= quote_name(database,quoted_database_buf,opt_quoted);
+      if (opt_comments)
+	fprintf(md_result_file,"\n--\n-- Current Database: %s\n--\n", database);
       if (!opt_create_db)
 	fprintf(md_result_file,"\nCREATE DATABASE /*!32312 IF NOT EXISTS*/ %s;\n",
-		database);
+		qdatabase);
       fprintf(md_result_file,"\nUSE %s;\n", database);
     }
   }
@@ -1273,7 +1388,7 @@ static int dump_all_tables_in_db(char *database)
 {
   char *table;
   uint numrows;
-  char table_buff[NAME_LEN+3];
+  char table_buff[NAME_LEN*2+3];
 
   if (init_dumping(database))
     return 1;
@@ -1285,7 +1400,7 @@ static int dump_all_tables_in_db(char *database)
     init_dynamic_string(&query, "LOCK TABLES ", 256, 1024);
     for (numrows=0 ; (table = getTableName(1)) ; numrows++)
     {
-      dynstr_append(&query, quote_name(table, table_buff));
+      dynstr_append(&query, quote_name(table, table_buff, 1));
       dynstr_append(&query, " READ /*!32311 LOCAL */,");
     }
     if (numrows && mysql_real_query(sock, query.str, query.length-1))
@@ -1308,7 +1423,7 @@ static int dump_all_tables_in_db(char *database)
   if (opt_xml)
     fprintf(md_result_file, "</database>\n");
   if (lock_tables)
-    mysql_query(sock,"UNLOCK_TABLES");
+    mysql_query(sock,"UNLOCK TABLES");
   return 0;
 } /* dump_all_tables_in_db */
 
@@ -1317,7 +1432,7 @@ static int dump_all_tables_in_db(char *database)
 static int dump_selected_tables(char *db, char **table_names, int tables)
 {
   uint numrows;
-  char table_buff[NAME_LEN+3];
+  char table_buff[NAME_LEN*+3];
 
   if (init_dumping(db))
     return 1;
@@ -1329,7 +1444,7 @@ static int dump_selected_tables(char *db, char **table_names, int tables)
     init_dynamic_string(&query, "LOCK TABLES ", 256, 1024);
     for (i=0 ; i < tables ; i++)
     {
-      dynstr_append(&query, quote_name(table_names[i], table_buff));
+      dynstr_append(&query, quote_name(table_names[i], table_buff, 1));
       dynstr_append(&query, " READ /*!32311 LOCAL */,");
     }
     if (mysql_real_query(sock, query.str, query.length-1))
@@ -1354,7 +1469,7 @@ static int dump_selected_tables(char *db, char **table_names, int tables)
   if (opt_xml)
     fprintf(md_result_file, "</database>\n");
   if (lock_tables)
-    mysql_query(sock,"UNLOCK_TABLES");
+    mysql_query(sock,"UNLOCK TABLES");
   return 0;
 } /* dump_selected_tables */
 
@@ -1387,6 +1502,61 @@ static void print_value(FILE *file, MYSQL_RES  *result, MYSQL_ROW row,
 } /* print_value */
 
 
+/*
+  Check if we the table is one of the table types that should be ignored:
+  MRG_ISAM, MRG_MYISAM
+
+  SYNOPSIS
+    check_if_ignore_table()
+    table_name			Table name to check
+
+  GLOBAL VARIABLES
+    sock			MySQL socket
+    verbose			Write warning messages
+
+  RETURN
+    0	Table should be backuped
+    #	Type of table (that should be skipped)
+*/
+
+static const char *check_if_ignore_table(const char *table_name)
+{
+  char buff[FN_REFLEN+80], show_name_buff[FN_REFLEN];
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+  const char *result= 0;
+
+  sprintf(buff,"show table status like %s",
+	  quote_for_like(table_name, show_name_buff));
+  if (mysql_query(sock, buff))
+  {
+    if (mysql_errno(sock) != ER_PARSE_ERROR)
+    {					/* If old MySQL version */
+      if (verbose)
+	fprintf(stderr,
+		"-- Warning: Couldn't get status information for table %s (%s)\n",
+		table_name,mysql_error(sock));
+      return 0;					/* assume table is ok */
+    }
+  }
+  if (!(res= mysql_store_result(sock)) ||
+      !(row= mysql_fetch_row(res)))
+  {
+    fprintf(stderr,
+	    "Error: Couldn't read status information for table %s (%s)\n",
+	    table_name, mysql_error(sock));
+    if (res)
+      mysql_free_result(res);
+    return 0;					/* assume table is ok */
+  }
+  if (strcmp(row[1], (result= "MRG_MyISAM")) &&
+      strcmp(row[1], (result= "MRG_ISAM")))
+    result= 0;
+  mysql_free_result(res);  
+  return result;
+}
+
+
 int main(int argc, char **argv)
 {
   MYSQL_ROW row;
@@ -1456,8 +1626,9 @@ int main(int argc, char **argv)
 	row = mysql_fetch_row(master);
 	if (row && row[0] && row[1])
 	{
-	  fprintf(md_result_file,
-		  "\n--\n-- Position to start replication from\n--\n\n");
+	  if (opt_comments)
+	    fprintf(md_result_file,
+		    "\n--\n-- Position to start replication from\n--\n\n");
 	  fprintf(md_result_file,
 		  "CHANGE MASTER TO MASTER_LOG_FILE='%s', \
 MASTER_LOG_POS=%s ;\n",row[0],row[1]); 
