@@ -85,6 +85,7 @@
 
 #define LONGLONG_MIN	((__int64) 0x8000000000000000)
 #define LONGLONG_MAX	((__int64) 0x7FFFFFFFFFFFFFFF)
+#define ULONGLONG_MAX	((unsigned __int64) 0xFFFFFFFFFFFFFFFF)
 #define LL(A)		((__int64) A)
 
 /* Type information */
@@ -150,6 +151,10 @@ typedef uint rf_SetTimer;
 #define USE_MB_IDENT 1
 #define USE_STRCOLL 1
 
+/* All windows servers should support .sym files */
+#undef USE_SYMDIR
+#define USE_SYMDIR
+
 /* If LOAD DATA LOCAL INFILE should be enabled by default */
 #define ENABLED_LOCAL_INFILE 1
 
@@ -173,8 +178,8 @@ inline double rint(double nr)
 }
 
 #ifdef _WIN64
-#define ulonglong2double(A) ((double) (A))
-#define my_off_t2double(A)  ((double) (A))
+#define ulonglong2double(A) ((double) (ulonglong) (A))
+#define my_off_t2double(A)  ((double) (my_off_t) (A))
 
 #else
 inline double ulonglong2double(ulonglong value)
@@ -311,7 +316,7 @@ inline double ulonglong2double(ulonglong value)
 #define FN_ROOTDIR	"\\"
 #define FN_NETWORK_DRIVES	/* Uses \\ to indicate network drives */
 #define FN_NO_CASE_SENCE	/* Files are not case-sensitive */
-#define MY_NFILE	1024
+#define MY_NFILE	2048
 
 #define DO_NOT_REMOVE_THREAD_WRAPPERS
 #define thread_safe_increment(V,L) InterlockedIncrement((long*) &(V))
