@@ -125,11 +125,13 @@ buf_LRU_search_and_free_block(
 
 		if (buf_flush_ready_for_replace(block)) {
 
+#ifdef UNIV_DEBUG
 			if (buf_debug_prints) {
-				printf(
+				fprintf(stderr,
 				"Putting space %lu page %lu to free list\n",
 					block->space, block->offset);
 			}
+#endif /* UNIV_DEBUG */
 			
 			buf_LRU_block_remove_hashed_page(block);
 
@@ -216,7 +218,7 @@ buf_LRU_get_free_block(void)
 	buf_block_t*	block		= NULL;
 	ibool		freed;
 	ulint		n_iterations	= 1;
-	ibool		mon_value_was   = 0; /* remove bug */
+	ibool		mon_value_was   = FALSE;
 	ibool		started_monitor	= FALSE;
 loop:
 	mutex_enter(&(buf_pool->mutex));
@@ -227,20 +229,26 @@ loop:
 
 	   	fprintf(stderr,
 "  InnoDB: ERROR: over 9 / 10 of the buffer pool is occupied by\n"
-"InnoDB: lock heaps or the adaptive hash index!\n"
+"InnoDB: lock heaps or the adaptive hash index! Check that your\n"
+"InnoDB: transactions do not set too many row locks.\n"
+"InnoDB: Your buffer pool size is %lu MB. Maybe you should make\n"
+"InnoDB: the buffer pool bigger?\n"
 "InnoDB: We intentionally generate a seg fault to print a stack trace\n"
-"InnoDB: on Linux!\n");
+"InnoDB: on Linux!\n",
+		(ulong)(buf_pool->curr_size / (1024 * 1024 / UNIV_PAGE_SIZE)));
 
-		ut_a(0);
+		ut_error;
 	   
 	} else if (!recv_recovery_on && UT_LIST_GET_LEN(buf_pool->free)
 	   + UT_LIST_GET_LEN(buf_pool->LRU) < buf_pool->max_size / 5) {
+		if (!srv_print_innodb_monitor) {
 
-	   	/* Over 80 % of the buffer pool is occupied by lock heaps
-	   	or the adaptive hash index. This may be a memory leak! */
+	   		/* Over 80 % of the buffer pool is occupied by lock
+			heaps or the adaptive hash index. This may be a memory
+			leak! */
 
-	   	ut_print_timestamp(stderr);
-	   	fprintf(stderr,
+	   		ut_print_timestamp(stderr);
+	   		fprintf(stderr,
 "  InnoDB: WARNING: over 4 / 5 of the buffer pool is occupied by\n"
 "InnoDB: lock heaps or the adaptive hash index! Check that your\n"
 "InnoDB: transactions do not set too many row locks.\n"
@@ -248,10 +256,11 @@ loop:
 "InnoDB: the buffer pool bigger?\n"
 "InnoDB: Starting the InnoDB Monitor to print diagnostics, including\n"
 "InnoDB: lock heap and hash index sizes.\n",
-		buf_pool->curr_size / (1024 * 1024 / UNIV_PAGE_SIZE));
-
-		srv_print_innodb_monitor = TRUE;
+		(ulong)(buf_pool->curr_size / (1024 * 1024 / UNIV_PAGE_SIZE)));
 
+			srv_print_innodb_monitor = TRUE;
+			os_event_set(srv_lock_timeout_thread_event);
+		}
 	} else if (!recv_recovery_on && UT_LIST_GET_LEN(buf_pool->free)
 	   + UT_LIST_GET_LEN(buf_pool->LRU) < buf_pool->max_size / 4) {
 
@@ -294,30 +303,25 @@ loop:
 		fprintf(stderr,
 		"InnoDB: Warning: difficult to find free blocks from\n"
 		"InnoDB: the buffer pool (%lu search iterations)! Consider\n"
-		"InnoDB: increasing the buffer pool size.\n",
-						n_iterations);
-		fprintf(stderr,
+		"InnoDB: increasing the buffer pool size.\n"
 		"InnoDB: It is also possible that in your Unix version\n"
 		"InnoDB: fsync is very slow, or completely frozen inside\n"
 		"InnoDB: the OS kernel. Then upgrading to a newer version\n"
 		"InnoDB: of your operating system may help. Look at the\n"
-		"InnoDB: number of fsyncs in diagnostic info below.\n");
-
-		fprintf(stderr,
-		"InnoDB: Pending flushes (fsync) log: %lu; buffer pool: %lu\n",
-	       			fil_n_pending_log_flushes,
-				fil_n_pending_tablespace_flushes);
-		fprintf(stderr,
-	"InnoDB: %lu OS file reads, %lu OS file writes, %lu OS fsyncs\n",
-			os_n_file_reads, os_n_file_writes, os_n_fsyncs);
-
-		fprintf(stderr,
+		"InnoDB: number of fsyncs in diagnostic info below.\n"
+		"InnoDB: Pending flushes (fsync) log: %lu; buffer pool: %lu\n"
+		"InnoDB: %lu OS file reads, %lu OS file writes, %lu OS fsyncs\n"
 		"InnoDB: Starting InnoDB Monitor to print further\n"
-		"InnoDB: diagnostics to the standard output.\n");
+		"InnoDB: diagnostics to the standard output.\n",
+			n_iterations,
+			fil_n_pending_log_flushes,
+			fil_n_pending_tablespace_flushes,
+			os_n_file_reads, os_n_file_writes, os_n_fsyncs);
 
 		mon_value_was = srv_print_innodb_monitor;
 		started_monitor = TRUE;
 		srv_print_innodb_monitor = TRUE;
+		os_event_set(srv_lock_timeout_thread_event);
 	}
 
 	/* No free block was found: try to flush the LRU list */
@@ -362,7 +366,9 @@ buf_LRU_old_adjust_len(void)
 	ulint	new_len;
 
 	ut_ad(buf_pool->LRU_old);
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(buf_pool->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_ad(3 * (BUF_LRU_OLD_MIN_LEN / 8) > BUF_LRU_OLD_TOLERANCE + 5);
 
 	for (;;) {
@@ -432,7 +438,9 @@ buf_LRU_remove_block(
 {
 	ut_ad(buf_pool);
 	ut_ad(block);
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(buf_pool->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 		
 	/* If the LRU_old pointer is defined and points to just this block,
 	move it backward one step */
@@ -485,7 +493,9 @@ buf_LRU_add_block_to_end_low(
 	
 	ut_ad(buf_pool);
 	ut_ad(block);
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(buf_pool->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	block->old = TRUE;
 
@@ -537,7 +547,9 @@ buf_LRU_add_block_low(
 	
 	ut_ad(buf_pool);
 	ut_ad(block);
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(buf_pool->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	block->old = old;
 	cl = buf_pool_clock_tic();
@@ -624,7 +636,9 @@ buf_LRU_block_free_non_file_page(
 /*=============================*/
 	buf_block_t*	block)	/* in: block, must not contain a file page */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(buf_pool->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_ad(block);
 	
 	ut_ad((block->state == BUF_BLOCK_MEMORY)
@@ -650,7 +664,9 @@ buf_LRU_block_remove_hashed_page(
 				be in a state where it can be freed; there
 				may or may not be a hash index to the page */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(buf_pool->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_ad(block);
 	
 	ut_ad(block->state == BUF_BLOCK_FILE_PAGE);
@@ -664,7 +680,7 @@ buf_LRU_block_remove_hashed_page(
 	buf_pool->freed_page_clock += 1;
 
  	buf_frame_modify_clock_inc(block->frame);
-		
+
 	HASH_DELETE(buf_block_t, hash, buf_pool->page_hash,
 			buf_page_address_fold(block->space, block->offset),
 			block);
@@ -681,7 +697,9 @@ buf_LRU_block_free_hashed_page(
 	buf_block_t*	block)	/* in: block, must contain a file page and
 				be in a state where it can be freed */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(buf_pool->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_ad(block->state == BUF_BLOCK_REMOVE_HASH);
 
 	block->state = BUF_BLOCK_MEMORY;
@@ -689,6 +707,7 @@ buf_LRU_block_free_hashed_page(
 	buf_LRU_block_free_non_file_page(block);
 }
 				
+#ifdef UNIV_DEBUG
 /**************************************************************************
 Validates the LRU list. */
 
@@ -775,7 +794,7 @@ buf_LRU_print(void)
 	ut_ad(buf_pool);
 	mutex_enter(&(buf_pool->mutex));
 
-	printf("Pool ulint clock %lu\n", buf_pool->ulint_clock);
+	fprintf(stderr, "Pool ulint clock %lu\n", buf_pool->ulint_clock);
 
 	block = UT_LIST_GET_FIRST(buf_pool->LRU);
 
@@ -783,39 +802,40 @@ buf_LRU_print(void)
 
 	while (block != NULL) {
 
-		printf("BLOCK %lu ", block->offset);
+		fprintf(stderr, "BLOCK %lu ", block->offset);
 
 		if (block->old) {
-			printf("old ");
+			fputs("old ", stderr);
 		}
 
 		if (block->buf_fix_count) {
-			printf("buffix count %lu ", block->buf_fix_count);
+			fprintf(stderr, "buffix count %lu ",
+				block->buf_fix_count);
 		}
 
 		if (block->io_fix) {
-			printf("io_fix %lu ", block->io_fix);
+			fprintf(stderr, "io_fix %lu ", block->io_fix);
 		}
 
 		if (ut_dulint_cmp(block->oldest_modification,
 				ut_dulint_zero) > 0) {
-			printf("modif. ");
+			fputs("modif. ", stderr);
 		}
 
-		printf("LRU pos %lu ", block->LRU_position);
-		
 		frame = buf_block_get_frame(block);
 
-		printf("type %lu ", fil_page_get_type(frame));
-		printf("index id %lu ", ut_dulint_get_low(
-					btr_page_get_index_id(frame)));
+		fprintf(stderr, "LRU pos %lu type %lu index id %lu ",
+			block->LRU_position,
+			fil_page_get_type(frame),
+			ut_dulint_get_low(btr_page_get_index_id(frame)));
 
 		block = UT_LIST_GET_NEXT(LRU, block);
-		len++;
-		if (len % 10 == 0) {
-			printf("\n");
+		if (++len == 10) {
+			len = 0;
+			putc('\n', stderr);
 		}
 	}
 
 	mutex_exit(&(buf_pool->mutex));
 }
+#endif /* UNIV_DEBUG */
