@@ -49,7 +49,7 @@ The solution is the following: We put into each tablespace an insert buffer
 of its own. Let all the tree and page latches connected with the insert buffer
 be later in the latching order than the fsp latch and fsp page latches.
 Insert buffer pages must be such that the insert buffer is never invoked
-when these pages area accessed as this would result in a recursion violating
+when these pages are accessed as this would result in a recursion violating
 the latching order. We let a special i/o-handler thread take care of i/o to
 the insert buffer pages and the ibuf bitmap pages, as well as the fsp bitmap
 pages and the first inode page, which contains the inode of the ibuf tree: let
@@ -102,9 +102,10 @@ pages, as long as it obeys the access order rules. */
 #define IBUF_POOL_SIZE_PER_MAX_SIZE	2
 
 /* The insert buffer control structure */
-ibuf_t*	ibuf	= NULL;
+ibuf_t*	ibuf			= NULL;
 
-ulint	ibuf_rnd = 986058871;
+static
+ulint	ibuf_rnd		= 986058871;
 
 ulint	ibuf_flush_count	= 0;
 
@@ -113,9 +114,9 @@ ulint	ibuf_flush_count	= 0;
 #define IBUF_COUNT_N_PAGES	10000
 
 /* Buffered entry counts for file pages, used in debugging */
-ulint*	ibuf_counts[IBUF_COUNT_N_SPACES];
+static ulint*	ibuf_counts[IBUF_COUNT_N_SPACES];
 
-ibool	ibuf_counts_inited	= FALSE;
+static ibool	ibuf_counts_inited	= FALSE;
 
 /* The start address for an insert buffer bitmap page bitmap */
 #define IBUF_BITMAP		PAGE_DATA
@@ -129,15 +130,18 @@ ibool	ibuf_counts_inited	= FALSE;
 
 /* Number of bits describing a single page */
 #define IBUF_BITS_PER_PAGE	4
+#if IBUF_BITS_PER_PAGE % 2
+# error "IBUF_BITS_PER_PAGE must be an even number!"
+#endif
 
 /* The mutex used to block pessimistic inserts to ibuf trees */
-mutex_t	ibuf_pessimistic_insert_mutex;
+static mutex_t	ibuf_pessimistic_insert_mutex;
 
 /* The mutex protecting the insert buffer structs */
-mutex_t	ibuf_mutex;
+static mutex_t	ibuf_mutex;
 
 /* The mutex protecting the insert buffer bitmaps */
-mutex_t	ibuf_bitmap_mutex;
+static mutex_t	ibuf_bitmap_mutex;
 
 /* The area in pages from which contract looks for page numbers for merge */
 #define	IBUF_MERGE_AREA			8
@@ -168,6 +172,7 @@ because ibuf merge is done to a page when it is read in, and it is
 still physically like the index page even if the index would have been
 dropped! So, there seems to be no problem. */
 
+#ifdef UNIV_DEBUG
 /**********************************************************************
 Validates the ibuf data structures when the caller owns ibuf_mutex. */
 static
@@ -175,6 +180,7 @@ ibool
 ibuf_validate_low(void);
 /*===================*/
 			/* out: TRUE if ok */
+#endif /* UNIV_DEBUG */
 
 /**********************************************************************
 Sets the flag in the current OS thread local storage denoting that it is
@@ -239,7 +245,9 @@ ibuf_header_page_get(
 
 	page = buf_page_get(space, FSP_IBUF_HEADER_PAGE_NO, RW_X_LATCH, mtr);
 
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(page, SYNC_IBUF_HEADER);
+#endif /* UNIV_SYNC_DEBUG */
 
 	return(page);
 }
@@ -263,7 +271,9 @@ ibuf_tree_root_get(
 
 	page = buf_page_get(space, FSP_IBUF_TREE_ROOT_PAGE_NO, RW_X_LATCH,
 									mtr);
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(page, SYNC_TREE_NODE);
+#endif /* UNIV_SYNC_DEBUG */
 
 	return(page);
 }
@@ -292,6 +302,7 @@ ibuf_count_get(
 
 /**********************************************************************
 Sets the ibuf count for a given page. */
+#ifdef UNIV_IBUF_DEBUG
 static
 void
 ibuf_count_set(
@@ -306,6 +317,7 @@ ibuf_count_set(
 
 	*(ibuf_counts[space] + page_no) = val;
 }
+#endif
 
 /**********************************************************************
 Creates the insert buffer data structure at a database startup and
@@ -373,7 +385,9 @@ ibuf_data_sizes_update(
 {
 	ulint	old_size;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&ibuf_mutex));
+#endif /* UNIV_SYNC_DEBUG */
 
 	old_size = data->size;
 	
@@ -397,7 +411,7 @@ ibuf_data_sizes_update(
 
 	ibuf->size = ibuf->size + data->size - old_size;
 
-/*	printf("ibuf size %lu, space ibuf size %lu\n", ibuf->size,
+/*	fprintf(stderr, "ibuf size %lu, space ibuf size %lu\n", ibuf->size,
 							data->size); */
 }	
 
@@ -426,7 +440,7 @@ ibuf_data_init_for_space(
 #ifdef UNIV_LOG_DEBUG
 	if (space % 2 == 1) {
 
-		printf("No ibuf op in replicate space\n");
+		fputs("No ibuf op in replicate space\n", stderr);
 
 		return(NULL);
 	}
@@ -453,7 +467,9 @@ ibuf_data_init_for_space(
 	
 	root = buf_page_get(space, FSP_IBUF_TREE_ROOT_PAGE_NO, RW_X_LATCH,
 								&mtr);
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(root, SYNC_TREE_NODE);
+#endif /* UNIV_SYNC_DEBUG */
 
 	data->size = 0;
 	data->n_inserts = 0;
@@ -472,19 +488,18 @@ ibuf_data_init_for_space(
 	
 	table = dict_mem_table_create(buf, space, 2);
 
-	dict_mem_table_add_col(table, "PAGE_NO", DATA_BINARY, 0, 0, 0);
-	dict_mem_table_add_col(table, "TYPES", DATA_BINARY, 0, 0, 0);
+	dict_mem_table_add_col(table,(char *) "PAGE_NO", DATA_BINARY, 0, 0, 0);
+	dict_mem_table_add_col(table,(char *) "TYPES", DATA_BINARY, 0, 0, 0);
 
 	table->id = ut_dulint_add(DICT_IBUF_ID_MIN, space);
 
 	dict_table_add_to_cache(table);
 
-	index = dict_mem_index_create(buf, "CLUST_IND", space,
-				DICT_CLUSTERED | DICT_UNIVERSAL | DICT_IBUF,
-				2);
+	index = dict_mem_index_create(buf, (char *) "CLUST_IND", space,
+				DICT_CLUSTERED | DICT_UNIVERSAL | DICT_IBUF,2);
 
-	dict_mem_index_add_field(index, "PAGE_NO", 0);
-	dict_mem_index_add_field(index, "TYPES", 0);
+	dict_mem_index_add_field(index, (char *) "PAGE_NO", 0, 0);
+	dict_mem_index_add_field(index, (char *) "TYPES", 0, 0);
 
 	index->page_no = FSP_IBUF_TREE_ROOT_PAGE_NO;
 	
@@ -538,7 +553,7 @@ ibuf_parse_bitmap_init(
 /*===================*/
 			/* out: end of log record or NULL */
 	byte*	ptr,	/* in: buffer */
-	byte*	end_ptr,/* in: buffer end */
+	byte*	end_ptr __attribute__((unused)), /* in: buffer end */
 	page_t*	page,	/* in: page or NULL */
 	mtr_t*	mtr)	/* in: mtr or NULL */
 {
@@ -561,7 +576,8 @@ ibuf_bitmap_page_get_bits(
 	page_t*	page,	/* in: bitmap page */
 	ulint	page_no,/* in: page whose bits to get */
 	ulint	bit,	/* in: IBUF_BITMAP_FREE, IBUF_BITMAP_BUFFERED, ... */
-	mtr_t*	mtr)	/* in: mtr containing an x-latch to the bitmap page */
+	mtr_t*	mtr __attribute__((unused))) /* in: mtr containing an x-latch
+                                               to the bitmap page */
 {
 	ulint	byte_offset;
 	ulint	bit_offset;
@@ -677,7 +693,9 @@ ibuf_bitmap_get_map_page(
 	
 	page = buf_page_get(space, ibuf_bitmap_page_no_calc(page_no),
 							RW_X_LATCH, mtr);
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(page, SYNC_IBUF_BITMAP);
+#endif /* UNIV_SYNC_DEBUG */
 
 	return(page);
 }
@@ -712,7 +730,8 @@ ibuf_set_free_bits_low(
 	bitmap_page = ibuf_bitmap_get_map_page(buf_frame_get_space_id(page),
 					buf_frame_get_page_no(page), mtr);
 #ifdef UNIV_IBUF_DEBUG
-	/* printf("Setting page no %lu free bits to %lu should be %lu\n",
+	/* fprintf(stderr,
+		"Setting page no %lu free bits to %lu should be %lu\n",
 					buf_frame_get_page_no(page), val,
 				ibuf_index_page_calc_free(page)); */
 	
@@ -766,7 +785,7 @@ ibuf_set_free_bits(
 					buf_frame_get_page_no(page),
 						IBUF_BITMAP_FREE, &mtr);
 		if (old_val != max_val) {
-			/* printf(
+			/* fprintf(stderr,
 			"Ibuf: page %lu old val %lu max val %lu\n",
 			buf_frame_get_page_no(page), old_val, max_val); */
 		}
@@ -775,7 +794,7 @@ ibuf_set_free_bits(
 #endif
 	}
 #ifdef UNIV_IBUF_DEBUG
-/*	printf("Setting page no %lu free bits to %lu should be %lu\n",
+/*	fprintf(stderr, "Setting page no %lu free bits to %lu should be %lu\n",
 					buf_frame_get_page_no(page), val,
 				ibuf_index_page_calc_free(page)); */
 
@@ -954,7 +973,7 @@ ibuf_page_low(
 #ifdef UNIV_LOG_DEBUG
 	if (space % 2 != 0) {
 
-		printf("No ibuf in a replicate space\n");
+		fputs("No ibuf in a replicate space\n", stderr);
 
 		return(FALSE);
 	}
@@ -1196,7 +1215,9 @@ ibuf_data_enough_free_for_insert(
 				/* out: TRUE if enough free pages in list */
 	ibuf_data_t*	data)	/* in: ibuf data for the space */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&ibuf_mutex));
+#endif /* UNIV_SYNC_DEBUG */
 
 	/* We want a big margin of free pages, because a B-tree can sometimes
 	grow in size also if records are deleted from it, as the node pointers
@@ -1222,7 +1243,9 @@ ibuf_data_too_much_free(
 				/* out: TRUE if enough free pages in list */
 	ibuf_data_t*	data)	/* in: ibuf data for the space */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&ibuf_mutex));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (data->free_list_len >= 3 + data->size / 2 + 3 * data->height) {
 
@@ -1280,7 +1303,9 @@ ibuf_add_free_page(
 
 	page = buf_page_get(space, page_no, RW_X_LATCH, &mtr);
 
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(page, SYNC_TREE_NODE_NEW);
+#endif /* UNIV_SYNC_DEBUG */
 
 	ibuf_enter();
 
@@ -1293,6 +1318,8 @@ ibuf_add_free_page(
 	flst_add_last(root + PAGE_HEADER + PAGE_BTR_IBUF_FREE_LIST,
 		      page + PAGE_HEADER + PAGE_BTR_IBUF_FREE_LIST_NODE, &mtr);
 
+	fil_page_set_type(page, FIL_PAGE_IBUF_FREE_LIST);
+		      
 	ibuf_data->seg_size++;
 	ibuf_data->free_list_len++;
 
@@ -1303,6 +1330,7 @@ ibuf_add_free_page(
 
 	ibuf_bitmap_page_set_bits(bitmap_page, page_no, IBUF_BITMAP_IBUF,
 								TRUE, &mtr);
+
 	mtr_commit(&mtr);
 
 	mutex_exit(&ibuf_mutex);
@@ -1397,7 +1425,9 @@ ibuf_remove_free_page(
 
 	page = buf_page_get(space, page_no, RW_X_LATCH, &mtr);
 
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(page, SYNC_TREE_NODE);
+#endif /* UNIV_SYNC_DEBUG */
 
 	/* Remove the page from the free list and update the ibuf size data */
 	
@@ -1438,8 +1468,9 @@ ibuf_free_excess_pages(
 {
 	ibuf_data_t*	ibuf_data;
 	ulint		i;
-	
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(rw_lock_own(fil_space_get_latch(space), RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_ad(rw_lock_get_x_lock_count(fil_space_get_latch(space)) == 1);
 	ut_ad(!ibuf_inside());
 	
@@ -1453,7 +1484,8 @@ ibuf_free_excess_pages(
 		/* Not yet initialized */
 
 #ifdef UNIV_DEBUG
-		/*printf("Ibuf for space %lu not yet initialized\n", space); */
+		/*fprintf(stderr,
+			"Ibuf for space %lu not yet initialized\n", space); */
 #endif
 
 		return;
@@ -1617,7 +1649,7 @@ ibuf_get_merge_page_nos(
 #ifdef UNIV_IBUF_DEBUG
 	ut_a(*n_stored <= IBUF_MAX_N_PAGES_MERGED);
 #endif
-/*	printf("Ibuf merge batch %lu pages %lu volume\n", *n_stored,
+/*	fprintf(stderr, "Ibuf merge batch %lu pages %lu volume\n", *n_stored,
 							sum_volumes); */
 	return(sum_volumes);
 }
@@ -1723,8 +1755,8 @@ loop:
 							page_nos, &n_stored);
 
 #ifdef UNIV_IBUF_DEBUG
-	/* printf("Ibuf contract sync %lu pages %lu volume %lu\n", sync,
-						n_stored, sum_sizes); */
+	/* fprintf(stderr, "Ibuf contract sync %lu pages %lu volume %lu\n",
+		sync, n_stored, sum_sizes); */
 #endif
 	ibuf_exit();
 
@@ -1904,7 +1936,9 @@ ibuf_get_volume_buffered(
 
 	prev_page = buf_page_get(space, prev_page_no, RW_X_LATCH, mtr);
 
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(prev_page, SYNC_TREE_NODE);
+#endif /* UNIV_SYNC_DEBUG */
 
 	rec = page_get_supremum_rec(prev_page);
 	rec = page_rec_get_prev(rec);
@@ -1963,7 +1997,9 @@ count_later:
 
 	next_page = buf_page_get(space, next_page_no, RW_X_LATCH, mtr);
 
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(next_page, SYNC_TREE_NODE);
+#endif /* UNIV_SYNC_DEBUG */
 
 	rec = page_get_infimum_rec(next_page);
 	rec = page_rec_get_next(rec);
@@ -2042,7 +2078,7 @@ ibuf_insert_low(
 		mutex_exit(&ibuf_mutex);
 
 #ifdef UNIV_IBUF_DEBUG
-		printf("Ibuf too big\n");
+		fputs("Ibuf too big\n", stderr);
 #endif		
 		/* Use synchronous contract (== TRUE) */
 		ibuf_contract(TRUE);
@@ -2270,8 +2306,8 @@ ibuf_insert(
 	
 	if (err == DB_SUCCESS) {
 #ifdef UNIV_IBUF_DEBUG
-		/* printf("Ibuf insert for page no %lu of index %s\n", page_no,
-							index->name); */
+		/* fprintf(stderr, "Ibuf insert for page no %lu of index %s\n",
+			page_no, index->name); */
 #endif
 		return(TRUE);
 
@@ -2299,7 +2335,6 @@ ibuf_insert_to_index_page(
 	rec_t*		rec;
 	page_t*		bitmap_page;
 	ulint		old_bits;
-	char		errbuf[1000];
 
 	ut_ad(ibuf_inside());
 	ut_ad(dtuple_check_typed(entry));
@@ -2329,16 +2364,13 @@ ibuf_insert_to_index_page(
 "InnoDB: Error: Insert buffer insert fails; page free %lu, dtuple size %lu\n",
 				page_get_max_insert_size(page, 1),
 				rec_get_converted_size(entry));
-
-				dtuple_sprintf(errbuf, 900, entry);
-				
-				fprintf(stderr,
-"InnoDB: Cannot insert index record %s\n", errbuf);
-
-				fprintf(stderr,
-"InnoDB: The table where where this index record belongs\n"
+				fputs("InnoDB: Cannot insert index record ",
+					stderr);
+				dtuple_print(stderr, entry);
+				fputs(
+"\nInnoDB: The table where where this index record belongs\n"
 "InnoDB: is now probably corrupt. Please run CHECK TABLE on\n"
-"InnoDB: that table.\n");
+"InnoDB: that table.\n", stderr);
 				
 				bitmap_page = ibuf_bitmap_get_map_page(
 						buf_frame_get_space_id(page),
@@ -2352,8 +2384,8 @@ ibuf_insert_to_index_page(
 
 				fprintf(stderr, "Bitmap bits %lu\n", old_bits);
 
-				fprintf(stderr,
-"InnoDB: Send a detailed bug report to mysql@lists.mysql.com!\n");
+				fputs(
+"InnoDB: Submit a detailed bug report to http://bugs.mysql.com\n", stderr);
 				
 			}	
 		}
@@ -2386,7 +2418,7 @@ ibuf_delete_rec(
 
 	ut_ad(ibuf_inside());
 
-	success = btr_cur_optimistic_delete(btr_pcur_get_btr_cur(pcur), mtr);	
+	success = btr_cur_optimistic_delete(btr_pcur_get_btr_cur(pcur), mtr);
 
 	if (success) {
 #ifdef UNIV_IBUF_DEBUG
@@ -2396,7 +2428,7 @@ ibuf_delete_rec(
 		return(FALSE);
 	}
 	
-	/* We have to resort to a pessimistic delete from ibuf */		
+	/* We have to resort to a pessimistic delete from ibuf */
 	btr_pcur_store_position(pcur, mtr);
 
 	btr_pcur_commit_specify_mtr(pcur, mtr);
@@ -2411,21 +2443,25 @@ ibuf_delete_rec(
 
 	if (!success) {
 		fprintf(stderr,
-		"InnoDB: ERROR: Send the output to heikki.tuuri@innodb.com\n");
-		fprintf(stderr, "InnoDB: ibuf cursor restoration fails!\n");
-		fprintf(stderr, "InnoDB: ibuf record inserted to page %lu\n",
-								page_no);
-		rec_print(btr_pcur_get_rec(pcur));
-		rec_print(pcur->old_rec);
-		dtuple_print(search_tuple);
+		"InnoDB: ERROR: Submit the output to http://bugs.mysql.com\n"
+		"InnoDB: ibuf cursor restoration fails!\n"
+		"InnoDB: ibuf record inserted to page %lu\n", page_no);
+		fflush(stderr);
 
-		rec_print(page_rec_get_next(btr_pcur_get_rec(pcur)));
+		rec_print(stderr, btr_pcur_get_rec(pcur));
+		rec_print(stderr, pcur->old_rec);
+		dtuple_print(stderr, search_tuple);
+
+		rec_print(stderr, page_rec_get_next(btr_pcur_get_rec(pcur)));
+		fflush(stderr);
 
 		mtr_commit(mtr);
 
-		fprintf(stderr, "InnoDB: Validating insert buffer tree:\n");
-		ut_a(btr_validate_tree(ibuf_data->index->tree));		
-		fprintf(stderr, "InnoDB: Ibuf tree ok\n");
+		fputs("InnoDB: Validating insert buffer tree:\n", stderr);
+		ut_a(btr_validate_tree(ibuf_data->index->tree));
+
+		fprintf(stderr, "InnoDB: ibuf tree ok\n");
+		fflush(stderr);
 	}
 	
 	ut_a(success);
@@ -2473,19 +2509,15 @@ ibuf_merge_or_delete_for_page(
 	dtuple_t*	entry;
 	dtuple_t*	search_tuple;
 	rec_t*		ibuf_rec;
-	ibool		closed;
 	buf_block_t*	block;
 	page_t*		bitmap_page;
 	ibuf_data_t*	ibuf_data;
-	ibool		success;
 	ulint		n_inserts;
+#ifdef UNIV_IBUF_DEBUG
 	ulint		volume;
-	ulint		old_bits;
-	ulint		new_bits;
-	dulint		max_trx_id;
+#endif
 	ibool		corruption_noticed	= FALSE;
 	mtr_t		mtr;
-	char		err_buf[500];
 
 	if (srv_force_recovery >= SRV_FORCE_NO_IBUF_MERGE) {
 
@@ -2495,7 +2527,7 @@ ibuf_merge_or_delete_for_page(
 #ifdef UNIV_LOG_DEBUG
 	if (space % 2 != 0) {
 
-		printf("No ibuf operation in a replicate space\n");
+		fputs("No ibuf operation in a replicate space\n", stderr);
 
 		return;
 	}
@@ -2545,8 +2577,8 @@ ibuf_merge_or_delete_for_page(
 
 			mtr_start(&mtr);
 
-			fprintf(stderr,
-"  InnoDB: Dump of the ibuf bitmap page:\n");
+			fputs("  InnoDB: Dump of the ibuf bitmap page:\n",
+				stderr);
 			
 			bitmap_page = ibuf_bitmap_get_map_page(space, page_no,
 									&mtr);
@@ -2554,7 +2586,7 @@ ibuf_merge_or_delete_for_page(
 		
 			mtr_commit(&mtr);
 
-			fprintf(stderr, "\nInnoDB: Dump of the page:\n");
+			fputs("\nInnoDB: Dump of the page:\n", stderr);
 
 			buf_page_print(page);
 
@@ -2565,25 +2597,27 @@ ibuf_merge_or_delete_for_page(
 "InnoDB: We try to resolve the problem by skipping the insert buffer\n"
 "InnoDB: merge for this page. Please run CHECK TABLE on your tables\n"
 "InnoDB: to determine if they are corrupt after this.\n\n"
-"InnoDB: Please make a detailed bug report and send it to\n"
-"InnoDB: mysql@lists.mysql.com\n\n",
+"InnoDB: Please submit a detailed bug report to http://bugs.mysql.com\n\n",
 				page_no, fil_page_get_type(page));
 		}
 	}
 
 	n_inserts = 0;
+#ifdef UNIV_IBUF_DEBUG
 	volume = 0;
+#endif
 loop:
 	mtr_start(&mtr);
 
 	if (page) {
-		success = buf_page_get_known_nowait(RW_X_LATCH, page,
+		ibool success = buf_page_get_known_nowait(RW_X_LATCH, page,
 					BUF_KEEP_OLD,
 					IB__FILE__, __LINE__,
 					&mtr);
 		ut_a(success);
-
+#ifdef UNIV_SYNC_DEBUG
 		buf_page_dbg_add_level(page, SYNC_TREE_NODE);
+#endif /* UNIV_SYNC_DEBUG */
 	}
 		
 	/* Position pcur in the insert buffer at the first entry for this
@@ -2612,12 +2646,18 @@ loop:
 			goto reset_bit;
 		}
 
-		if (corruption_noticed) {
-			rec_sprintf(err_buf, 450, ibuf_rec);
+		/* Do NOT merge to the 4.1 code base! */
+		if (trx_sys_downgrading_from_4_1_1) {
+			fputs(
+"InnoDB: Fatal error: you are downgrading from >= 4.1.1 to 4.0, but\n"
+"InnoDB: the insert buffer was not empty.\n", stderr);
+			ut_error;
+		}
 
-			fprintf(stderr,
-"InnoDB: Discarding record\n %s\n from the insert buffer!\n\n", err_buf);
-	
+		if (corruption_noticed) {
+			fputs("InnoDB: Discarding record\n ", stderr);
+			rec_print(stderr, ibuf_rec);
+			fputs("\n from the insert buffer!\n\n", stderr);
 	   	} else if (page) {
 			/* Now we have at pcur a record which should be
 			inserted to the index page; NOTE that the call below
@@ -2625,7 +2665,7 @@ loop:
 			keep the latch to the ibuf_rec page until the
 			insertion is finished! */
 
-			max_trx_id = page_get_max_trx_id(
+			dulint	max_trx_id = page_get_max_trx_id(
 						buf_frame_align(ibuf_rec));
 	
 			page_update_max_trx_id(page, max_trx_id);
@@ -2644,9 +2684,8 @@ loop:
 		n_inserts++;
 		
 		/* Delete the record from ibuf */
-		closed = ibuf_delete_rec(space, page_no, &pcur, search_tuple,
-									&mtr);
-		if (closed) {
+		if (ibuf_delete_rec(space, page_no, &pcur, search_tuple,
+								&mtr)) {
 			/* Deletion was pessimistic and mtr was committed:
 			we start from the beginning again */
 
@@ -2675,13 +2714,12 @@ reset_bit:
 	ibuf_bitmap_page_set_bits(bitmap_page, page_no,
 					IBUF_BITMAP_BUFFERED, FALSE, &mtr);
 	if (page) {
-		old_bits = ibuf_bitmap_page_get_bits(bitmap_page, page_no,
-						IBUF_BITMAP_FREE, &mtr);
-		new_bits = ibuf_index_page_calc_free(page);
-
+		ulint old_bits = ibuf_bitmap_page_get_bits(bitmap_page,
+				page_no, IBUF_BITMAP_FREE, &mtr);
+		ulint new_bits = ibuf_index_page_calc_free(page);
 #ifdef UNIV_IBUF_DEBUG
-		/* printf("Old bits %lu new bits %lu max size %lu\n", old_bits,
-			new_bits,
+		/* fprintf(stderr, "Old bits %lu new bits %lu max size %lu\n",
+			old_bits, new_bits,
 			page_get_max_insert_size_after_reorganize(page, 1)); */
 #endif
 		if (old_bits != new_bits) {
@@ -2691,12 +2729,10 @@ reset_bit:
 							new_bits, &mtr);
 		}
 	}
-
-	ibuf_data->n_merges++;	
-	ibuf_data->n_merged_recs += n_inserts;
-
+	
 #ifdef UNIV_IBUF_DEBUG
-	/* printf("Ibuf merge %lu records volume %lu to page no %lu\n",
+	/* fprintf(stderr,
+		"Ibuf merge %lu records volume %lu to page no %lu\n",
 					n_inserts, volume, page_no); */
 #endif
 	mtr_commit(&mtr);
@@ -2704,12 +2740,21 @@ reset_bit:
  	
 	mem_heap_free(heap);
 
+	/* Protect our statistics keeping from race conditions */
+	mutex_enter(&ibuf_mutex);
+
+	ibuf_data->n_merges++;	
+	ibuf_data->n_merged_recs += n_inserts;
+
+	mutex_exit(&ibuf_mutex);
+
 	ibuf_exit();
 #ifdef UNIV_IBUF_DEBUG
 	ut_a(ibuf_count_get(space, page_no) == 0);
 #endif
 }
 
+#ifdef UNIV_DEBUG
 /**********************************************************************
 Validates the ibuf data structures when the caller owns ibuf_mutex. */
 static
@@ -2721,7 +2766,9 @@ ibuf_validate_low(void)
 	ibuf_data_t*	data;
 	ulint		sum_sizes;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&ibuf_mutex));
+#endif /* UNIV_SYNC_DEBUG */
 
 	sum_sizes = 0;
 	
@@ -2737,6 +2784,7 @@ ibuf_validate_low(void)
 
 	return(TRUE);
 }
+#endif /* UNIV_DEBUG */
 
 /**********************************************************************
 Prints info of ibuf. */
@@ -2744,34 +2792,29 @@ Prints info of ibuf. */
 void
 ibuf_print(
 /*=======*/
-	char*	buf,	/* in/out: buffer where to print */
-	char*	buf_end)/* in: buffer end */
+	FILE*	file)	/* in: file where to print */
 {
 	ibuf_data_t*	data;
 #ifdef UNIV_IBUF_DEBUG
 	ulint		i;
 #endif
-	if (buf_end - buf < 500) {
-		return;
-	}
 
 	mutex_enter(&ibuf_mutex);
 
 	data = UT_LIST_GET_FIRST(ibuf->data_list);
 
 	while (data) {
-		buf += sprintf(buf,
-  	"Ibuf for space %lu: size %lu, free list len %lu, seg size %lu,\n",
-		data->space, data->size, data->free_list_len, data->seg_size);
-
-		buf += sprintf(buf,
+		fprintf(file,
+	"Ibuf for space %lu: size %lu, free list len %lu, seg size %lu,\n"
 			"%lu inserts, %lu merged recs, %lu merges\n",
+		data->space, data->size, data->free_list_len, data->seg_size,
 			data->n_inserts, data->n_merged_recs, data->n_merges);
 #ifdef UNIV_IBUF_DEBUG
 		for (i = 0; i < IBUF_COUNT_N_PAGES; i++) {
 			if (ibuf_count_get(data->space, i) > 0) {
 
-				printf("Ibuf count for page %lu is %lu\n",
+				fprintf(stderr,
+					"Ibuf count for page %lu is %lu\n",
 					i, ibuf_count_get(data->space, i));
 			}
 		}
