@@ -212,7 +212,7 @@ os_file_get_last_error(void)
 		ut_print_timestamp(stderr);
 	     	fprintf(stderr,
   "  InnoDB: Operating system error number %lu in a file operation.\n"
-  "InnoDB: See http://www.innodb.com/ibman.html for installation help.\n",
+  "InnoDB: See http://www.innodb.com/ibman.php for installation help.\n",
 		err);
 
 		if (err == ERROR_PATH_NOT_FOUND) {
@@ -227,7 +227,7 @@ os_file_get_last_error(void)
   "InnoDB: of the same name as a data file.\n"); 
 		} else {
 			 fprintf(stderr,
-  "InnoDB: See section 13.2 at http://www.innodb.com/ibman.html\n"
+  "InnoDB: See section 13.2 at http://www.innodb.com/ibman.php\n"
   "InnoDB: about operating system error numbers.\n");
 		}
 	}
@@ -251,7 +251,7 @@ os_file_get_last_error(void)
 
 	     	fprintf(stderr,
   "  InnoDB: Operating system error number %lu in a file operation.\n"
-  "InnoDB: See http://www.innodb.com/ibman.html for installation help.\n",
+  "InnoDB: See http://www.innodb.com/ibman.php for installation help.\n",
 		err);
 
 		if (err == ENOENT) {
@@ -270,7 +270,7 @@ os_file_get_last_error(void)
 			 }
 
 			 fprintf(stderr,
-  "InnoDB: See also section 13.2 at http://www.innodb.com/ibman.html\n"
+  "InnoDB: See also section 13.2 at http://www.innodb.com/ibman.php\n"
   "InnoDB: about operating system error numbers.\n");
 		}
 	}
@@ -302,7 +302,7 @@ os_file_handle_error(
 				/* out: TRUE if we should retry the
 				operation */
 	os_file_t	file,	/* in: file pointer */
-	char*		name,	/* in: name of a file or NULL */
+	const char*	name,	/* in: name of a file or NULL */
 	const char*	operation)/* in: operation */
 {
 	ulint	err;
@@ -441,7 +441,7 @@ try_again:
 	}
 
 	return(file);
-#else
+#else /* __WIN__ */
 	os_file_t	file;
 	int		create_flag;
 	ibool		retry;
@@ -483,7 +483,7 @@ try_again:
 	}
 
 	return(file);	
-#endif
+#endif /* __WIN__ */
 }
 
 /********************************************************************
@@ -544,7 +544,7 @@ os_file_create_simple_no_error_handling(
 	}
 
 	return(file);
-#else
+#else /* __WIN__ */
 	os_file_t	file;
 	int		create_flag;
 	
@@ -577,7 +577,7 @@ os_file_create_simple_no_error_handling(
 	}
 
 	return(file);	
-#endif
+#endif /* __WIN__ */
 }
 
 /********************************************************************
@@ -689,7 +689,7 @@ try_again:
 	}
 
 	return(file);
-#else
+#else /* __WIN__ */
 	os_file_t	file;
 	int		create_flag;
 	ibool		retry;
@@ -722,7 +722,7 @@ try_again:
 	} else if (type == OS_DATA_FILE) {
 		type_str = "DATA";
 	} else {
-	        ut_a(0);
+	        ut_error;
 	}
 	  
 	if (purpose == OS_FILE_AIO) {
@@ -730,10 +730,10 @@ try_again:
 	} else if (purpose == OS_FILE_NORMAL) {
 		purpose_str = "NORMAL";
 	} else {
-	        ut_a(0);
+	        ut_error;
 	}
 
-/*	printf("Opening file %s, mode %s, type %s, purpose %s\n",
+/*	fprintf(stderr, "Opening file %s, mode %s, type %s, purpose %s\n",
 			       name, mode_str, type_str, purpose_str); */
 #ifdef O_SYNC
         /* We let O_SYNC only affect log files; note that we map O_DSYNC to
@@ -742,7 +742,7 @@ try_again:
 	if (type == OS_LOG_FILE
 	    && srv_unix_file_flush_method == SRV_UNIX_O_DSYNC) {
 
-/*		printf("Using O_SYNC for file %s\n", name); */
+/*		fprintf(stderr, "Using O_SYNC for file %s\n", name); */
 
 	        create_flag = create_flag | O_SYNC;
 	}
@@ -752,7 +752,7 @@ try_again:
 	if (type != OS_LOG_FILE
 	    && srv_unix_file_flush_method == SRV_UNIX_O_DIRECT) {
 
-/*		printf("Using O_DIRECT for file %s\n", name); */
+/*		fprintf(stderr, "Using O_DIRECT for file %s\n", name); */
 
 	        create_flag = create_flag | O_DIRECT;
 	}
@@ -777,7 +777,7 @@ try_again:
 	}
 
 	return(file);	
-#endif
+#endif /* __WIN__ */
 }
 
 /***************************************************************************
@@ -990,6 +990,23 @@ error_handling:
 	return(FALSE);
 }
 
+/***************************************************************************
+Truncates a file at its current position. */
+
+ibool
+os_file_set_eof(
+/*============*/
+				/* out: TRUE if success */
+	FILE*		file)	/* in: file to be truncated */
+{
+#ifdef __WIN__
+	HANDLE h = (HANDLE) _get_osfhandle(fileno(file));
+	return(SetEndOfFile(h));
+#else /* __WIN__ */
+	return(!ftruncate(fileno(file), ftell(file)));
+#endif /* __WIN__ */
+}
+
 /***************************************************************************
 Flushes the write buffers of a given file to the disk. */
 
@@ -1016,7 +1033,7 @@ os_file_flush(
 
 	/* It is a fatal error if a file flush does not succeed, because then
 	the database can get corrupt on disk */
-	ut_a(0);
+	ut_error;
 
 	return(FALSE);
 #else
@@ -1025,7 +1042,7 @@ os_file_flush(
 #ifdef HAVE_FDATASYNC
 	ret = fdatasync(file);
 #else
-/*	printf("Flushing to file %lu\n", (ulint)file); */
+/*	fprintf(stderr, "Flushing to file %p\n", file); */
 	ret = fsync(file);
 #endif
 	os_n_fsyncs++;
@@ -1050,7 +1067,7 @@ os_file_flush(
 
 	/* It is a fatal error if a file flush does not succeed, because then
 	the database can get corrupt on disk */
-	ut_a(0);
+	ut_error;
 
 	return(FALSE);
 #endif
@@ -1382,7 +1399,7 @@ retry:
 		fprintf(stderr,
 "  InnoDB: Error: File pointer positioning to file %s failed at\n"
 "InnoDB: offset %lu %lu. Operating system error number %lu.\n"
-"InnoDB: Look from section 13.2 at http://www.innodb.com/ibman.html\n"
+"InnoDB: Look from section 13.2 at http://www.innodb.com/ibman.php\n"
 "InnoDB: what the error number means.\n",
 			name, offset_high, offset,
 			(ulint)GetLastError());
@@ -1440,7 +1457,7 @@ retry:
 		}
 
 		fprintf(stderr,
-"InnoDB: See also section 13.2 at http://www.innodb.com/ibman.html\n"
+"InnoDB: See also section 13.2 at http://www.innodb.com/ibman.php\n"
 "InnoDB: about operating system error numbers.\n");
 
 		os_has_said_disk_full = TRUE;
@@ -1475,7 +1492,7 @@ retry:
 		}
 
 		fprintf(stderr,
-"InnoDB: See also section 13.2 at http://www.innodb.com/ibman.html\n"
+"InnoDB: See also section 13.2 at http://www.innodb.com/ibman.php\n"
 "InnoDB: about operating system error numbers.\n");
 
 		os_has_said_disk_full = TRUE;
@@ -1586,33 +1603,35 @@ os_aio_init(
 	os_io_init_simple();
 
 	for (i = 0; i < n_segments; i++) {
-	        srv_io_thread_op_info[i] = (char*)"not started yet";
+	        srv_set_io_thread_op_info(i, "not started yet");
 	}
 
 	n_per_seg = n / n_segments;
 	n_write_segs = (n_segments - 2) / 2;
 	n_read_segs = n_segments - 2 - n_write_segs;
 	
-	/* printf("Array n per seg %lu\n", n_per_seg); */
+	/* fprintf(stderr, "Array n per seg %lu\n", n_per_seg); */
 
 	os_aio_ibuf_array = os_aio_array_create(n_per_seg, 1);
 
-	srv_io_thread_function[0] = (char*)"insert buffer thread";
+	srv_io_thread_function[0] = "insert buffer thread";
 
 	os_aio_log_array = os_aio_array_create(n_per_seg, 1);
 
-	srv_io_thread_function[1] = (char*)"log thread";
+	srv_io_thread_function[1] = "log thread";
 
 	os_aio_read_array = os_aio_array_create(n_read_segs * n_per_seg,
 							n_read_segs);
 	for (i = 2; i < 2 + n_read_segs; i++) {
-	        srv_io_thread_function[i] = (char*)"read thread";
+		ut_a(i < SRV_MAX_N_IO_THREADS);
+	        srv_io_thread_function[i] = "read thread";
 	}
 
 	os_aio_write_array = os_aio_array_create(n_write_segs * n_per_seg,
 							n_write_segs);
 	for (i = 2 + n_read_segs; i < n_segments; i++) {
-	        srv_io_thread_function[i] = (char*)"write thread";
+		ut_a(i < SRV_MAX_N_IO_THREADS);
+	        srv_io_thread_function[i] = "write thread";
 	}
 
 	os_aio_sync_array = os_aio_array_create(n_slots_sync, 1);
@@ -1801,7 +1820,7 @@ os_aio_get_array_no(
 
 		return(3);
 	} else {
-		ut_a(0);
+		ut_error;
 
 		return(0);
 	}
@@ -1828,7 +1847,7 @@ os_aio_get_array_from_no(
 
 		return(os_aio_write_array);
 	} else {
-		ut_a(0);
+		ut_error;
 
 		return(NULL);
 	}
@@ -1942,7 +1961,8 @@ loop:
 			SIGRTMIN + 1 + os_aio_get_array_no(array);
 			/* TODO: How to choose the signal numbers? */
 /*
-	printf("AIO signal number %lu\n", (ulint) control->aio_sigevent.sigev_signo);
+	fprintf(stderr, "AIO signal number %lu\n",
+		(ulint) control->aio_sigevent.sigev_signo);
 */
 	control->aio_sigevent.sigev_value.sival_ptr = slot;
 #endif
@@ -2196,7 +2216,7 @@ try_again:
 #elif defined(POSIX_ASYNC_IO)
 			slot->control.aio_lio_opcode = LIO_READ;
 			err = (ulint) aio_read(&(slot->control));
-			printf("Starting Posix aio read %lu\n", err);
+			fprintf(stderr, "Starting POSIX aio read %lu\n", err);
 #endif
 		} else {
 			if (!wake_later) {
@@ -2213,7 +2233,7 @@ try_again:
 #elif defined(POSIX_ASYNC_IO)
 			slot->control.aio_lio_opcode = LIO_WRITE;
 			err = (ulint) aio_write(&(slot->control));
-			printf("Starting Posix aio write %lu\n", err);
+			fprintf(stderr, "Starting POSIX aio write %lu\n", err);
 #endif
 		} else {
 			if (!wake_later) {
@@ -2324,13 +2344,10 @@ os_aio_windows_handle(
 	n = array->n_slots / array->n_segments;
 
 	if (array == os_aio_sync_array) {
-		srv_io_thread_op_info[orig_seg] =
-						"wait Windows aio for 1 page";
 		os_event_wait(os_aio_array_get_nth_slot(array, pos)->event);
 		i = pos;
 	} else {
-		srv_io_thread_op_info[orig_seg] =
-						"wait Windows aio";
+		srv_set_io_thread_op_info(orig_seg, "wait Windows aio");
 		i = os_event_wait_multiple(n,
 				(array->native_events) + segment * n);
 	}
@@ -2341,7 +2358,11 @@ os_aio_windows_handle(
 
 	ut_a(slot->reserved);
 
-	srv_io_thread_op_info[orig_seg] = "get windows aio return value";
+	if (orig_seg != ULINT_UNDEFINED) {
+		srv_set_io_thread_op_info(orig_seg,
+					"get windows aio return value");
+	}
+
 	ret = GetOverlappedResult(slot->file, &(slot->control), &len, TRUE);
 
 	*message1 = slot->message1;
@@ -2408,7 +2429,7 @@ os_aio_posix_handle(
 	pthread_sigmask(0, NULL, &thr_sigset);
 
 	for (i = 32 ; i < 40; i++) {
-	  printf("%lu : %lu %lu\n", (ulint)i,
+		fprintf(stderr, "%lu : %lu %lu\n", (ulint)i,
 		 (ulint)sigismember(&proc_sigset, i),
 		 (ulint)sigismember(&thr_sigset, i));
 	}
@@ -2418,12 +2439,12 @@ os_aio_posix_handle(
 
 	if (sig != SIGRTMIN + 1 + array_no) {
 
-		ut_a(0);
+		ut_error;
 	
 		return(FALSE);
 	}
 	
-	printf("Handling Posix aio\n");
+	fputs("Handling POSIX aio\n", stderr);
 
 	array = os_aio_get_array_from_no(array_no);
 
@@ -2663,8 +2684,8 @@ consecutive_loop:
 			offs += consecutive_ios[i]->len;
 		}
 	}
-
-	srv_io_thread_op_info[global_segment] = (char*) "doing file i/o";
+	
+	srv_set_io_thread_op_info(global_segment, "doing file i/o");
 
 	if (os_aio_print_debug) {
 		fprintf(stderr,
@@ -2676,7 +2697,15 @@ consecutive_loop:
 	/* Do the i/o with ordinary, synchronous i/o functions: */
 	if (slot->type == OS_FILE_WRITE) {
 		if (array == os_aio_write_array) {
-
+			if ((total_len % UNIV_PAGE_SIZE != 0)
+			    || (slot->offset % UNIV_PAGE_SIZE != 0)) {
+				fprintf(stderr,
+"InnoDB: Error: trying a displaced write to %s %lu %lu, len %lu\n",
+					slot->name, slot->offset_high,
+					slot->offset, total_len);
+				ut_error;
+			}
+			  
 			/* Do a 'last millisecond' check that the page end
 			is sensible; reported page checksum errors from
 			Linux seem to wipe over the page end */
@@ -2706,11 +2735,11 @@ consecutive_loop:
 	}
 
 	ut_a(ret);
-	srv_io_thread_op_info[global_segment] = (char*) "file i/o done";
+	srv_set_io_thread_op_info(global_segment, "file i/o done");
 
-/* printf("aio: %lu consecutive %lu:th segment, first offs %lu blocks\n",
-			n_consecutive, global_segment, slot->offset
-					/ UNIV_PAGE_SIZE); */
+/* fprintf(stderr,
+	"aio: %lu consecutive %lu:th segment, first offs %lu blocks\n",
+	n_consecutive, global_segment, slot->offset / UNIV_PAGE_SIZE); */
 
 	if (slot->type == OS_FILE_READ && n_consecutive > 1) {
 		/* Copy the combined buffer to individual buffers */
@@ -2764,8 +2793,7 @@ wait_for_io:
 	os_mutex_exit(array->mutex);
 
 recommended_sleep:
-	srv_io_thread_op_info[global_segment] =
-				(char*)"waiting for i/o request";
+	srv_set_io_thread_op_info(global_segment, "waiting for i/o request");
 
 	os_event_wait(os_aio_segment_wait_events[global_segment]);
 
@@ -2837,8 +2865,7 @@ Prints info of the aio arrays. */
 void
 os_aio_print(
 /*=========*/
-	char*	buf,	/* in/out: buffer where to print */
-	char*	buf_end)/* in: buffer end */
+	FILE*	file)	/* in: file where to print */
 {
 	os_aio_array_t*	array;
 	os_aio_slot_t*	slot;
@@ -2848,18 +2875,13 @@ os_aio_print(
 	double		avg_bytes_read;
 	ulint		i;
 
-	if (buf_end - buf < 1200) {
-
-		return;
-	}
-
 	for (i = 0; i < srv_n_file_io_threads; i++) {
-		buf += sprintf(buf, "I/O thread %lu state: %s (%s)\n", i,
+		fprintf(file, "I/O thread %lu state: %s (%s)\n", i,
 					srv_io_thread_op_info[i],
 					srv_io_thread_function[i]);
 	}
 
-	buf += sprintf(buf, "Pending normal aio reads:");
+	fputs("Pending normal aio reads:", file);
 
 	array = os_aio_read_array;
 loop:
@@ -2877,21 +2899,20 @@ loop:
 	
 		if (slot->reserved) {
 			n_reserved++;
-			/* printf("Reserved slot, messages %lx %lx\n",
-					(ulint)slot->message1,
-					(ulint)slot->message2);
-			*/			ut_a(slot->len > 0);
+			/* fprintf(stderr, "Reserved slot, messages %p %p\n",
+				slot->message1, slot->message2); */
+			ut_a(slot->len > 0);
 		}
 	}
 
 	ut_a(array->n_reserved == n_reserved);
 
-	buf += sprintf(buf, " %lu", n_reserved);
+	fprintf(file, " %lu", n_reserved);
 	
 	os_mutex_exit(array->mutex);
 
 	if (array == os_aio_read_array) {
-		buf += sprintf(buf, ", aio writes:");
+		fputs(", aio writes:", file);
 	
 		array = os_aio_write_array;
 
@@ -2899,40 +2920,38 @@ loop:
 	}
 
 	if (array == os_aio_write_array) {
-		buf += sprintf(buf, ",\n ibuf aio reads:");
+		fputs(",\n ibuf aio reads:", file);
 		array = os_aio_ibuf_array;
 
 		goto loop;
 	}
 
 	if (array == os_aio_ibuf_array) {
-		buf += sprintf(buf, ", log i/o's:");
+		fputs(", log i/o's:", file);
 		array = os_aio_log_array;
 
 		goto loop;
 	}
 
 	if (array == os_aio_log_array) {
-		buf += sprintf(buf, ", sync i/o's:");		
+		fputs(", sync i/o's:", file);
 		array = os_aio_sync_array;
 
 		goto loop;
 	}
 
-	buf += sprintf(buf, "\n");
-	
+	putc('\n', file);
 	current_time = time(NULL);
 	time_elapsed = 0.001 + difftime(current_time, os_last_printout);
 
-	buf += sprintf(buf,
-		"Pending flushes (fsync) log: %lu; buffer pool: %lu\n",
-	       fil_n_pending_log_flushes, fil_n_pending_tablespace_flushes);
-	buf += sprintf(buf,
+	fprintf(file,
+		"Pending flushes (fsync) log: %lu; buffer pool: %lu\n"
 		"%lu OS file reads, %lu OS file writes, %lu OS fsyncs\n",
+		fil_n_pending_log_flushes, fil_n_pending_tablespace_flushes,
 		os_n_file_reads, os_n_file_writes, os_n_fsyncs);
 
 	if (os_file_n_pending_preads != 0 || os_file_n_pending_pwrites != 0) {
-	        buf += sprintf(buf,
+		fprintf(file,
 		    "%lu pending preads, %lu pending pwrites\n",
 		    os_file_n_pending_preads, os_file_n_pending_pwrites);
 	}
@@ -2944,7 +2963,7 @@ loop:
 				(os_n_file_reads - os_n_file_reads_old);
 	}
 
-	buf += sprintf(buf,
+	fprintf(file,
 "%.2f reads/s, %lu avg bytes/read, %.2f writes/s, %.2f fsyncs/s\n",
 		(os_n_file_reads - os_n_file_reads_old)
 		/ time_elapsed,
