@@ -77,8 +77,8 @@ index record. This is only used in foreign key checks and we can assume
 that index does not contain column prefixes. */
 static
 ibool
-row_upd_changes_first_fields(
-/*=========================*/
+row_upd_changes_first_fields_binary(
+/*================================*/
 				/* out: TRUE if changes */
 	dtuple_t*	entry,	/* in: old value of index entry */
 	dict_index_t*	index,	/* in: index of entry */
@@ -196,7 +196,7 @@ row_upd_check_references_constraints(
 
 		if (foreign->referenced_index == index
 		    && (node->is_delete
-		       || row_upd_changes_first_fields(entry, index,
+		       || row_upd_changes_first_fields_binary(entry, index,
 			    		node->update, foreign->n_fields))) {
 			    				
 			if (foreign->foreign_table == NULL) {
@@ -287,7 +287,9 @@ upd_node_create(
 	node->select = NULL;
 	
 	node->heap = mem_heap_create(128);
-	node->magic_n = UPD_NODE_MAGIC_N;	
+#ifdef UNIV_DEBUG
+	node->magic_n = UPD_NODE_MAGIC_N;
+#endif /* UNIV_DEBUG */
 
 	node->cmpl_info = 0;
 	
@@ -361,8 +363,8 @@ row_upd_changes_field_size_or_external(
 				/* out: TRUE if the update changes the size of
 				some field in index or the field is external
 				in rec or update */
-	rec_t*		rec,	/* in: record in clustered index */
-	dict_index_t*	index,	/* in: clustered index */
+	rec_t*		rec,	/* in: record in index */
+	dict_index_t*	index,	/* in: index */
 	upd_t*		update)	/* in: update vector */
 {
 	upd_field_t*	upd_field;
@@ -372,8 +374,6 @@ row_upd_changes_field_size_or_external(
 	ulint		n_fields;
 	ulint		i;
 
-	ut_ad(index->type & DICT_CLUSTERED);
-
 	n_fields = upd_get_n_fields(update);
 
 	for (i = 0; i < n_fields; i++) {
@@ -698,7 +698,7 @@ row_upd_build_sec_rec_difference_binary(
 	ulint		i;
 
 	/* This function is used only for a secondary index */
-	ut_ad(0 == (index->type & DICT_CLUSTERED));
+	ut_a(0 == (index->type & DICT_CLUSTERED));
 
 	update = upd_create(dtuple_get_n_fields(entry), heap);
 
@@ -710,7 +710,13 @@ row_upd_build_sec_rec_difference_binary(
 
 		dfield = dtuple_get_nth_field(entry, i);
 
-		ut_a(len == dfield_get_len(dfield));
+		/* NOTE that it may be that len != dfield_get_len(dfield) if we
+		are updating in a character set and collation where strings of
+		different length can be equal in an alphabetical comparison,
+		and also in the case where we have a column prefix index
+		and the last characters in the index field are spaces; the
+		latter case probably caused the assertion failures reported at
+		row0upd.c line 713 in versions 4.0.14 - 4.0.16. */
 
 		/* NOTE: we compare the fields as binary strings!
 		(No collation) */
@@ -819,13 +825,77 @@ skip_compare:
 Replaces the new column values stored in the update vector to the index entry
 given. */
 
+void
+row_upd_index_replace_new_col_vals_index_pos(
+/*=========================================*/
+	dtuple_t*	entry,	/* in/out: index entry where replaced */
+	dict_index_t*	index,	/* in: index; NOTE that this may also be a
+				non-clustered index */
+	upd_t*		update,	/* in: an update vector built for the index so
+				that the field number in an upd_field is the
+				index position */
+	mem_heap_t*	heap)	/* in: memory heap to which we allocate and
+				copy the new values, set this as NULL if you
+				do not want allocation */
+{
+	dict_field_t*	field;
+	upd_field_t*	upd_field;
+	dfield_t*	dfield;
+	dfield_t*	new_val;
+	ulint		j;
+	ulint		i;
+
+	ut_ad(index);
+
+	dtuple_set_info_bits(entry, update->info_bits);
+
+	for (j = 0; j < dict_index_get_n_fields(index); j++) {
+
+	        field = dict_index_get_nth_field(index, j);
+
+		for (i = 0; i < upd_get_n_fields(update); i++) {
+
+		        upd_field = upd_get_nth_field(update, i);
+
+			if (upd_field->field_no == j) {
+
+			        dfield = dtuple_get_nth_field(entry, j);
+
+				new_val = &(upd_field->new_val);
+
+				dfield_set_data(dfield, new_val->data,
+								new_val->len);
+				if (heap && new_val->len != UNIV_SQL_NULL) {
+				        dfield->data = mem_heap_alloc(heap,
+								new_val->len);
+					ut_memcpy(dfield->data, new_val->data,
+								new_val->len);
+				}
+
+				if (field->prefix_len > 0
+			            && new_val->len != UNIV_SQL_NULL
+			            && new_val->len > field->prefix_len) {
+
+				        dfield->len = field->prefix_len;
+				}
+			}
+		}
+	}
+}
+
+/***************************************************************
+Replaces the new column values stored in the update vector to the index entry
+given. */
+
 void
 row_upd_index_replace_new_col_vals(
 /*===============================*/
 	dtuple_t*	entry,	/* in/out: index entry where replaced */
 	dict_index_t*	index,	/* in: index; NOTE that this may also be a
 				non-clustered index */
-	upd_t*		update,	/* in: update vector */
+	upd_t*		update,	/* in: an update vector built for the
+				CLUSTERED index so that the field number in
+				an upd_field is the clustered index position */
 	mem_heap_t*	heap)	/* in: memory heap to which we allocate and
 				copy the new values, set this as NULL if you
 				do not want allocation */
@@ -893,7 +963,9 @@ row_upd_changes_ord_field_binary(
 				known when this function is called, e.g., at
 				compile time */
 	dict_index_t*	index,	/* in: index of the record */
-	upd_t*		update)	/* in: update vector for the row */
+	upd_t*		update)	/* in: update vector for the row; NOTE: the
+				field numbers in this MUST be clustered index
+				positions! */
 {
 	upd_field_t*	upd_field;
 	dict_field_t*	ind_field;
@@ -978,8 +1050,8 @@ index record. This is only used in foreign key checks and we can assume
 that index does not contain column prefixes. */
 static
 ibool
-row_upd_changes_first_fields(
-/*=========================*/
+row_upd_changes_first_fields_binary(
+/*================================*/
 				/* out: TRUE if changes */
 	dtuple_t*	entry,	/* in: index entry */
 	dict_index_t*	index,	/* in: index of entry */
@@ -1004,15 +1076,16 @@ row_upd_changes_first_fields(
 		col = dict_field_get_col(ind_field);
 		col_pos = dict_col_get_clust_pos(col);
 
+		ut_a(ind_field->prefix_len == 0);
+
 		for (j = 0; j < n_upd_fields; j++) {
 
 			upd_field = upd_get_nth_field(update, j);
 
 			if (col_pos == upd_field->field_no
-			    && (ind_field->prefix_len > 0
-			        || 0 != cmp_dfield_dfield(
+			    && !dfield_datas_are_binary_equal(
 					     dtuple_get_nth_field(entry, i),
-					     &(upd_field->new_val)))) {
+					     &(upd_field->new_val))) {
 				return(TRUE);
 			}
 		}
@@ -1129,7 +1202,6 @@ row_upd_sec_index_entry(
 	rec_t*		rec;
 	ulint		err	= DB_SUCCESS;
 	mtr_t		mtr;
-	char           	err_buf[1000];
 	
 	index = node->index;
 	
@@ -1150,21 +1222,21 @@ row_upd_sec_index_entry(
 	rec = btr_cur_get_rec(btr_cur);
 
 	if (!found) {
-	  	fprintf(stderr, "InnoDB: error in sec index entry update in\n"
-		  	"InnoDB: index %s table %s\n", index->name,
-		  	index->table->name);
-	  	dtuple_sprintf(err_buf, 900, entry);
-	  	fprintf(stderr, "InnoDB: tuple %s\n", err_buf);
+		fputs("InnoDB: error in sec index entry update in\n"
+			"InnoDB: ", stderr);
+		dict_index_name_print(stderr, index);
+		fputs("\n"
+			"InnoDB: tuple ", stderr);
+		dtuple_print(stderr, entry);
+		fputs("\n"
+			"InnoDB: record ", stderr);
+		rec_print(stderr, rec);
+		putc('\n', stderr);
 
-	  	rec_sprintf(err_buf, 900, rec);
-	  	fprintf(stderr, "InnoDB: record %s\n", err_buf);
+		trx_print(stderr, thr_get_trx(thr));
 
-		trx_print(err_buf, thr_get_trx(thr));
-
-	  	fprintf(stderr,
-		"%s\nInnoDB: Make a detailed bug report and send it\n",
-							err_buf);
-	  	fprintf(stderr, "InnoDB: to mysql@lists.mysql.com\n");
+		fputs("\n"
+"InnoDB: Submit a detailed bug report to http://bugs.mysql.com\n", stderr);
 	} else {
  	  	/* Delete mark the old index record; it can already be
           	delete marked if we return after a lock wait in
@@ -1733,6 +1805,7 @@ row_upd_step(
 	trx_start_if_not_started(trx);
 
 	node = thr->run_node;
+	ut_ad(node->magic_n == UPD_NODE_MAGIC_N);
 	
 	sel_node = node->select;
 
@@ -1852,6 +1925,7 @@ row_upd_in_place_in_select(
 
 	node = que_node_get_parent(sel_node);
 
+	ut_ad(node->magic_n == UPD_NODE_MAGIC_N);
 	ut_ad(que_node_get_type(node) == QUE_NODE_UPDATE);
 
 	pcur = node->pcur;
