@@ -64,8 +64,8 @@ uint		mysql_port=0;
 my_string	mysql_unix_port=0;
 ulong 		net_buffer_length=8192;
 ulong		max_allowed_packet= 1024L*1024L*1024L;
-ulong		net_read_timeout=  NET_READ_TIMEOUT;
-ulong		net_write_timeout= NET_WRITE_TIMEOUT;
+ulong		net_read_timeout=  CLIENT_NET_READ_TIMEOUT;
+ulong		net_write_timeout= CLIENT_NET_WRITE_TIMEOUT;
 
 #define CLIENT_CAPABILITIES	(CLIENT_LONG_PASSWORD | CLIENT_LONG_FLAG | CLIENT_TRANSACTIONS)
 
@@ -102,8 +102,7 @@ int STDCALL mysql_server_init(int argc __attribute__((unused)),
 			      char **argv __attribute__((unused)),
 			      char **groups __attribute__((unused)))
 {
-  mysql_once_init();
-  return 0;
+  return (int) mysql_once_init();
 }
 
 void STDCALL mysql_server_end()
@@ -317,7 +316,7 @@ HANDLE create_named_pipe(NET *net, uint connect_timeout, char **arg_host,
 			 char **arg_unix_socket)
 {
   HANDLE hPipe=INVALID_HANDLE_VALUE;
-  char szPipeName [ 257 ];
+  char pipe_name[1024];
   DWORD dwMode;
   int i;
   my_bool testing_named_pipes=0;
@@ -328,13 +327,15 @@ HANDLE create_named_pipe(NET *net, uint connect_timeout, char **arg_host,
   if (!host || !strcmp(host,LOCAL_HOST))
     host=LOCAL_HOST_NAMEDPIPE;
 
-  sprintf( szPipeName, "\\\\%s\\pipe\\%s", host, unix_socket);
-  DBUG_PRINT("info",("Server name: '%s'.  Named Pipe: %s",
-		     host, unix_socket));
+  
+  pipe_name[sizeof(pipe_name)-1]= 0;		/* Safety if too long string */
+  strxnmov(pipe_name, sizeof(pipe_name)-1, "\\\\", host, "\\pipe\\",
+	   unix_socket, NullS);
+  DBUG_PRINT("info",("Server name: '%s'.  Named Pipe: %s", host, unix_socket));
 
   for (i=0 ; i < 100 ; i++)			/* Don't retry forever */
   {
-    if ((hPipe = CreateFile(szPipeName,
+    if ((hPipe = CreateFile(pipe_name,
 			    GENERIC_READ | GENERIC_WRITE,
 			    0,
 			    NULL,
@@ -350,7 +351,7 @@ HANDLE create_named_pipe(NET *net, uint connect_timeout, char **arg_host,
       return INVALID_HANDLE_VALUE;
     }
     /* wait for for an other instance */
-    if (! WaitNamedPipe(szPipeName, connect_timeout*1000) )
+    if (! WaitNamedPipe(pipe_name, connect_timeout*1000) )
     {
       net->last_errno=CR_NAMEDPIPEWAIT_ERROR;
       sprintf(net->last_error,ER(net->last_errno),host, unix_socket,
@@ -586,7 +587,7 @@ char* getlogin(void);
 
 
 #if defined(__NETWARE__)
-/* default to "root" on NetWare */
+/* Default to value of USER on NetWare, if unset use "UNKNOWN_USER" */
 static void read_user_name(char *name)
 {
   char *str=getenv("USER");
@@ -695,7 +696,8 @@ mysql_debug(const char *debug __attribute__((unused)))
 #else
     {
       char buff[80];
-      strmov(strmov(buff,"libmysql: "),env);
+      buff[sizeof(buff)-1]= 0;
+      strxnmov(buff,sizeof(buff)-1,"libmysql: ", env, NullS);
       MessageBox((HWND) 0,"Debugging variable MYSQL_DEBUG used",buff,MB_OK);
     }
 #endif
@@ -941,7 +943,8 @@ static void mysql_read_default_options(struct st_mysql_options *options,
 	  options->rpl_parse= 1;
 	  break;
 	case 27:
-	  options->max_allowed_packet= atoi(opt_arg);
+          if (opt_arg)
+            options->max_allowed_packet= atoi(opt_arg);
 	  break;
 	default:
 	  DBUG_PRINT("warning",("unknown option: %s",option[0]));
@@ -1435,7 +1438,8 @@ STDCALL mysql_rpl_query_type(const char* q, int len)
 MYSQL * STDCALL
 mysql_init(MYSQL *mysql)
 {
-  mysql_once_init();
+  if (mysql_once_init())
+    return 0;
   if (!mysql)
   {
     if (!(mysql=(MYSQL*) my_malloc(sizeof(*mysql),MYF(MY_WME | MY_ZEROFILL))))
@@ -1452,10 +1456,6 @@ mysql_init(MYSQL *mysql)
     after we return if this is not the case.
   */
   mysql->rpl_pivot = 1; 
-#if defined(SIGPIPE) && defined(THREAD) && !defined(__WIN__)
-  if (!((mysql)->client_flag & CLIENT_IGNORE_SIGPIPE))
-    (void) signal(SIGPIPE,pipe_sig_handler);
-#endif
 
 /*
   Only enable LOAD DATA INFILE by default if configured with
@@ -1479,15 +1479,20 @@ mysql_init(MYSQL *mysql)
     This function is called by mysql_init() and indirectly called
     by mysql_query(), so one should never have to call this from an
     outside program.
+
+  RETURN
+    0  ok
+    1  could not initialize environment (out of memory or thread keys)
 */
 
-void mysql_once_init(void)
+int mysql_once_init(void)
 {
   if (!mysql_client_init)
   {
     mysql_client_init=1;
     org_my_init_done=my_init_done;
-    my_init();					/* Will init threads */
+    if (my_init())				/* Will init threads */
+      return 1;
     init_client_errs();
     if (!mysql_port)
     {
@@ -1515,16 +1520,21 @@ void mysql_once_init(void)
 	mysql_unix_port = env;
     }
     mysql_debug(NullS);
-#if defined(SIGPIPE) && !defined(THREAD) && !defined(__WIN__)
-    (void) signal(SIGPIPE,SIG_IGN);
+#if defined(SIGPIPE) && !defined(__WIN__)
+    (void) signal(SIGPIPE, SIG_IGN);
 #endif
   }
 #ifdef THREAD
   else
-    my_thread_init();         /* Init if new thread */
+  {
+    if (my_thread_init())         /* Init if new thread */
+      return 1;
+  }
 #endif
+  return 0;
 }
 
+
 /**************************************************************************
   Fill in SSL part of MYSQL structure and set 'use_ssl' flag.
   NB! Errors are not reported until you do mysql_real_connect.
@@ -1602,6 +1612,56 @@ mysql_connect(MYSQL *mysql,const char *host,
 #endif
 
 
+#ifdef CHECK_LICENSE
+/*
+  Check server side variable 'license'.
+  If the variable does not exist or does not contain 'Commercial', 
+  we're talking to non-commercial server from commercial client.
+  SYNOPSIS
+    check_license()
+  RETURN VALUE
+    0  success
+   !0  network error or the server is not commercial.
+       Error code is saved in mysql->net.last_errno.
+*/
+
+static int check_license(MYSQL *mysql)
+{
+  MYSQL_ROW row;
+  MYSQL_RES *res;
+  NET *net= &mysql->net;
+  static const char query[]= "SELECT @@license";
+  static const char required_license[]= LICENSE;
+
+  if (mysql_real_query(mysql, query, sizeof(query)-1))
+  {
+    if (net->last_errno == ER_UNKNOWN_SYSTEM_VARIABLE)
+    {
+      net->last_errno= CR_WRONG_LICENSE;
+      sprintf(net->last_error, ER(net->last_errno), required_license);
+    }
+    return 1;
+  }
+  if (!(res= mysql_use_result(mysql)))
+    return 1;
+  row= mysql_fetch_row(res);
+  /* 
+    If no rows in result set, or column value is NULL (none of these
+    two is ever true for server variables now), or column value
+    mismatch, set wrong license error.
+  */
+  if (!net->last_errno &&
+      (!row || !row[0] ||
+       strncmp(row[0], required_license, sizeof(required_license))))
+  {
+    net->last_errno= CR_WRONG_LICENSE;
+    sprintf(net->last_error, ER(net->last_errno), required_license);
+  }
+  mysql_free_result(res);
+  return net->last_errno;
+}
+#endif /* CHECK_LICENSE */
+
 /*
   The following union is used to force a struct to be double allgined.
   This is to avoid warings with gethostname_r() on Linux itanium systems
@@ -1627,7 +1687,7 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
   char		buff[NAME_LEN+USERNAME_LENGTH+100],charset_name_buff[16];
   char		*end,*host_info,*charset_name;
   my_socket	sock;
-  uint32	ip_addr;
+  in_addr_t	ip_addr;
   struct	sockaddr_in sock_addr;
   ulong		pkt_length;
   NET		*net= &mysql->net;
@@ -1739,7 +1799,7 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
 	    (unix_socket && !strcmp(unix_socket,MYSQL_NAMEDPIPE)))
 	{
 	  net->last_errno= CR_SERVER_LOST;
-	  strmov(net->last_error,ER(net->last_errno));    
+	  strmov(net->last_error,ER(net->last_errno));
 	  goto error;		/* User only requested named pipes */
 	}
 	/* Try also with TCP/IP */
@@ -1825,7 +1885,7 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
       vio_poll_read(net->vio, mysql->options.connect_timeout))
   {
     net->last_errno= CR_SERVER_LOST;
-    strmov(net->last_error,ER(net->last_errno));    
+    strmov(net->last_error,ER(net->last_errno));
     goto error;
   }
   if ((pkt_length=net_safe_read(mysql)) == packet_error)
@@ -1977,7 +2037,7 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
     if (my_net_write(net,buff,(uint) (2)) || net_flush(net))
     {
       net->last_errno= CR_SERVER_LOST;
-      strmov(net->last_error,ER(net->last_errno));    
+      strmov(net->last_error,ER(net->last_errno));
       goto error;
     }
     /* Do the SSL layering. */
@@ -1989,7 +2049,7 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
 				       options->ssl_cipher)))
     {
       net->last_errno= CR_SSL_CONNECTION_ERROR;
-      strmov(net->last_error,ER(net->last_errno));    
+      strmov(net->last_error,ER(net->last_errno));
       goto error;
     }
     DBUG_PRINT("info", ("IO layer change in progress..."));
@@ -2029,7 +2089,7 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
   if (my_net_write(net,buff,(ulong) (end-buff)) || net_flush(net))
   {
     net->last_errno= CR_SERVER_LOST;
-    strmov(net->last_error,ER(net->last_errno));    
+    strmov(net->last_error,ER(net->last_errno));
     goto error;
   }
   if (net_safe_read(mysql) == packet_error)
@@ -2038,6 +2098,10 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
     net->compress=1;
   if (mysql->options.max_allowed_packet)
     net->max_packet_size= mysql->options.max_allowed_packet;
+#ifdef CHECK_LICENSE 
+  if (check_license(mysql))
+    goto error;
+#endif
   if (db && mysql_select_db(mysql,db))
     goto error;
   if (mysql->options.init_command)
@@ -2102,8 +2166,8 @@ static my_bool mysql_reconnect(MYSQL *mysql)
   {
    /* Allow reconnect next time */
     mysql->server_status&= ~SERVER_STATUS_IN_TRANS;
-    mysql->net.last_errno=CR_SERVER_GONE_ERROR;
-    strmov(mysql->net.last_error,ER(mysql->net.last_errno));
+    mysql->net.last_errno= CR_SERVER_GONE_ERROR;
+    strmov(mysql->net.last_error, ER(mysql->net.last_errno));
     DBUG_RETURN(1);
   }
   mysql_init(&tmp_mysql);
@@ -2260,29 +2324,33 @@ static MYSQL* spawn_init(MYSQL* parent, const char* host,
 			 const char* passwd)
 {
   MYSQL* child;
-  if (!(child = mysql_init(0)))
-    return 0;
+  DBUG_ENTER("spawn_init");
+  if (!(child= mysql_init(0)))
+    DBUG_RETURN(0);
 
-  child->options.user = my_strdup((user) ? user :
-				  (parent->user ? parent->user :
-				   parent->options.user), MYF(0));
-  child->options.password = my_strdup((passwd) ? passwd :
-				      (parent->passwd ?
-				       parent->passwd :
-				       parent->options.password), MYF(0));
-  child->options.port = port;
-  child->options.host = my_strdup((host) ? host :
-				  (parent->host ?
-				   parent->host :
-				   parent->options.host), MYF(0));
+  child->options.user= my_strdup((user) ? user :
+				 (parent->user ? parent->user :
+				  parent->options.user), MYF(0));
+  child->options.password= my_strdup((passwd) ? passwd :
+				     (parent->passwd ?
+				      parent->passwd :
+				      parent->options.password), MYF(0));
+  child->options.port= port;
+  child->options.host= my_strdup((host) ? host :
+				 (parent->host ?
+				  parent->host :
+				  parent->options.host), MYF(0));
   if (parent->db)
-    child->options.db = my_strdup(parent->db, MYF(0));
+    child->options.db= my_strdup(parent->db, MYF(0));
   else if (parent->options.db)
-    child->options.db = my_strdup(parent->options.db, MYF(0));
+    child->options.db= my_strdup(parent->options.db, MYF(0));
 
-  child->options.rpl_parse = child->options.rpl_probe = child->rpl_pivot = 0;
-
-  return child;
+  /*
+    rpl_pivot is set to 1 in mysql_init();  Reset it as we are not doing
+    replication here
+  */
+  child->rpl_pivot= 0;
+  DBUG_RETURN(child);
 }
 
 
@@ -2295,9 +2363,6 @@ STDCALL mysql_set_master(MYSQL* mysql, const char* host,
     mysql_close(mysql->master);
   if (!(mysql->master = spawn_init(mysql, host, port, user, passwd)))
     return 1;
-  mysql->master->rpl_pivot = 0;
-  mysql->master->options.rpl_parse = 0;
-  mysql->master->options.rpl_probe = 0;
   return 0;
 }
 
@@ -2911,6 +2976,11 @@ mysql_get_client_info(void)
   return (char*) MYSQL_SERVER_VERSION;
 }
 
+ulong STDCALL mysql_get_client_version(void)
+{
+  return MYSQL_VERSION_ID;
+}
+
 
 int STDCALL
 mysql_options(MYSQL *mysql,enum mysql_option option, const char *arg)
