@@ -224,6 +224,7 @@ static bool check_user(THD *thd,enum_server_command command, const char *user,
     send_error(net,ER_OUT_OF_RESOURCES);
     return 1;
   }
+  strmake(thd->priv_host, LOCAL_HOST, sizeof(thd->priv_host)-1);
   thd->master_access=acl_getroot(thd, thd->host, thd->ip, thd->user,
 				 passwd, thd->scramble,
                                  &thd->priv_user, thd->priv_host,
@@ -276,6 +277,12 @@ static bool check_user(THD *thd,enum_server_command command, const char *user,
 }
 
 
+void THD::clear_error()
+{
+  net.last_error[0]= 0;
+  net.last_errno= 0;
+}
+
 /*
   Make a copy of array and the strings array points to
 */
@@ -306,7 +313,8 @@ char **copy_arguments(int argc, char **argv)
 extern "C"
 {
 
-static my_bool inited, org_my_init_done;
+static my_bool  org_my_init_done;
+my_bool         server_inited;
 ulong		max_allowed_packet, net_buffer_length;
 char **		copy_arguments_ptr= 0; 
 
@@ -338,9 +346,9 @@ int STDCALL mysql_server_init(int argc, char **argv, char **groups)
   my_umask_dir=0700;		// Default umask for new directories
 
   /* Only call MY_INIT() if it hasn't been called before */
-  if (!inited)
+  if (!server_inited)
   {
-    inited=1;
+    server_inited=1;
     org_my_init_done=my_init_done;
   }
   if (!org_my_init_done)
@@ -620,11 +628,6 @@ void STDCALL mysql_server_end()
   my_free((char*) copy_arguments_ptr, MYF(MY_ALLOW_ZERO_PTR));
   copy_arguments_ptr=0;
   clean_up(0);
-#ifdef THREAD
-  /* Don't call my_thread_end() if the application is using MY_INIT() */
-  if (!org_my_init_done)
-    my_thread_end();
-#endif
   /* If library called my_init(), free memory allocated by it */
   if (!org_my_init_done)
     my_end(0);
