@@ -802,6 +802,17 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
 		      db ? db : "(Null)",
 		      user ? user : "(Null)"));
 
+  /* 
+     Check mysql_server_init was called.
+     This code shouldn't be merged to 4.1
+  */
+  if (!server_inited)
+  {
+    net->last_errno=CR_UNKNOWN_ERROR;
+    strmov(net->last_error,ER(net->last_errno));
+    goto error;
+  }
+
   net->vio = 0;				/* If something goes wrong */
   /* use default options */
   if (mysql->options.my_cnf_file || mysql->options.my_cnf_group)
