@@ -183,7 +183,7 @@ my_bool acl_init(THD *org_thd, bool dont_read_acl_tables)
 		    thd->net.last_error);
     goto end;
   }
-  init_sql_alloc(&mem,1024,0);
+  init_sql_alloc(&mem, ACL_ALLOC_BLOCK_SIZE, 0);
   init_read_record(&read_record_info,thd,table= tables[0].table,NULL,1,0);
   VOID(my_init_dynamic_array(&acl_hosts,sizeof(ACL_HOST),20,50));
   while (!(read_record_info.read_record(&read_record_info)))
@@ -588,6 +588,11 @@ ulong acl_getroot(THD *thd, const char *host, const char *ip, const char *user,
 	      /* Prepare certificate (if exists) */
 	      DBUG_PRINT("info",("checkpoint 1"));
 	      X509* cert=SSL_get_peer_certificate(ssl);
+	      if (!cert)
+	      {
+		user_access=NO_ACCESS;
+		break;
+	      }
 	      DBUG_PRINT("info",("checkpoint 2"));
 	      /* If X509 issuer is speified, we check it... */
 	      if (acl_user->x509_issuer)
@@ -735,12 +740,12 @@ static void acl_insert_user(const char *user, const char *host,
 {
   ACL_USER acl_user;
   acl_user.user=*user ? strdup_root(&mem,user) : 0;
-  update_hostname(&acl_user.host,strdup_root(&mem,host));
+  update_hostname(&acl_user.host, *host ? strdup_root(&mem,host): 0);
   acl_user.password=0;
   acl_user.access=privileges;
   acl_user.user_resource = *mqh;
   acl_user.sort=get_sort(2,acl_user.host.hostname,acl_user.user);
-  acl_user.hostname_length=(uint) strlen(acl_user.host.hostname);
+  acl_user.hostname_length=(uint) strlen(host);
   acl_user.ssl_type= (ssl_type != SSL_TYPE_NOT_SPECIFIED ?
 		      ssl_type : SSL_TYPE_NONE);
   acl_user.ssl_cipher=	ssl_cipher   ? strdup_root(&mem,ssl_cipher) : 0;
@@ -1125,6 +1130,7 @@ bool change_password(THD *thd, const char *host, const char *user,
 		acl_user->user ? acl_user->user : "",
 		acl_user->host.hostname ? acl_user->host.hostname : "",
 		new_password));
+  thd->clear_error();
   mysql_update_log.write(thd, buff, query_length);
   Query_log_event qinfo(thd, buff, query_length, 0);
   mysql_bin_log.write(&qinfo);
@@ -1197,11 +1203,11 @@ static const char *calc_ip(const char *ip, long *val, char end)
 static void update_hostname(acl_host_and_ip *host, const char *hostname)
 {
   host->hostname=(char*) hostname;		// This will not be modified!
-  if (hostname &&
+  if (!hostname ||
       (!(hostname=calc_ip(hostname,&host->ip,'/')) ||
        !(hostname=calc_ip(hostname+1,&host->ip_mask,'\0'))))
   {
-    host->ip=host->ip_mask=0;			// Not a masked ip
+    host->ip= host->ip_mask=0;			// Not a masked ip
   }
 }
 
@@ -1605,106 +1611,123 @@ static byte* get_key_column(GRANT_COLUMN *buff,uint *length,
 class GRANT_TABLE :public Sql_alloc
 {
 public:
-  char *host,*db,*user,*tname, *hash_key;
+  char *host,*db,*user,*tname, *hash_key, *orig_host;
   ulong privs, cols;
+  ulong sort;
   uint key_length;
   HASH hash_columns;
-  GRANT_TABLE (const char *h, const char *d,const char *u, const char *t,
-	       ulong p, ulong c)
-    : privs(p), cols(c)
-  {
-    host = strdup_root(&memex,h);
-    db =   strdup_root(&memex,d);
-    user = strdup_root(&memex,u);
-    tname= strdup_root(&memex,t);
-    if (lower_case_table_names)
-    {
-      casedn_str(db);
-      casedn_str(tname);
-    }
-    key_length =(uint) strlen(d)+(uint) strlen(u)+(uint) strlen(t)+3;
-    hash_key = (char*) alloc_root(&memex,key_length);
-    strmov(strmov(strmov(hash_key,user)+1,db)+1,tname);
-    (void) hash_init(&hash_columns,0,0,0, (hash_get_key) get_key_column,0,
-		     HASH_CASE_INSENSITIVE);
-  }
-
-  GRANT_TABLE (TABLE *form, TABLE *col_privs)
-  {
-    byte key[MAX_KEY_LENGTH];
-
-    host =  get_field(&memex,form,0);
-    db =    get_field(&memex,form,1);
-    user =  get_field(&memex,form,2);
-    if (!user)
-      user=(char*) "";
-    tname = get_field(&memex,form,3);
-    if (!host || !db || !tname)
-    {
-      /* Wrong table row; Ignore it */
-      privs = cols = 0;				/* purecov: inspected */
-      return;					/* purecov: inspected */
-    }
-    if (lower_case_table_names)
-    {
-      casedn_str(db);
-      casedn_str(tname);
-    }
-    key_length = ((uint) strlen(db) + (uint) strlen(user) +
-		  (uint) strlen(tname) + 3);
-    hash_key = (char*) alloc_root(&memex,key_length);
-    strmov(strmov(strmov(hash_key,user)+1,db)+1,tname);
-    privs = (ulong) form->field[6]->val_int();
-    cols  = (ulong) form->field[7]->val_int();
-    privs = fix_rights_for_table(privs);
-    cols =  fix_rights_for_column(cols);
-
-    (void) hash_init(&hash_columns,0,0,0, (hash_get_key) get_key_column,0,
-		     HASH_CASE_INSENSITIVE);
-    if (cols)
-    {
-      int key_len;
-      col_privs->field[0]->store(host,(uint) strlen(host));
-      col_privs->field[1]->store(db,(uint) strlen(db));
-      col_privs->field[2]->store(user,(uint) strlen(user));
-      col_privs->field[3]->store(tname,(uint) strlen(tname));
-      key_len=(col_privs->field[0]->pack_length()+
-	       col_privs->field[1]->pack_length()+
-	       col_privs->field[2]->pack_length()+
-	       col_privs->field[3]->pack_length());
-      key_copy(key,col_privs,0,key_len);
-      col_privs->field[4]->store("",0);
-      col_privs->file->index_init(0);
-      if (col_privs->file->index_read(col_privs->record[0],
-				      (byte*) col_privs->field[0]->ptr,
-				      key_len, HA_READ_KEY_EXACT))
-      {
-	cols = 0; /* purecov: deadcode */
-	return;
-      }
-      do
-      {
-	String *res,column_name;
-	GRANT_COLUMN *mem_check;
-	/* As column name is a string, we don't have to supply a buffer */
-	res=col_privs->field[4]->val_str(&column_name,&column_name);
-	ulong priv= (ulong) col_privs->field[6]->val_int();
-	if (!(mem_check = new GRANT_COLUMN(*res,
-					   fix_rights_for_column(priv))))
-	{
-	  /* Don't use this entry */
-	  privs = cols = 0;			/* purecov: deadcode */
-	  return;				/* purecov: deadcode */
-	}
-	hash_insert(&hash_columns, (byte *) mem_check);
-      } while (!col_privs->file->index_next(col_privs->record[0]) &&
-	       !key_cmp(col_privs,key,0,key_len));
-    }
-  }
+  GRANT_TABLE(const char *h, const char *d,const char *u, const char *t,
+              ulong p, ulong c);
+  GRANT_TABLE(TABLE *form, TABLE *col_privs);
   bool ok() { return privs != 0 || cols != 0; }
 };
 
 
+GRANT_TABLE::GRANT_TABLE(const char *h, const char *d,const char *u,
+                         const char *t, ulong p, ulong c)
+  :privs(p), cols(c)
+{
+  /* Host given by user */
+  orig_host=  strdup_root(&memex,h);
+  /* Convert empty hostname to '%' for easy comparision */
+  host=  orig_host[0] ? orig_host : (char*) "%";
+  db =   strdup_root(&memex,d);
+  user = strdup_root(&memex,u);
+  sort=  get_sort(3,host,db,user);
+  tname= strdup_root(&memex,t);
+  if (lower_case_table_names)
+  {
+    casedn_str(db);
+    casedn_str(tname);
+  }
+  key_length =(uint) strlen(d)+(uint) strlen(u)+(uint) strlen(t)+3;
+  hash_key = (char*) alloc_root(&memex,key_length);
+  strmov(strmov(strmov(hash_key,user)+1,db)+1,tname);
+  (void) hash_init(&hash_columns,0,0,0, (hash_get_key) get_key_column,0,
+                   HASH_CASE_INSENSITIVE);
+}
+
+
+GRANT_TABLE::GRANT_TABLE(TABLE *form, TABLE *col_privs)
+{
+  byte key[MAX_KEY_LENGTH];
+
+  orig_host= host= get_field(&memex,form,0);
+  db =    get_field(&memex,form,1);
+  user =  get_field(&memex,form,2);
+  if (!user)
+    user= (char*) "";
+  if (!orig_host)
+  {
+    orig_host= (char*) "";
+    host= (char*) "%";
+  }
+  sort=   get_sort(3,orig_host,db,user);
+  tname = get_field(&memex,form,3);
+  if (!db || !tname)
+  {
+    /* Wrong table row; Ignore it */
+    privs = cols = 0;				/* purecov: inspected */
+    return;					/* purecov: inspected */
+  }
+  if (lower_case_table_names)
+  {
+    casedn_str(db);
+    casedn_str(tname);
+  }
+  key_length = ((uint) strlen(db) + (uint) strlen(user) +
+                (uint) strlen(tname) + 3);
+  hash_key = (char*) alloc_root(&memex,key_length);
+  strmov(strmov(strmov(hash_key,user)+1,db)+1,tname);
+  privs = (ulong) form->field[6]->val_int();
+  cols  = (ulong) form->field[7]->val_int();
+  privs = fix_rights_for_table(privs);
+  cols =  fix_rights_for_column(cols);
+
+  (void) hash_init(&hash_columns,0,0,0, (hash_get_key) get_key_column,0,
+                   HASH_CASE_INSENSITIVE);
+  if (cols)
+  {
+    int key_len;
+    col_privs->field[0]->store(orig_host,(uint) strlen(orig_host));
+    col_privs->field[1]->store(db,(uint) strlen(db));
+    col_privs->field[2]->store(user,(uint) strlen(user));
+    col_privs->field[3]->store(tname,(uint) strlen(tname));
+    key_len=(col_privs->field[0]->pack_length()+
+             col_privs->field[1]->pack_length()+
+             col_privs->field[2]->pack_length()+
+             col_privs->field[3]->pack_length());
+    key_copy(key,col_privs,0,key_len);
+    col_privs->field[4]->store("",0);
+    col_privs->file->index_init(0);
+    if (col_privs->file->index_read(col_privs->record[0],
+                                    (byte*) col_privs->field[0]->ptr,
+                                    key_len, HA_READ_KEY_EXACT))
+    {
+      cols = 0; /* purecov: deadcode */
+      return;
+    }
+    do
+    {
+      String *res,column_name;
+      GRANT_COLUMN *mem_check;
+      /* As column name is a string, we don't have to supply a buffer */
+      res=col_privs->field[4]->val_str(&column_name,&column_name);
+      ulong priv= (ulong) col_privs->field[6]->val_int();
+      if (!(mem_check = new GRANT_COLUMN(*res,
+                                         fix_rights_for_column(priv))))
+      {
+        /* Don't use this entry */
+        privs = cols = 0;			/* purecov: deadcode */
+        return;				/* purecov: deadcode */
+      }
+      hash_insert(&hash_columns, (byte *) mem_check);
+    } while (!col_privs->file->index_next(col_privs->record[0]) &&
+             !key_cmp(col_privs,key,0,key_len));
+  }
+}
+
+
 static byte* get_grant_table(GRANT_TABLE *buff,uint *length,
 			     my_bool not_used __attribute__((unused)))
 {
@@ -1745,9 +1768,10 @@ static GRANT_TABLE *table_hash_search(const char *host,const char* ip,
     }
     else
     {
-      if ((host && !wild_case_compare(host,grant_table->host)) ||
-	  (ip && !wild_case_compare(ip,grant_table->host)))
-	found=grant_table;					// Host ok
+      if (((host && !wild_case_compare(host,grant_table->host)) ||
+	  (ip && !wild_case_compare(ip,grant_table->host))) &&
+          (!found || found->sort < grant_table->sort))
+	found=grant_table;
     }
   }
   return found;
@@ -2276,8 +2300,8 @@ int mysql_table_grant(THD *thd, TABLE_LIST *table_list,
 }
 
 
-int mysql_grant (THD *thd, const char *db, List <LEX_USER> &list,
-		 ulong rights, bool revoke_grant)
+int mysql_grant(THD *thd, const char *db, List <LEX_USER> &list,
+                ulong rights, bool revoke_grant)
 {
   List_iterator <LEX_USER> str_list (list);
   LEX_USER *Str;
@@ -2413,7 +2437,7 @@ my_bool grant_init(THD *org_thd)
   grant_option = FALSE;
   (void) hash_init(&hash_tables,0,0,0, (hash_get_key) get_grant_table,
 		   (hash_free_key) free_grant_table,0);
-  init_sql_alloc(&memex,1024,0);
+  init_sql_alloc(&memex, ACL_ALLOC_BLOCK_SIZE, 0);
 
   /* Don't do anything if running with --skip-grant */
   if (!initialized)
@@ -2528,8 +2552,8 @@ void grant_reload(THD *thd)
 
 
 /****************************************************************************
-  Check grants
-  All errors are written directly to the client if command name is given !
+  Check table level grants
+  All errors are written directly to the client if no_errors is given !
 ****************************************************************************/
 
 bool check_grant(THD *thd, ulong want_access, TABLE_LIST *tables,
@@ -2631,9 +2655,9 @@ bool check_grant_column(THD *thd,TABLE *table, const char *name,
   if (table->grant.version != grant_version)
   {
     table->grant.grant_table=
-      table_hash_search(thd->host,thd->ip,thd->db,
+      table_hash_search(thd->host, thd->ip, table->table_cache_key,
 			thd->priv_user,
-			table->real_name,0);	/* purecov: inspected */
+			table->real_name, 0);	/* purecov: inspected */
     table->grant.version=grant_version;		/* purecov: inspected */
   }
   if (!(grant_table=table->grant.grant_table))
@@ -2888,7 +2912,7 @@ int mysql_show_grants(THD *thd,LEX_USER *lex_user)
     if (!(user=acl_user->user))
       user="";
     if (!(host=acl_user->host.hostname))
-      host="%";
+      host="";
     if (!strcmp(lex_user->user.str,user) &&
 	!my_strcasecmp(lex_user->host.str,host))
       break;
@@ -3011,7 +3035,7 @@ int mysql_show_grants(THD *thd,LEX_USER *lex_user)
   /* Add database access */
   for (counter=0 ; counter < acl_dbs.elements ; counter++)
   {
-    const char *user,*host;
+    const char *user, *host;
 
     acl_db=dynamic_element(&acl_dbs,counter,ACL_DB*);
     if (!(user=acl_db->user))
@@ -3069,35 +3093,36 @@ int mysql_show_grants(THD *thd,LEX_USER *lex_user)
     }
   }
 
-  /* Add column access */
+  /* Add table & column access */
   for (index=0 ; index < hash_tables.records ; index++)
   {
-    const char *user,*host;
+    const char *user;
     GRANT_TABLE *grant_table= (GRANT_TABLE*) hash_element(&hash_tables,index);
 
     if (!(user=grant_table->user))
-      user="";
-    if (!(host=grant_table->host))
-      host="";
+      user= "";
 
     if (!strcmp(lex_user->user.str,user) &&
-	!my_strcasecmp(lex_user->host.str,host))
+	!my_strcasecmp(lex_user->host.str, grant_table->orig_host))
     {
       ulong table_access= grant_table->privs;
       if ((table_access | grant_table->cols) != 0)
       {
 	String global(buff,sizeof(buff));
+	ulong test_access= (table_access | grant_table->cols) & ~GRANT_ACL;
+
 	global.length(0);
 	global.append("GRANT ",6);
 
 	if (test_all_bits(table_access, (TABLE_ACLS & ~GRANT_ACL)))
 	  global.append("ALL PRIVILEGES",14);
- 	else if (!(table_access & ~GRANT_ACL))
+ 	else if (!test_access)
  	  global.append("USAGE",5);
 	else
 	{
+          /* Add specific column access */
 	  int found= 0;
-	  ulong j,test_access= (table_access | grant_table->cols) & ~GRANT_ACL;
+	  ulong j;
 
 	  for (counter= 0, j= SELECT_ACL; j <= TABLE_ACLS; counter++, j<<= 1)
 	  {
