@@ -55,18 +55,20 @@ static double _nwghts[11]=
 static double *nwghts=_nwghts+5; /* nwghts[i] = -0.5*1.5**i */
 
 #define FTB_FLAG_TRUNC 1                  /* MUST be 1                  */
-#define FTB_FLAG_YES   2                  /* These two - YES and NO     */
-#define FTB_FLAG_NO    4                  /*  should NEVER be set both  */
+#define FTB_FLAG_YES   2                  /*  no two from these three   */
+#define FTB_FLAG_NO    4                  /*   YES, NO, WONLY           */
+#define FTB_FLAG_WONLY 8                  /*  should be ever set both   */
 
 typedef struct st_ftb_expr FTB_EXPR;
 struct st_ftb_expr
 {
   FTB_EXPR *up;
-  byte     *quot, *qend;
   float     weight;
   uint      flags;
-  my_off_t  docid[2];             /* for index search and for scan */
+  my_off_t  docid[2];
+/* ^^^^^^^^^^^^^^^^^^ FTB_{EXPR,WORD} common section */
   float     cur_weight;
+  byte     *quot, *qend;
   int       yesses;               /* number of "yes" words matched */
   int       nos;                  /* number of "no"  words matched */
   int       ythresh;              /* number of "yes" words in expr */
@@ -78,7 +80,8 @@ typedef struct st_ftb_word
   FTB_EXPR *up;
   float     weight;
   uint      flags;
-  my_off_t  docid[2];             /* for index search and for scan */
+  my_off_t  docid[2];
+/* ^^^^^^^^^^^^^^^^^^ FTB_{EXPR,WORD} common section */
   uint      ndepth;
   int       len;
   /* ... docid cache can be added here. SerG */
@@ -217,13 +220,15 @@ static void _ftb_init_index_search(FT_INFO *ftb)
     if (ftbw->flags & FTB_FLAG_TRUNC)
     {
       /*
-	special treatment for truncation operator :((
-        1. +trunc* and there're other (not +trunc*) words
+	special treatment for truncation operator
+        1. there are some (besides this) +words
            | no need to search in the index, it can never ADD new rows
            | to the result, and to remove half-matched rows we do scan anyway
         2. -trunc*
            | same as 1.
-        3. trunc*
+        3. in 1 and 2, +/- need not be on the same expr. level,
+           but can be on any upper level, as in +word +(trunc1* trunc2*)
+        4. otherwise
            | We have to index-search for this prefix.
            | It may cause duplicates, as in the index (sorted by <word,docid>)
            |   <aaaa,row1>
@@ -231,25 +236,40 @@ static void _ftb_init_index_search(FT_INFO *ftb)
            |   <aacc,row1>
            | Searching for "aa*" will find row1 twice...
       */
-      if ( test(ftbw->flags&FTB_FLAG_NO) ||                 /* 2 */
-          (test(ftbw->flags&FTB_FLAG_YES) &&                /* 1 */
-           ftbw->up->ythresh - ftbw->up->yweaks >1))        /* 1 */
+      FTB_EXPR *ftbe;
+      for (ftbe=(FTB_EXPR*)ftbw;
+           ftbe->up && !(ftbe->up->flags & FTB_FLAG_TRUNC);
+           ftbe->up->flags|= FTB_FLAG_TRUNC, ftbe=ftbe->up)
       {
-        ftbw->docid[0]=HA_POS_ERROR;
-        ftbw->up->yweaks++;
+        if (ftbe->flags & FTB_FLAG_NO ||                     /* 2 */
+             ftbe->up->ythresh - ftbe->up->yweaks >1)        /* 1 */
+        {
+          FTB_EXPR *top_ftbe=ftbe->up->up;
+          ftbw->docid[0]=HA_POS_ERROR;
+          for (ftbe=ftbw->up; ftbe != top_ftbe; ftbe=ftbe->up)
+            if (ftbe->flags & FTB_FLAG_YES)
+              ftbe->yweaks++;
+          ftbe=0;
+          break;
+        }
+      }
+      if (!ftbe)
         continue;
-      }
-      else /* 3 */
-      {
-        if (!is_tree_inited(& ftb->no_dupes))
-          init_tree(& ftb->no_dupes,0,0,sizeof(my_off_t),
-              _ftb_no_dupes_cmp,0,0,0);
-        else
-          reset_tree(& ftb->no_dupes);
-      }
+      /* 3 */
+      if (!is_tree_inited(& ftb->no_dupes))
+        init_tree(& ftb->no_dupes,0,0,sizeof(my_off_t),
+            _ftb_no_dupes_cmp,0,0,0);
+      else
+        reset_tree(& ftb->no_dupes);
     }
-    r=_mi_search(info, keyinfo, (uchar*) ftbw->word, ftbw->len,
-                              SEARCH_FIND | SEARCH_BIGGER, keyroot);
+    for (
+         r=_mi_search(info, keyinfo, (uchar*) ftbw->word, ftbw->len,
+                      SEARCH_FIND | SEARCH_BIGGER, keyroot) ;
+         !r && info->lastpos >= info->state->data_file_length;
+         r=_mi_search_next(info, keyinfo, info->lastkey, info->lastkey_length,
+                           SEARCH_BIGGER, keyroot)
+        );
+
     if (!r)
     {
       r=_mi_compare_text(ftb->charset,
@@ -308,11 +328,14 @@ FT_INFO * ft_init_boolean_search(MI_INFO *info, uint keynr, byte *query,
     Hack: instead of init_queue, we'll use reinit queue to be able
     to alloc queue with alloc_root()
   */
-  res=ftb->queue.max_elements=1+query_len/(ft_min_word_len+1);
-  ftb->queue.root=(byte **)alloc_root(&ftb->mem_root, (res+1)*sizeof(void*));
+  res=ftb->queue.max_elements=1+query_len/(min(ft_min_word_len,2)+1);
+  if (!(ftb->queue.root=
+        (byte **)alloc_root(&ftb->mem_root, (res+1)*sizeof(void*))))
+    goto err;
   reinit_queue(& ftb->queue, res, 0, 0,
                          (int (*)(void*,byte*,byte*))FTB_WORD_cmp, 0);
-  ftbe=(FTB_EXPR *)alloc_root(&ftb->mem_root, sizeof(FTB_EXPR));
+  if (!(ftbe=(FTB_EXPR *)alloc_root(&ftb->mem_root, sizeof(FTB_EXPR))))
+    goto err;
   ftbe->weight=1;
   ftbe->flags=FTB_FLAG_YES;
   ftbe->nos=1;
@@ -330,28 +353,41 @@ FT_INFO * ft_init_boolean_search(MI_INFO *info, uint keynr, byte *query,
   if (ftb->queue.elements<2) ftb->with_scan &= ~FTB_FLAG_TRUNC;
   ftb->state=READY;
   return ftb;
+err:
+  free_root(& ftb->mem_root, MYF(0));
+  my_free((gptr)ftb,MYF(0));
+  return 0;
 }
 
 
-/* returns 1 if str0 contain str1 */
+/* returns 1 if str0 ~= /\<str1\>/ */
 static int _ftb_strstr(const byte *s0, const byte *e0,
                 const byte *s1, const byte *e1,
                 CHARSET_INFO *cs)
 {
-  const byte *p;
+  const byte *p0, *p1;
+  my_bool s_after, e_before;
 
-  while (s0 < e0)
+  s_after=true_word_char(s1[0]);
+  e_before=true_word_char(e1[-1]);
+  p0=s0;
+
+  while (p0 < e0)
   {
-    while (s0 < e0 && cs->to_upper[(uint) (uchar) *s0++] !=
+    while (p0 < e0 && cs->to_upper[(uint) (uchar) *p0++] !=
 	   cs->to_upper[(uint) (uchar) *s1])
       /* no-op */;
-    if (s0 >= e0)
+    if (p0 >= e0)
       return 0;
-    p=s1+1;
-    while (s0 < e0 && p < e1 && cs->to_upper[(uint) (uchar) *s0] ==
-	   cs->to_upper[(uint) (uchar) *p])
-      s0++, p++;
-    if (p >= e1)
+
+    if (s_after && p0-1 > s0 && true_word_char(p0[-2]))
+      continue;
+
+    p1=s1+1;
+    while (p0 < e0 && p1 < e1 && cs->to_upper[(uint) (uchar) *p0] ==
+	   cs->to_upper[(uint) (uchar) *p1])
+      p0++, p1++;
+    if (p1 == e1 && (!e_before || p0 == e0 || !true_word_char(p0[0])))
       return 1;
   }
   return 0;
@@ -424,7 +460,8 @@ static void _ftb_climb_the_tree(FTB *ftb, FTB_WORD *ftbw, FT_SEG_ITERATOR *ftsi_
       ftbe->cur_weight +=  weight;
       if (ftbe->yesses < ythresh)
         break;
-      yn= (ftbe->yesses++ == ythresh) ? ftbe->flags : 0 ;
+      if (!(yn & FTB_FLAG_WONLY))
+        yn= (ftbe->yesses++ == ythresh) ? ftbe->flags : FTB_FLAG_WONLY ;
       weight*= ftbe->weight;
     }
   }
@@ -466,8 +503,13 @@ int ft_boolean_read_next(FT_INFO *ftb, char *record)
       _ftb_climb_the_tree(ftb, ftbw, 0);
 
       /* update queue */
-      r=_mi_search(info, keyinfo, (uchar*) ftbw->word, USE_WHOLE_KEY,
-                   SEARCH_BIGGER , keyroot);
+      for (
+          r=_mi_search(info, keyinfo, (uchar*) ftbw->word, USE_WHOLE_KEY,
+                       SEARCH_BIGGER, keyroot) ;
+           !r && info->lastpos >= info->state->data_file_length ;
+           r=_mi_search_next(info, keyinfo, info->lastkey, info->lastkey_length,
+                             SEARCH_BIGGER, keyroot)
+          );
       if (!r)
       {
         r=_mi_compare_text(ftb->charset,
