@@ -730,6 +730,7 @@ uint mi_state_info_write(File file, MI_STATE_INFO *state, uint pWrite)
   uchar *ptr=buff;
   uint	i, keys= (uint) state->header.keys,
 	key_blocks=state->header.max_block_size;
+  DBUG_ENTER("mi_state_info_write");
 
   memcpy_fixed(ptr,&state->header,sizeof(state->header));
   ptr+=sizeof(state->header);
@@ -780,10 +781,10 @@ uint mi_state_info_write(File file, MI_STATE_INFO *state, uint pWrite)
   }
 
   if (pWrite & 1)
-     return my_pwrite(file,(char*) buff, (uint) (ptr-buff), 0L,
-		      MYF(MY_NABP | MY_THREADSAFE));
-  else
-    return my_write(file,  (char*) buff, (uint) (ptr-buff), MYF(MY_NABP));
+    DBUG_RETURN(my_pwrite(file,(char*) buff, (uint) (ptr-buff), 0L,
+			  MYF(MY_NABP | MY_THREADSAFE)));
+  DBUG_RETURN(my_write(file,  (char*) buff, (uint) (ptr-buff),
+		       MYF(MY_NABP)));
 }
 
 
@@ -814,6 +815,8 @@ char *mi_state_info_read(char *ptr, MI_STATE_INFO *state)
   state->status = mi_uint4korr(ptr);		ptr +=4;
   state->update_count=mi_uint4korr(ptr);	ptr +=4;
 
+  ptr+= state->state_diff_length;
+
   for (i=0; i < keys; i++)
   {
     state->key_root[i]= mi_sizekorr(ptr);	ptr +=8;
@@ -822,7 +825,6 @@ char *mi_state_info_read(char *ptr, MI_STATE_INFO *state)
   {
     state->key_del[i] = mi_sizekorr(ptr);	ptr +=8;
   }
-  ptr+= state->state_diff_length;
   state->sec_index_changed = mi_uint4korr(ptr); ptr +=4;
   state->sec_index_used =    mi_uint4korr(ptr); ptr +=4;
   state->version     = mi_uint4korr(ptr);	ptr +=4;
