@@ -783,7 +783,8 @@ int _mi_key_cmp(register MI_KEYSEG *keyseg, register uchar *a,
 
         if (piks &&
             (flag=_mi_compare_text(keyseg->charset,a,a_length,b,b_length,
-             (my_bool) ((nextflag & SEARCH_PREFIX) && next_key_length <= 0))))
+				   (my_bool) ((nextflag & SEARCH_PREFIX) &&
+					      next_key_length <= 0))))
           return ((keyseg->flag & HA_REVERSE_SORT) ? -flag : flag);
         a+=a_length;
         b+=b_length;
@@ -801,7 +802,8 @@ int _mi_key_cmp(register MI_KEYSEG *keyseg, register uchar *a,
 	}
         if (piks &&
             (flag=_mi_compare_text(keyseg->charset,a,a_length,b,b_length,
-             (my_bool) ((nextflag & SEARCH_PREFIX) && next_key_length <= 0))))
+				   (my_bool) ((nextflag & SEARCH_PREFIX)
+					      && next_key_length <= 0))))
           return ((keyseg->flag & HA_REVERSE_SORT) ? -flag : flag);
         a=end;
         b+=length;
@@ -817,7 +819,8 @@ int _mi_key_cmp(register MI_KEYSEG *keyseg, register uchar *a,
 
         if (piks &&
             (flag=compare_bin(a,a_length,b,b_length,
-             (my_bool) ((nextflag & SEARCH_PREFIX) && next_key_length <= 0))))
+			      (my_bool) ((nextflag & SEARCH_PREFIX) &&
+					 next_key_length <= 0))))
           return ((keyseg->flag & HA_REVERSE_SORT) ? -flag : flag);
         a+=a_length;
         b+=b_length;
@@ -828,7 +831,8 @@ int _mi_key_cmp(register MI_KEYSEG *keyseg, register uchar *a,
         uint length=keyseg->length;
         if (piks &&
             (flag=compare_bin(a,length,b,length,
-             (my_bool) ((nextflag & SEARCH_PREFIX) && next_key_length <= 0))))
+			      (my_bool) ((nextflag & SEARCH_PREFIX) &&
+					 next_key_length <= 0))))
           return ((keyseg->flag & HA_REVERSE_SORT) ? -flag : flag);
         a+=length;
         b+=length;
@@ -836,17 +840,27 @@ int _mi_key_cmp(register MI_KEYSEG *keyseg, register uchar *a,
       break;
     case HA_KEYTYPE_VARTEXT:
       {
-        int a_length,b_length,pack_length;
+        int a_length,full_a_length,b_length,full_b_length,pack_length;
         get_key_length(a_length,a);
         get_key_pack_length(b_length,pack_length,b);
+        full_a_length=a_length;
+        full_b_length=b_length;
         next_key_length=key_length-b_length-pack_length;
 
+	if ((nextflag & (SEARCH_FIND | SEARCH_UPDATE)) == SEARCH_FIND)
+	{
+	  while (a_length && a[a_length-1] == ' ')
+	    a_length--;
+	  while (b_length && b[b_length-1] == ' ')
+	    b_length--;
+	}
         if (piks &&
             (flag=_mi_compare_text(keyseg->charset,a,a_length,b,b_length,
-             (my_bool) ((nextflag & SEARCH_PREFIX) && next_key_length <= 0))))
+				   (my_bool) ((nextflag & SEARCH_PREFIX) &&
+					      next_key_length <= 0))))
           return ((keyseg->flag & HA_REVERSE_SORT) ? -flag : flag);
-        a+=a_length;
-        b+=b_length;
+        a+=full_a_length;
+        b+=full_b_length;
         break;
       }
       break;
@@ -859,7 +873,8 @@ int _mi_key_cmp(register MI_KEYSEG *keyseg, register uchar *a,
 
         if (piks &&
             (flag=compare_bin(a,a_length,b,b_length,
-             (my_bool) ((nextflag & SEARCH_PREFIX) && next_key_length <= 0))))
+			      (my_bool) ((nextflag & SEARCH_PREFIX) &&
+					 next_key_length <= 0))))
           return ((keyseg->flag & HA_REVERSE_SORT) ? -flag : flag);
         a+=a_length;
         b+=b_length;
@@ -950,7 +965,7 @@ int _mi_key_cmp(register MI_KEYSEG *keyseg, register uchar *a,
 
       if (keyseg->flag & HA_REVERSE_SORT)
       {
-        swap(uchar*,a,b);       
+        swap(uchar*,a,b);
         swap_flag=1;                            /* Remember swap of a & b */
         end= a+ (int) (end-b);
       }
