@@ -210,13 +210,13 @@ uint cleanup_dirname(register my_string to, const char *from)
 } /* cleanup_dirname */
 
 
-	/*
-	  On system where you don't have symbolic links, the following
-	  code will allow you to create a file: 
-	  directory-name.lnk that should contain the real path
-	  to the directory.  This will be used if the directory name
-	  doesn't exists
-	*/
+/*
+  On system where you don't have symbolic links, the following
+  code will allow you to create a file: 
+  directory-name.sym that should contain the real path
+  to the directory.  This will be used if the directory name
+  doesn't exists
+*/
 
 
 my_bool my_use_symdir=0;	/* Set this if you want to use symdirs */
@@ -228,16 +228,17 @@ void symdirget(char *dir)
   char *pos=strend(dir);
   if (dir[0] && pos[-1] != FN_DEVCHAR && access(dir, F_OK))
   {
-    FILE *fp;
+    File file;
+    uint length;
     char temp= *(--pos);            /* May be "/" or "\" */
     strmov(pos,".sym");
-    fp = my_fopen(dir, O_RDONLY,MYF(0));
+    file= my_open(dir, O_RDONLY, MYF(0));
     *pos++=temp; *pos=0;	  /* Restore old filename */
-    if (fp)
+    if (file >= 0)
     {
-      if (fgets(buff, sizeof(buff)-1, fp))
+      if ((length= my_read(file, buff, sizeof(buff), MYF(0))) > 0)
       {
-	for (pos=strend(buff);
+	for (pos= buff + length ;
 	     pos > buff && (iscntrl(pos[-1]) || isspace(pos[-1])) ;
 	     pos --);
 
@@ -247,7 +248,7 @@ void symdirget(char *dir)
 
 	strmake(dir,buff, (uint) (pos-buff));
       }
-      my_fclose(fp,MYF(0));
+      my_close(file, MYF(0));
     }
   }
 }
@@ -346,11 +347,25 @@ static my_string NEAR_F expand_tilde(my_string *path)
   return (my_string) 0;
 }
 
-	/* fix filename so it can be used by open, create .. */
-	/* to may be == from */
-	/* Returns to */
 
-my_string unpack_filename(my_string to, const char *from)
+/*
+  Fix filename so it can be used by open, create
+
+  SYNOPSIS
+    unpack_filename()
+    to		Store result here. Must be at least of size FN_REFLEN.
+    from	Filename in unix format (with ~)
+
+  RETURN
+    # length of to
+
+  NOTES
+    to may be == from
+    ~ will only be expanded if total length < FN_REFLEN
+*/
+
+
+uint unpack_filename(my_string to, const char *from)
 {
   uint length,n_length;
   char buff[FN_REFLEN];
@@ -361,17 +376,17 @@ my_string unpack_filename(my_string to, const char *from)
   if (n_length+strlen(from+length) < FN_REFLEN)
   {
     (void) strmov(buff+n_length,from+length);
-    (void) system_filename(to,buff);		/* Fix to usably filename */
+    length= system_filename(to,buff);		/* Fix to usably filename */
   }
   else
-    (void) system_filename(to,from);		/* Fix to usably filename */
-  DBUG_RETURN(to);
+    length= system_filename(to,from);		/* Fix to usably filename */
+  DBUG_RETURN(length);
 } /* unpack_filename */
 
 
 	/* Convert filename (unix standard) to system standard */
 	/* Used before system command's like open(), create() .. */
-	/* Returns to */
+	/* Returns length of to */
 
 uint system_filename(my_string to, const char *from)
 {
