@@ -37,7 +37,7 @@ WARNING: THIS PROGRAM IS STILL IN BETA. Comments/patches welcome.
 
 # Documentation continued at end of file
 
-my $VERSION = "1.19";
+my $VERSION = "1.21";
 
 my $opt_tmpdir = $ENV{TMPDIR} || "/tmp";
 
@@ -73,6 +73,7 @@ Usage: $0 db_name[./table_regex/] [new_db_name | directory]
   --resetslave         reset the master.info once all tables are locked
   --tmpdir=#	       temporary directory (instead of $opt_tmpdir)
   --record_log_pos=#   record slave and master status in specified db.table
+  --chroot=#           base directory of chroot jail in which mysqld operates
 
   Try \'perldoc $0 for more complete documentation\'
 _OPTIONS
@@ -117,6 +118,7 @@ GetOptions( \%opt,
     "resetslave",
     "tmpdir|t=s",
     "dryrun|n",
+    "chroot=s",
 ) or usage("Invalid option");
 
 # @db_desc
@@ -210,6 +212,7 @@ while ( my ($var,$value) = $sth_vars->fetchrow_array ) {
 }
 my $datadir = $mysqld_vars{'datadir'}
     || die "datadir not in mysqld variables";
+    $datadir= $opt{chroot}.$datadir if ($opt{chroot});
 $datadir =~ s:/$::;
 
 
@@ -235,10 +238,15 @@ else
 
 # --- resolve database names from regexp ---
 if ( defined $opt{regexp} ) {
+    my $t_regex = '.*';
+    if ( $opt{regexp} =~ s{^/(.+)/\./(.+)/$}{$1} ) {
+        $t_regex = $2;
+    }
+
     my $sth_dbs = $dbh->prepare("show databases");
     $sth_dbs->execute;
     while ( my ($db_name) = $sth_dbs->fetchrow_array ) {
-	push @db_desc, { 'src' => $db_name } if ( $db_name =~ m/$opt{regexp}/o );
+	push @db_desc, { 'src' => $db_name, 't_regex' => $t_regex } if ( $db_name =~ m/$opt{regexp}/o );
     }
 }
 
@@ -413,6 +421,8 @@ foreach my $rdb ( @db_desc ) {
 	else {
 	    mkdir($tgt_dirpath, 0750) or die "Can't create '$tgt_dirpath': $!\n"
 		unless -d $tgt_dirpath;
+	    my @f_info= stat "$datadir/$rdb->{src}";
+	    chown $f_info[4], $f_info[5], $tgt_dirpath;
 	}
     }
 }
@@ -938,6 +948,14 @@ server in a mutual replication setup.
 
 Copy all databases with names matching the pattern
 
+=item --regexp /pattern1/./pattern2/
+
+Copy all tables with names matching pattern2 from all databases with
+names matching pattern1. For example, to select all tables which
+names begin with 'bar' from all databases which names end with 'foo':
+
+   mysqlhotcopy --indices --method=cp --regexp /foo$/./^bar/
+
 =item db_name./pattern/
 
 Copy only tables matching pattern. Shell metacharacters ( (, ), |, !,
