@@ -33,7 +33,7 @@ class ha_isammrg: public handler
   const char *table_type() const { return "MRG_ISAM"; }
   const char **bas_ext() const;
   ulong table_flags() const { return (HA_READ_RND_SAME | HA_KEYPOS_TO_RNDPOS |
-				      HA_REC_NOT_IN_SEQ); }
+				      HA_REC_NOT_IN_SEQ | HA_FILE_BASED); }
   ulong index_flags(uint idx) const { return HA_NOT_READ_PREFIX_LAST; }
 
   uint max_record_length() const { return HA_MAX_REC_LENGTH; }
