@@ -30,13 +30,12 @@
 #include "../myisam/myisamdef.h"
 #endif
 
-ulong myisam_sort_buffer_size;
 ulong myisam_recover_options= HA_RECOVER_NONE;
 
 /* bits in myisam_recover_options */
 const char *myisam_recover_names[] =
 { "DEFAULT", "BACKUP", "FORCE", "QUICK", NullS};
-TYPELIB myisam_recover_typelib= {array_elements(myisam_recover_names),"",
+TYPELIB myisam_recover_typelib= {array_elements(myisam_recover_names)-1,"",
 				 myisam_recover_names};
 
 
@@ -45,6 +44,7 @@ TYPELIB myisam_recover_typelib= {array_elements(myisam_recover_names),"",
 *****************************************************************************/
 
 // collect errors printed by mi_check routines
+
 static void mi_check_print_msg(MI_CHECK *param,	const char* msg_type,
 			       const char *fmt, va_list args)
 {
@@ -66,8 +66,7 @@ static void mi_check_print_msg(MI_CHECK *param,	const char* msg_type,
     sql_print_error(msgbuf);
     return;
   }
-  if (param->testflag & (T_CREATE_MISSING_KEYS | T_SAFE_REPAIR |
-			 T_AUTO_REPAIR))
+  if (param->testflag & (T_CREATE_MISSING_KEYS | T_SAFE_REPAIR | T_AUTO_REPAIR))
   {
     my_message(ER_NOT_KEYFILE,msgbuf,MYF(MY_WME));
     return;
@@ -80,9 +79,8 @@ static void mi_check_print_msg(MI_CHECK *param,	const char* msg_type,
 
   net_store_data(packet, msgbuf);
   if (my_net_write(&thd->net, (char*)thd->packet.ptr(), thd->packet.length()))
-    fprintf(stderr,
-            "Failed on my_net_write, writing to stderr instead: %s\n",
-            msgbuf);
+    sql_print_error("Failed on my_net_write, writing to stderr instead: %s\n",
+		    msgbuf);
   return;
 }
 
@@ -119,18 +117,25 @@ void mi_check_print_warning(MI_CHECK *param, const char *fmt,...)
 }
 
 const char **ha_myisam::bas_ext() const
-{ static const char *ext[]= { ".MYD",".MYI", NullS }; return ext; }
+{ static const char *ext[]= { ".MYI",".MYD", NullS }; return ext; }
 
 
+const char *ha_myisam::index_type(uint key_number)
+{
+  return ((table->key_info[key_number].flags & HA_FULLTEXT) ?
+	  "FULLTEXT" :
+	  "BTREE");
+}
+
 int ha_myisam::net_read_dump(NET* net)
 {
   int data_fd = file->dfile;
   int error = 0;
 
   my_seek(data_fd, 0L, MY_SEEK_SET, MYF(MY_WME));
-  for(;;)
+  for (;;)
   {
-    uint packet_len = my_net_read(net);
+    ulong packet_len = my_net_read(net);
     if (!packet_len)
       break ; // end of file
     if (packet_len == packet_error)
@@ -139,7 +144,7 @@ int ha_myisam::net_read_dump(NET* net)
       error= -1;
       goto err;
     }
-    if (my_write(data_fd, (byte*)net->read_pos, packet_len,
+    if (my_write(data_fd, (byte*)net->read_pos, (uint) packet_len,
 		 MYF(MY_WME|MY_FNABP)))
     {
       error = errno;
@@ -165,7 +170,7 @@ int ha_myisam::dump(THD* thd, int fd)
 
   int error = 0;
   my_seek(data_fd, 0L, MY_SEEK_SET, MYF(MY_WME));
-  for(; bytes_to_read > 0;)
+  for (; bytes_to_read > 0;)
   {
     uint bytes = my_read(data_fd, buf, blocksize, MYF(MY_WME));
     if (bytes == MY_FILE_ERROR)
@@ -212,12 +217,12 @@ int ha_myisam::open(const char *name, int mode, uint test_if_locked)
     return (my_errno ? my_errno : -1);
 
   if (test_if_locked & (HA_OPEN_IGNORE_IF_LOCKED | HA_OPEN_TMP_TABLE))
-    VOID(mi_extra(file,HA_EXTRA_NO_WAIT_LOCK));
+    VOID(mi_extra(file, HA_EXTRA_NO_WAIT_LOCK, 0));
   info(HA_STATUS_NO_LOCK | HA_STATUS_VARIABLE | HA_STATUS_CONST);
   if (!(test_if_locked & HA_OPEN_WAIT_IF_LOCKED))
-    VOID(mi_extra(file,HA_EXTRA_WAIT_LOCK));
+    VOID(mi_extra(file, HA_EXTRA_WAIT_LOCK, 0));
   if (!table->db_record_offset)
-    int_option_flag|=HA_REC_NOT_IN_SEQ;
+    int_table_flags|=HA_REC_NOT_IN_SEQ;
   return (0);
 }
 
@@ -283,7 +288,7 @@ int ha_myisam::check(THD* thd, HA_CHECK_OPT* check_opt)
     error = chk_key(&param, file);
   if (!error)
   {
-    if ((!check_opt->quick &&
+    if ((!(param.testflag & T_QUICK) &&
 	 ((share->options &
 	   (HA_OPTION_PACK_RECORD | HA_OPTION_COMPRESS_RECORD)) ||
 	  (param.testflag & (T_EXTEND | T_MEDIUM)))) ||
@@ -324,7 +329,6 @@ int ha_myisam::check(THD* thd, HA_CHECK_OPT* check_opt)
     mi_mark_crashed(file);
     file->update |= HA_STATE_CHANGED | HA_STATE_ROW_CHANGED;
   }
-  check_opt->retry_without_quick=param.retry_without_quick;
 
   thd->proc_info=old_proc_info;
   return error ? HA_ADMIN_CORRUPT : HA_ADMIN_OK;
@@ -374,14 +378,14 @@ int ha_myisam::restore(THD* thd, HA_CHECK_OPT *check_opt)
   char* backup_dir = thd->lex.backup_dir;
   char src_path[FN_REFLEN], dst_path[FN_REFLEN];
   char* table_name = table->real_name;
+  int error;
+  const char* errmsg;
   DBUG_ENTER("restore");
 
-  if (!fn_format(src_path, table_name, backup_dir, MI_NAME_DEXT, 4 + 64))
+  if (fn_format_relative_to_data_home(src_path, table_name, backup_dir,
+				      MI_NAME_DEXT))
     DBUG_RETURN(HA_ADMIN_INVALID);
 
-  int error = 0;
-  const char* errmsg = "";
-
   if (my_copy(src_path, fn_format(dst_path, table->path, "",
 				  MI_NAME_DEXT, 4), MYF(MY_WME)))
   {
@@ -391,8 +395,7 @@ int ha_myisam::restore(THD* thd, HA_CHECK_OPT *check_opt)
   }
 
   tmp_check_opt.init();
-  tmp_check_opt.quick = 1;
-  tmp_check_opt.flags |= T_VERY_SILENT | T_CALC_CHECKSUM;
+  tmp_check_opt.flags |= T_VERY_SILENT | T_CALC_CHECKSUM | T_QUICK;
   DBUG_RETURN(repair(thd, &tmp_check_opt));
 
  err:
@@ -404,7 +407,7 @@ int ha_myisam::restore(THD* thd, HA_CHECK_OPT *check_opt)
     param.db_name    = table->table_cache_key;
     param.table_name = table->table_name;
     param.testflag = 0;
-    mi_check_print_error(&param,errmsg, errno );
+    mi_check_print_error(&param, errmsg, my_errno);
     DBUG_RETURN(error);
   }
 }
@@ -415,17 +418,19 @@ int ha_myisam::backup(THD* thd, HA_CHECK_OPT *check_opt)
   char* backup_dir = thd->lex.backup_dir;
   char src_path[FN_REFLEN], dst_path[FN_REFLEN];
   char* table_name = table->real_name;
-  int error = 0;
-  const char* errmsg = "";
-  
-  if (!fn_format(dst_path, table_name, backup_dir, reg_ext, 4 + 64))
+  int error;
+  const char *errmsg;
+  DBUG_ENTER("ha_myisam::backup");
+
+  if (fn_format_relative_to_data_home(dst_path, table_name, backup_dir,
+				      reg_ext))
   {
     errmsg = "Failed in fn_format() for .frm file (errno: %d)";
     error = HA_ADMIN_INVALID;
     goto err;
   }
-  
-  if (my_copy(fn_format(src_path, table->path,"", reg_ext, 4),
+
+  if (my_copy(fn_format(src_path, table->path,"", reg_ext, MY_UNPACK_FILENAME),
 	      dst_path,
 	      MYF(MY_WME | MY_HOLD_ORIGINAL_MODES | MY_DONT_OVERWRITE_FILE)))
   {
@@ -434,14 +439,17 @@ int ha_myisam::backup(THD* thd, HA_CHECK_OPT *check_opt)
     goto err;
   }
 
-  if (!fn_format(dst_path, table_name, backup_dir, MI_NAME_DEXT, 4 + 64))
+  /* Change extension */
+  if (!fn_format(dst_path, dst_path, "", MI_NAME_DEXT,
+		 MY_REPLACE_EXT | MY_UNPACK_FILENAME | MY_SAFE_PATH))
   {
     errmsg = "Failed in fn_format() for .MYD file (errno: %d)";
     error = HA_ADMIN_INVALID;
     goto err;
   }
 
-  if (my_copy(fn_format(src_path, table->path,"", MI_NAME_DEXT, 4),
+  if (my_copy(fn_format(src_path, table->path,"", MI_NAME_DEXT,
+			MY_UNPACK_FILENAME),
 	      dst_path,
 	      MYF(MY_WME | MY_HOLD_ORIGINAL_MODES | MY_DONT_OVERWRITE_FILE)))
   {
@@ -449,8 +457,9 @@ int ha_myisam::backup(THD* thd, HA_CHECK_OPT *check_opt)
     error= HA_ADMIN_FAILED;
     goto err;
   }
-  return HA_ADMIN_OK;
-err:
+  DBUG_RETURN(HA_ADMIN_OK);
+
+ err:
   {
     MI_CHECK param;
     myisamchk_init(&param);
@@ -459,8 +468,8 @@ err:
     param.db_name    = table->table_cache_key;
     param.table_name = table->table_name;
     param.testflag = 0;
-    mi_check_print_error(&param, errmsg, my_errno);
-    return error;
+    mi_check_print_error(&param,errmsg, my_errno);
+    DBUG_RETURN(error);
   }
 }
 
@@ -476,28 +485,27 @@ int ha_myisam::repair(THD* thd, HA_CHECK_OPT *check_opt)
   myisamchk_init(&param);
   param.thd = thd;
   param.op_name = (char*) "repair";
-  param.testflag = ((check_opt->flags & ~(T_EXTEND)) | 
-		    T_SILENT | T_FORCE_CREATE |
+  param.testflag = ((check_opt->flags & ~(T_EXTEND)) |
+		    T_SILENT | T_FORCE_CREATE | T_CALC_CHECKSUM |
 		    (check_opt->flags & T_EXTEND ? T_REP : T_REP_BY_SORT));
-  if (check_opt->quick)
-    param.opt_rep_quick++;
   param.sort_buffer_length=  check_opt->sort_buffer_size;
   start_records=file->state->records;
   while ((error=repair(thd,param,0)) && param.retry_repair)
   {
     param.retry_repair=0;
-    if (param.retry_without_quick && param.opt_rep_quick)
+    if (test_all_bits(param.testflag,
+		      (uint) (T_RETRY_WITHOUT_QUICK | T_QUICK)))
     {
-      param.opt_rep_quick=0;
-      sql_print_error("Warning: Retrying repair of: '%s' without quick",
+      param.testflag&= ~T_RETRY_WITHOUT_QUICK;
+      sql_print_error("Note: Retrying repair of: '%s' without quick",
 		      table->path);
       continue;
     }
-    param.opt_rep_quick=0;			// Safety
+    param.testflag&= ~T_QUICK;
     if ((param.testflag & T_REP_BY_SORT))
     {
       param.testflag= (param.testflag & ~T_REP_BY_SORT) | T_REP;
-      sql_print_error("Warning: Retrying repair of: '%s' with keycache",
+      sql_print_error("Note: Retrying repair of: '%s' with keycache",
 		      table->path);
       continue;
     }
@@ -507,7 +515,7 @@ int ha_myisam::repair(THD* thd, HA_CHECK_OPT *check_opt)
       !(check_opt->flags & T_VERY_SILENT))
   {
     char llbuff[22],llbuff2[22];
-    sql_print_error("Warning: Found %s of %s rows when repairing '%s'",
+    sql_print_error("Note: Found %s of %s rows when repairing '%s'",
 		    llstr(file->state->records, llbuff),
 		    llstr(start_records, llbuff2),
 		    table->path);
@@ -525,8 +533,6 @@ int ha_myisam::optimize(THD* thd, HA_CHECK_OPT *check_opt)
   param.op_name = (char*) "optimize";
   param.testflag = (check_opt->flags | T_SILENT | T_FORCE_CREATE |
 		    T_REP_BY_SORT | T_STATISTICS | T_SORT_INDEX);
-  if (check_opt->quick)
-    param.opt_rep_quick++;
   param.sort_buffer_length=  check_opt->sort_buffer_size;
   return repair(thd,param,1);
 }
@@ -537,8 +543,8 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool optimize)
   int error=0;
   uint local_testflag=param.testflag;
   bool optimize_done= !optimize, statistics_done=0;
-  char fixed_name[FN_REFLEN];
   const char *old_proc_info=thd->proc_info;
+  char fixed_name[FN_REFLEN];
   MYISAM_SHARE* share = file->s;
   ha_rows rows= file->state->records;
   DBUG_ENTER("ha_myisam::repair");
@@ -550,11 +556,11 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool optimize)
   param.thd=thd;
   param.tmpdir=mysql_tmpdir;
   param.out_flag=0;
-  VOID(fn_format(fixed_name,file->filename,"",MI_NAME_IEXT,
-		     4+ (param.opt_follow_links ? 16 : 0)));
+  strmov(fixed_name,file->filename);
 
   // Don't lock tables if we have used LOCK TABLE
-  if (!thd->locked_tables && mi_lock_database(file,F_WRLCK))
+  if (!thd->locked_tables && 
+      mi_lock_database(file, table->tmp_table ? F_EXTRA_LCK : F_WRLCK))
   {
     mi_check_print_error(&param,ER(ER_CANT_LOCK),my_errno);
     DBUG_RETURN(HA_ADMIN_FAILED);
@@ -562,10 +568,10 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool optimize)
 
   if (!optimize ||
       ((file->state->del || share->state.split != file->state->records) &&
-       (!param.opt_rep_quick ||
+       (!(param.testflag & T_QUICK) ||
 	!(share->state.changed & STATE_NOT_OPTIMIZED_KEYS))))
   {
-    ulonglong key_map= ((local_testflag & T_CREATE_MISSING_KEYS) ? 
+    ulonglong key_map= ((local_testflag & T_CREATE_MISSING_KEYS) ?
 			((ulonglong) 1L << share->base.keys)-1 :
 			share->state.key_map);
     uint testflag=param.testflag;
@@ -574,15 +580,31 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool optimize)
     {
       local_testflag|= T_STATISTICS;
       param.testflag|= T_STATISTICS;		// We get this for free
-      thd->proc_info="Repair by sorting";
       statistics_done=1;
-      error = mi_repair_by_sort(&param, file, fixed_name, param.opt_rep_quick);
+      if (current_thd->variables.myisam_repair_threads>1)
+      {
+        char buf[40];
+        /* TODO: respect myisam_repair_threads variable */
+        my_snprintf(buf, 40, "Repair with %d threads", my_count_bits(key_map));
+        thd->proc_info=buf;
+        error = mi_repair_parallel(&param, file, fixed_name,
+            param.testflag & T_QUICK);
+        thd->proc_info="Repair done"; // to reset proc_info, as
+                                      // it was pointing to local buffer
+      }
+      else
+      {
+        thd->proc_info="Repair by sorting";
+        error = mi_repair_by_sort(&param, file, fixed_name,
+            param.testflag & T_QUICK);
+      }
     }
     else
     {
       thd->proc_info="Repair with keycache";
       param.testflag &= ~T_REP_BY_SORT;
-      error=  mi_repair(&param, file, fixed_name, param.opt_rep_quick);
+      error=  mi_repair(&param, file, fixed_name,
+			param.testflag & T_QUICK);
     }
     param.testflag=testflag;
     optimize_done=1;
@@ -654,12 +676,50 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool optimize)
 }
 
 
-/* Deactive all not unique index that can be recreated fast */
+/*
+  Deactive all not unique index that can be recreated fast
+
+  SYNOPSIS
+    deactivate_non_unique_index()
+    rows		Rows to be inserted
+			0 if we don't know
+			HA_POS_ERROR if we want to disable all keys
+*/
 
 void ha_myisam::deactivate_non_unique_index(ha_rows rows)
 {
-  if (!(specialflag & SPECIAL_SAFE_MODE))
-    mi_disable_non_unique_index(file,rows);
+  MYISAM_SHARE* share = file->s;
+  if (share->state.key_map == ((ulonglong) 1L << share->base.keys)-1)
+  {
+    if (!(specialflag & SPECIAL_SAFE_MODE))
+    {
+      if (rows == HA_POS_ERROR)
+        mi_extra(file, HA_EXTRA_NO_KEYS, 0);
+      else
+      {
+	/*
+	  Only disable old index if the table was empty and we are inserting
+	  a lot of rows.
+	  We should not do this for only a few rows as this is slower and
+	  we don't want to update the key statistics based of only a few rows.
+	*/
+	if (file->state->records == 0 &&
+	    (!rows || rows >= MI_MIN_ROWS_TO_USE_BULK_INSERT))
+	  mi_disable_non_unique_index(file,rows);
+        else
+        {
+          mi_init_bulk_insert(file,
+			      current_thd->variables.bulk_insert_buff_size,
+			      rows);
+	  table->bulk_insert= 1;
+	}
+      }
+    }
+    enable_activate_all_index=1;
+    info(HA_STATUS_CONST);			// Read new key info
+  }
+  else
+    enable_activate_all_index=0;
 }
 
 
@@ -669,21 +729,27 @@ bool ha_myisam::activate_all_index(THD *thd)
   MI_CHECK param;
   MYISAM_SHARE* share = file->s;
   DBUG_ENTER("activate_all_index");
-  if (share->state.key_map != set_bits(ulonglong, share->base.keys))
+
+  mi_end_bulk_insert(file);
+  table->bulk_insert= 0;
+  if (enable_activate_all_index &&
+     share->state.key_map != set_bits(ulonglong, share->base.keys))
   {
     const char *save_proc_info=thd->proc_info;
     thd->proc_info="Creating index";
     myisamchk_init(&param);
     param.op_name = (char*) "recreating_index";
-    param.testflag = (T_SILENT | T_REP_BY_SORT |
-		      T_CREATE_MISSING_KEYS | T_TRUST_HEADER);
+    param.testflag = (T_SILENT | T_REP_BY_SORT | T_QUICK |
+		      T_CREATE_MISSING_KEYS);
     param.myf_rw&= ~MY_WAIT_IF_FULL;
-    param.sort_buffer_length=  myisam_sort_buffer_size;
-    param.opt_rep_quick++;			// Don't copy data file
+    param.sort_buffer_length=  thd->variables.myisam_sort_buff_size;
     param.tmpdir=mysql_tmpdir;
     error=repair(thd,param,0) != HA_ADMIN_OK;
+    info(HA_STATUS_CONST);
     thd->proc_info=save_proc_info;
   }
+  else
+    enable_activate_all_index=1;
   DBUG_RETURN(error);
 }
 
@@ -699,16 +765,16 @@ bool ha_myisam::check_and_repair(THD *thd)
   check_opt.flags= T_MEDIUM | T_AUTO_REPAIR;
   // Don't use quick if deleted rows
   if (!file->state->del && (myisam_recover_options & HA_RECOVER_QUICK))
-    check_opt.quick=1;
+    check_opt.flags|=T_QUICK;
   sql_print_error("Warning: Checking table:   '%s'",table->path);
   if ((marked_crashed=mi_is_crashed(file)) || check(thd, &check_opt))
   {
     sql_print_error("Warning: Recovering table: '%s'",table->path);
-    check_opt.quick= !check_opt.retry_without_quick && !marked_crashed;
-    check_opt.flags=(((myisam_recover_options & HA_RECOVER_BACKUP) ?
-		      T_BACKUP_DATA : 0) |
-		     (!(myisam_recover_options & HA_RECOVER_FORCE) ?
-		      T_SAFE_REPAIR : 0)) | T_AUTO_REPAIR;
+    check_opt.flags=
+      ((myisam_recover_options & HA_RECOVER_BACKUP ? T_BACKUP_DATA : 0) |
+       (marked_crashed                             ? 0 : T_QUICK) |
+       (myisam_recover_options & HA_RECOVER_FORCE  ? 0 : T_SAFE_REPAIR) |
+       T_AUTO_REPAIR);
     if (repair(thd, &check_opt))
       error=1;
   }
@@ -753,6 +819,14 @@ int ha_myisam::index_read_idx(byte * buf, uint index, const byte * key,
   return error;
 }
 
+int ha_myisam::index_read_last(byte * buf, const byte * key, uint key_len)
+{
+  statistic_increment(ha_read_key_count,&LOCK_status);
+  int error=mi_rkey(file,buf,active_index, key, key_len, HA_READ_PREFIX_LAST);
+  table->status=error ? STATUS_NOT_FOUND: 0;
+  return error;
+}
+
 int ha_myisam::index_next(byte * buf)
 {
   statistic_increment(ha_read_next_count,&LOCK_status);
@@ -800,8 +874,7 @@ int ha_myisam::rnd_init(bool scan)
 {
   if (scan)
     return mi_scan_init(file);
-  else
-    return mi_extra(file,HA_EXTRA_RESET);
+  return mi_extra(file, HA_EXTRA_RESET, 0);
 }
 
 int ha_myisam::rnd_next(byte *buf)
@@ -834,6 +907,8 @@ void ha_myisam::position(const byte* record)
 void ha_myisam::info(uint flag)
 {
   MI_ISAMINFO info;
+  char name_buff[FN_REFLEN];
+
   (void) mi_status(file,&info,flag);
   if (flag & HA_STATUS_VARIABLE)
   {
@@ -854,15 +929,29 @@ void ha_myisam::info(uint flag)
     ref_length=info.reflength;
     table->db_options_in_use    = info.options;
     block_size=myisam_block_size;
-    table->keys_in_use &= info.key_map;
+    table->keys_in_use= (set_bits(key_map, table->keys) &
+			 (key_map) info.key_map);
+    table->keys_for_keyread= table->keys_in_use & ~table->read_only_keys;
     table->db_record_offset=info.record_offset;
     if (table->key_parts)
       memcpy((char*) table->key_info[0].rec_per_key,
 	     (char*) info.rec_per_key,
-	     sizeof(ulong)*table->key_parts);
+	     sizeof(table->key_info[0].rec_per_key)*table->key_parts);
     raid_type=info.raid_type;
     raid_chunks=info.raid_chunks;
     raid_chunksize=info.raid_chunksize;
+
+   /*
+     Set data_file_name and index_file_name to point at the symlink value
+     if table is symlinked (Ie;  Real name is not same as generated name)
+   */
+    data_file_name=index_file_name=0;
+    fn_format(name_buff, file->filename, "", MI_NAME_DEXT, 2);
+    if (strcmp(name_buff, info.data_file_name))
+      data_file_name=info.data_file_name;
+    strmov(fn_ext(name_buff),MI_NAME_IEXT);
+    if (strcmp(name_buff, info.index_file_name))
+      index_file_name=info.index_file_name;
   }
   if (flag & HA_STATUS_ERRKEY)
   {
@@ -878,16 +967,25 @@ void ha_myisam::info(uint flag)
 
 int ha_myisam::extra(enum ha_extra_function operation)
 {
-  if (((specialflag & SPECIAL_SAFE_MODE) || (test_flags & TEST_NO_EXTRA)) &&
-      (operation == HA_EXTRA_WRITE_CACHE ||
-       operation == HA_EXTRA_KEYREAD))
+  if ((specialflag & SPECIAL_SAFE_MODE) && operation == HA_EXTRA_KEYREAD)
     return 0;
-  return mi_extra(file,operation);
+  return mi_extra(file, operation, 0);
 }
 
+
+/* To be used with WRITE_CACHE and EXTRA_CACHE */
+
+int ha_myisam::extra_opt(enum ha_extra_function operation, ulong cache_size)
+{
+  if ((specialflag & SPECIAL_SAFE_MODE) && operation == HA_EXTRA_WRITE_CACHE)
+    return 0;
+  return mi_extra(file, operation, (void*) &cache_size);
+}
+
+
 int ha_myisam::reset(void)
 {
-  return mi_extra(file,HA_EXTRA_RESET);
+  return mi_extra(file, HA_EXTRA_RESET, 0);
 }
 
 int ha_myisam::delete_all_rows()
@@ -902,7 +1000,9 @@ int ha_myisam::delete_table(const char *name)
 
 int ha_myisam::external_lock(THD *thd, int lock_type)
 {
-  return mi_lock_database(file,lock_type);
+  return mi_lock_database(file, !table->tmp_table ?
+			  lock_type : ((lock_type == F_UNLCK) ?
+				       F_UNLCK : F_EXTRA_LCK));
 }
 
 
@@ -918,7 +1018,7 @@ THR_LOCK_DATA **ha_myisam::store_lock(THD *thd,
 
 void ha_myisam::update_create_info(HA_CREATE_INFO *create_info)
 {
-  table->file->info(HA_STATUS_AUTO | HA_STATUS_CONST);
+  ha_myisam::info(HA_STATUS_AUTO | HA_STATUS_CONST);
   if (!(create_info->used_fields & HA_CREATE_USED_AUTO))
   {
     create_info->auto_increment_value=auto_increment_value;
@@ -929,35 +1029,38 @@ void ha_myisam::update_create_info(HA_CREATE_INFO *create_info)
     create_info->raid_chunks= raid_chunks;
     create_info->raid_chunksize= raid_chunksize;
   }
+  create_info->data_file_name=data_file_name;
+  create_info->index_file_name=index_file_name;
 }
 
 
-int ha_myisam::create(const char *name, register TABLE *form,
+int ha_myisam::create(const char *name, register TABLE *table_arg,
 		      HA_CREATE_INFO *info)
 {
   int error;
   uint i,j,recpos,minpos,fieldpos,temp_length,length;
-  bool found_auto_increment=0;
+  bool found_real_auto_increment=0;
   enum ha_base_keytype type;
   char buff[FN_REFLEN];
   KEY *pos;
   MI_KEYDEF *keydef;
   MI_COLUMNDEF *recinfo,*recinfo_pos;
   MI_KEYSEG *keyseg;
-  uint options=form->db_options_in_use;
+  uint options=table_arg->db_options_in_use;
   DBUG_ENTER("ha_myisam::create");
 
   type=HA_KEYTYPE_BINARY;				// Keep compiler happy
   if (!(my_multi_malloc(MYF(MY_WME),
-			&recinfo,(form->fields*2+2)*sizeof(MI_COLUMNDEF),
-			&keydef, form->keys*sizeof(MI_KEYDEF),
+			&recinfo,(table_arg->fields*2+2)*sizeof(MI_COLUMNDEF),
+			&keydef, table_arg->keys*sizeof(MI_KEYDEF),
 			&keyseg,
-			((form->key_parts + form->keys) * sizeof(MI_KEYSEG)),
-			0)))
-    DBUG_RETURN(1);
+			((table_arg->key_parts + table_arg->keys) *
+			 sizeof(MI_KEYSEG)),
+			NullS)))
+    DBUG_RETURN(HA_ERR_OUT_OF_MEM);
 
-  pos=form->key_info;
-  for (i=0; i < form->keys ; i++, pos++)
+  pos=table_arg->key_info;
+  for (i=0; i < table_arg->keys ; i++, pos++)
   {
     keydef[i].flag= (pos->flags & (HA_NOSAME | HA_FULLTEXT));
     keydef[i].seg=keyseg;
@@ -1000,44 +1103,44 @@ int ha_myisam::create(const char *name, register TABLE *form,
       {
 	keydef[i].seg[j].null_bit=field->null_bit;
 	keydef[i].seg[j].null_pos= (uint) (field->null_ptr-
-					   (uchar*) form->record[0]);
+					   (uchar*) table_arg->record[0]);
       }
       else
       {
 	keydef[i].seg[j].null_bit=0;
 	keydef[i].seg[j].null_pos=0;
       }
-      if (j == 0 && field->flags & AUTO_INCREMENT_FLAG &&
-	  !found_auto_increment)
-      {
-	keydef[i].flag|=HA_AUTO_KEY;
-	found_auto_increment=1;
-      }
       if (field->type() == FIELD_TYPE_BLOB)
       {
 	keydef[i].seg[j].flag|=HA_BLOB_PART;
 	/* save number of bytes used to pack length */
 	keydef[i].seg[j].bit_start= (uint) (field->pack_length() -
-					    form->blob_ptr_size);
+					    table_arg->blob_ptr_size);
       }
     }
     keyseg+=pos->key_parts;
   }
 
+  if (table_arg->found_next_number_field)
+  {
+    keydef[table_arg->next_number_index].flag|= HA_AUTO_KEY;
+    found_real_auto_increment= table_arg->next_number_key_offset == 0;
+  }
+
   recpos=0; recinfo_pos=recinfo;
-  while (recpos < (uint) form->reclength)
+  while (recpos < (uint) table_arg->reclength)
   {
     Field **field,*found=0;
-    minpos=form->reclength; length=0;
+    minpos=table_arg->reclength; length=0;
 
-    for (field=form->field ; *field ; field++)
+    for (field=table_arg->field ; *field ; field++)
     {
       if ((fieldpos=(*field)->offset()) >= recpos &&
 	  fieldpos <= minpos)
       {
 	/* skip null fields */
 	if (!(temp_length= (*field)->pack_length()))
-	  continue;				/* Skipp null-fields */
+	  continue;				/* Skip null-fields */
 	if (! found || fieldpos < minpos ||
 	    (fieldpos == minpos && temp_length < length))
 	{
@@ -1063,20 +1166,20 @@ int ha_myisam::create(const char *name, register TABLE *form,
     else if (!(options & HA_OPTION_PACK_RECORD))
       recinfo_pos->type= (int) FIELD_NORMAL;
     else if (found->zero_pack())
-      recinfo_pos->type= (int) FIELD_SKIPP_ZERO;
+      recinfo_pos->type= (int) FIELD_SKIP_ZERO;
     else
       recinfo_pos->type= (int) ((length <= 3 ||
 				      (found->flags & ZEROFILL_FLAG)) ?
 				     FIELD_NORMAL :
 				     found->type() == FIELD_TYPE_STRING ||
 				     found->type() == FIELD_TYPE_VAR_STRING ?
-				     FIELD_SKIPP_ENDSPACE :
-				     FIELD_SKIPP_PRESPACE);
+				     FIELD_SKIP_ENDSPACE :
+				     FIELD_SKIP_PRESPACE);
     if (found->null_ptr)
     {
       recinfo_pos->null_bit=found->null_bit;
       recinfo_pos->null_pos= (uint) (found->null_ptr-
-				  (uchar*) form->record[0]);
+				  (uchar*) table_arg->record[0]);
     }
     else
     {
@@ -1091,18 +1194,25 @@ int ha_myisam::create(const char *name, register TABLE *form,
   }
   MI_CREATE_INFO create_info;
   bzero((char*) &create_info,sizeof(create_info));
-  create_info.max_rows=form->max_rows;
-  create_info.reloc_rows=form->min_rows;
+  create_info.max_rows=table_arg->max_rows;
+  create_info.reloc_rows=table_arg->min_rows;
+  create_info.with_auto_increment=found_real_auto_increment;
   create_info.auto_increment=(info->auto_increment_value ?
 			      info->auto_increment_value -1 :
 			      (ulonglong) 0);
-  create_info.data_file_length=(ulonglong) form->max_rows*form->avg_row_length;
+  create_info.data_file_length= ((ulonglong) table_arg->max_rows *
+				 table_arg->avg_row_length);
   create_info.raid_type=info->raid_type;
-  create_info.raid_chunks=info->raid_chunks ? info->raid_chunks : RAID_DEFAULT_CHUNKS;
-  create_info.raid_chunksize=info->raid_chunksize ? info->raid_chunksize : RAID_DEFAULT_CHUNKSIZE;
+  create_info.raid_chunks= (info->raid_chunks ? info->raid_chunks :
+			    RAID_DEFAULT_CHUNKS);
+  create_info.raid_chunksize=(info->raid_chunksize ? info->raid_chunksize :
+			      RAID_DEFAULT_CHUNKSIZE);
+  create_info.data_file_name= info->data_file_name;
+  create_info.index_file_name=info->index_file_name;
 
-  error=mi_create(fn_format(buff,name,"","",2+4+16),
-		  form->keys,keydef,
+  /* TODO: Check that the following fn_format is really needed */
+  error=mi_create(fn_format(buff,name,"","",2+4),
+		  table_arg->keys,keydef,
 		  (uint) (recinfo_pos-recinfo), recinfo,
 		  0, (MI_UNIQUEDEF*) 0,
 		  &create_info,
@@ -1131,9 +1241,12 @@ longlong ha_myisam::get_auto_increment()
     return auto_increment_value;
   }
 
+  if (table->bulk_insert)
+    mi_flush_bulk_insert(file, table->next_number_index);
+
   longlong nr;
   int error;
-  byte key[MAX_KEY_LENGTH];
+  byte key[MI_MAX_KEY_LENGTH];
   (void) extra(HA_EXTRA_KEYREAD);
   key_copy(key,table,table->next_number_index,
 	   table->next_number_key_offset);
@@ -1149,6 +1262,35 @@ longlong ha_myisam::get_auto_increment()
 }
 
 
+/*
+  Find out how many rows there is in the given range
+
+  SYNOPSIS
+    records_in_range()
+    inx			Index to use
+    start_key		Start of range.  Null pointer if from first key
+    start_key_len	Length of start key
+    start_search_flag	Flag if start key should be included or not
+    end_key		End of range. Null pointer if to last key
+    end_key_len		Length of end key
+    end_search_flag	Flag if start key should be included or not
+
+  NOTES
+    start_search_flag can have one of the following values:
+      HA_READ_KEY_EXACT		Include the key in the range
+      HA_READ_AFTER_KEY		Don't include key in range
+
+    end_search_flag can have one of the following values:  
+      HA_READ_BEFORE_KEY	Don't include key in range
+      HA_READ_AFTER_KEY		Include all 'end_key' values in the range
+
+  RETURN
+   HA_POS_ERROR		Something is wrong with the index tree.
+   0			There is no matching keys in the given range
+   number > 0		There is approximately 'number' matching rows in
+			the range.
+*/
+
 ha_rows ha_myisam::records_in_range(int inx,
 				    const byte *start_key,uint start_key_len,
 				    enum ha_rkey_function start_search_flag,
@@ -1163,6 +1305,7 @@ ha_rows ha_myisam::records_in_range(int inx,
 				       end_search_flag);
 }
 
+
 int ha_myisam::ft_read(byte * buf)
 {
   int error;
@@ -1172,9 +1315,8 @@ int ha_myisam::ft_read(byte * buf)
 
   thread_safe_increment(ha_read_next_count,&LOCK_status); // why ?
 
-  error=ft_read_next((FT_DOCLIST *) ft_handler,(char*) buf);
+  error=ft_handler->please->read_next(ft_handler,(char*) buf);
 
   table->status=error ? STATUS_NOT_FOUND: 0;
   return error;
 }
-
