@@ -1,15 +1,15 @@
 /* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
@@ -37,7 +37,7 @@ void item_init(void)
 Item::Item()
 {
   marker=0;
-  binary=maybe_null=null_value=with_sum_func=0;
+  binary=maybe_null=null_value=with_sum_func=unsigned_flag=0;
   name=0;
   decimals=0; max_length=0;
   next=current_thd->free_list;			// Put in free list
@@ -117,6 +117,14 @@ bool Item::get_time(TIME *ltime)
   return 0;
 }
 
+Item_ident::Item_ident(const char *db_name_par,const char *table_name_par,
+		       const char *field_name_par)
+  :db_name(db_name_par),table_name(table_name_par),field_name(field_name_par)
+{
+  name = (char*) field_name_par;
+}
+
+
 Item_field::Item_field(Field *f) :Item_ident(NullS,f->table_name,f->field_name)
 {
   set_field(f);
@@ -132,6 +140,7 @@ void Item_field::set_field(Field *field_par)
   table_name=field_par->table_name;
   field_name=field_par->field_name;
   binary=field_par->binary();
+  unsigned_flag=test(field_par->flags & UNSIGNED_FLAG);
 }
 
 const char *Item_ident::full_name() const
@@ -258,6 +267,22 @@ void Item_int::print(String *str)
   str->append(name);
 }
 
+String *Item_uint::val_str(String *str)
+{
+  str->set((ulonglong) value);
+  return str;
+}
+
+void Item_uint::print(String *str)
+{
+  if (!name)
+  {
+    str_value.set((ulonglong) value);
+    name=str_value.c_ptr();
+  }
+  str->append(name);
+}
+
 
 String *Item_real::val_str(String *str)
 {
@@ -297,6 +322,15 @@ String *Item_copy_string::val_str(String *str)
   return &str_value;
 }
 
+bool Item_copy_string::save_in_field(Field *field, bool no_conversions)
+{
+  if (null_value)
+    return set_field_to_null(field);
+  field->set_notnull();
+  field->store(str_value.ptr(), str_value.length());
+  return 0;
+}
+
 /*
 ** Functions to convert item to field (for send_fields)
 */
@@ -310,13 +344,21 @@ bool Item::fix_fields(THD *thd,
 
 bool Item_field::fix_fields(THD *thd,TABLE_LIST *tables)
 {
-  if (!field)
+  if (!field)					// If field is not checked
   {
     Field *tmp;
     if (!(tmp=find_field_in_tables(thd,this,tables)))
       return 1;
     set_field(tmp);
   }
+  else if (thd && thd->set_query_id && field->query_id != thd->query_id)
+  {
+    /* We only come here in unions */
+    TABLE *table=field->table;
+    field->query_id=thd->query_id;
+    table->used_fields++;
+    table->used_keys&=field->part_of_key;
+  }
   return 0;
 }
 
@@ -330,6 +372,8 @@ void Item::init_make_field(Send_field *tmp_field,
   tmp_field->type=field_type;
   tmp_field->length=max_length;
   tmp_field->decimals=decimals;
+  if (unsigned_flag)
+    tmp_field->flags |= UNSIGNED_FLAG;
 }
 
 /* ARGSUSED */
@@ -345,6 +389,13 @@ void Item_int::make_field(Send_field *tmp_field)
   init_make_field(tmp_field,FIELD_TYPE_LONGLONG);
 }
 
+void Item_uint::make_field(Send_field *tmp_field)
+{
+  init_make_field(tmp_field,FIELD_TYPE_LONGLONG);
+  tmp_field->flags|= UNSIGNED_FLAG;
+  unsigned_flag=1;
+}
+
 void Item_real::make_field(Send_field *tmp_field)
 {
   init_make_field(tmp_field,FIELD_TYPE_DOUBLE);
@@ -355,17 +406,24 @@ void Item_string::make_field(Send_field *tmp_field)
   init_make_field(tmp_field,FIELD_TYPE_STRING);
 }
 
+void Item_empty_string::make_field(Send_field *tmp_field)
+{
+  init_make_field(tmp_field,FIELD_TYPE_VAR_STRING);
+}
+
 void Item_datetime::make_field(Send_field *tmp_field)
 {
   init_make_field(tmp_field,FIELD_TYPE_DATETIME);
 }
 
+
 void Item_null::make_field(Send_field *tmp_field)
 {
   init_make_field(tmp_field,FIELD_TYPE_NULL);
   tmp_field->length=4;
 }
 
+
 void Item_func::make_field(Send_field *tmp_field)
 {
   init_make_field(tmp_field, ((result_type() == STRING_RESULT) ?
@@ -394,7 +452,7 @@ void Item_field::save_org_in_field(Field *to)
   if (field->is_null())
   {
     null_value=1;
-    set_field_to_null_with_conversions(to);
+    set_field_to_null_with_conversions(to, 1);
   }
   else
   {
@@ -404,12 +462,12 @@ void Item_field::save_org_in_field(Field *to)
   }
 }
 
-bool Item_field::save_in_field(Field *to)
+bool Item_field::save_in_field(Field *to, bool no_conversions)
 {
   if (result_field->is_null())
   {
     null_value=1;
-    return set_field_to_null_with_conversions(to);
+    return set_field_to_null_with_conversions(to, no_conversions);
   }
   else
   {
@@ -436,9 +494,9 @@ bool Item_field::save_in_field(Field *to)
     1	 Field doesn't support NULL values and can't handle 'field = NULL'
 */   
 
-bool Item_null::save_in_field(Field *field)
+bool Item_null::save_in_field(Field *field, bool no_conversions)
 {
-  return set_field_to_null_with_conversions(field);
+  return set_field_to_null_with_conversions(field, no_conversions);
 }
 
 
@@ -460,7 +518,7 @@ bool Item_null::save_safe_in_field(Field *field)
 }
 
 
-bool Item::save_in_field(Field *field)
+bool Item::save_in_field(Field *field, bool no_conversions)
 {
   if (result_type() == STRING_RESULT ||
       result_type() == REAL_RESULT &&
@@ -471,7 +529,10 @@ bool Item::save_in_field(Field *field)
     str_value.set_quick(buff,sizeof(buff));
     result=val_str(&str_value);
     if (null_value)
-      return set_field_to_null_with_conversions(field);
+    {
+      str_value.set_quick(0, 0);
+      return set_field_to_null_with_conversions(field, no_conversions);
+    }
     field->set_notnull();
     field->store(result->ptr(),result->length());
     str_value.set_quick(0, 0);
@@ -488,7 +549,7 @@ bool Item::save_in_field(Field *field)
   {
     longlong nr=val_int();
     if (null_value)
-      return set_field_to_null_with_conversions(field);
+      return set_field_to_null_with_conversions(field, no_conversions);
     field->set_notnull();
     field->store(nr);
   }
@@ -496,7 +557,7 @@ bool Item::save_in_field(Field *field)
 }
 
 
-bool Item_string::save_in_field(Field *field)
+bool Item_string::save_in_field(Field *field, bool no_conversions)
 {
   String *result;
   result=val_str(&str_value);
@@ -507,7 +568,8 @@ bool Item_string::save_in_field(Field *field)
   return 0;
 }
 
-bool Item_int::save_in_field(Field *field)
+
+bool Item_int::save_in_field(Field *field, bool no_conversions)
 {
   longlong nr=val_int();
   if (null_value)
@@ -517,7 +579,7 @@ bool Item_int::save_in_field(Field *field)
   return 0;
 }
 
-bool Item_real::save_in_field(Field *field)
+bool Item_real::save_in_field(Field *field, bool no_conversions)
 {
   double nr=val();
   if (null_value)
@@ -540,6 +602,14 @@ inline uint char_val(char X)
 		 X-'a'+10);
 }
 
+/* In MySQL 4.1 this will always return STRING_RESULT */
+
+enum Item_result Item_varbinary::result_type () const
+{
+  return (current_thd->variables.new_mode) ? STRING_RESULT : INT_RESULT;
+}
+
+
 Item_varbinary::Item_varbinary(const char *str, uint str_length)
 {
   name=(char*) str-2;				// Lex makes this start with 0x
@@ -572,7 +642,7 @@ longlong Item_varbinary::val_int()
 }
 
 
-bool Item_varbinary::save_in_field(Field *field)
+bool Item_varbinary::save_in_field(Field *field, bool no_conversions)
 {
   field->set_notnull();
   if (field->result_type() == STRING_RESULT)
@@ -597,19 +667,19 @@ void Item_varbinary::make_field(Send_field *tmp_field)
 ** pack data in buffer for sending
 */
 
-bool Item::send(String *packet)
+bool Item::send(THD *thd, String *packet)
 {
   char buff[MAX_FIELD_WIDTH];
+  CONVERT *convert;
   String s(buff,sizeof(buff)),*res;
   if (!(res=val_str(&s)))
     return net_store_null(packet);
-  CONVERT *convert;
-  if ((convert=current_thd->convert_set))
+  if ((convert=thd->variables.convert_set))
     return convert->store(packet,res->ptr(),res->length());
   return net_store_data(packet,res->ptr(),res->length());
 }
 
-bool Item_null::send(String *packet)
+bool Item_null::send(THD *thd, String *packet)
 {
   return net_store_null(packet);
 }
@@ -623,29 +693,30 @@ bool Item_ref::fix_fields(THD *thd,TABLE_LIST *tables)
 {
   if (!ref)
   {
-    if (!(ref=find_item_in_list(this,thd->lex.item_list)))
+    if (!(ref=find_item_in_list(this,thd->lex.select->item_list)))
       return 1;
     max_length= (*ref)->max_length;
     maybe_null= (*ref)->maybe_null;
     decimals=	(*ref)->decimals;
     binary=	(*ref)->binary;
+    with_sum_func= (*ref)->with_sum_func;
   }
   return 0;
 }
 
+
 /*
-** If item is a const function, calculate it and return a const item
-** The original item is freed if not returned
+  If item is a const function, calculate it and return a const item
+  The original item is freed if not returned
 */
 
 Item_result item_cmp_type(Item_result a,Item_result b)
 {
   if (a == STRING_RESULT && b == STRING_RESULT)
     return STRING_RESULT;
-  else if (a == INT_RESULT && b == INT_RESULT)
+  if (a == INT_RESULT && b == INT_RESULT)
     return INT_RESULT;
-  else
-    return REAL_RESULT;
+  return REAL_RESULT;
 }
 
 
@@ -739,5 +810,6 @@ bool field_is_equal_to_item(Field *field,Item *item)
 #ifdef __GNUC__
 template class List<Item>;
 template class List_iterator<Item>;
+template class List_iterator_fast<Item>;
 template class List<List_item>;
 #endif
