@@ -119,9 +119,7 @@ public:
   const char *table_name;
   const char *field_name;
   Item_ident(const char *db_name_par,const char *table_name_par,
-	     const char *field_name_par)
-    :db_name(db_name_par),table_name(table_name_par),field_name(field_name_par)
-    { name = (char*) field_name_par; }
+	     const char *field_name_par);
   const char *full_name() const;
   unsigned int size_of() { return sizeof(*this);}  
 };
@@ -238,7 +236,7 @@ public:
     return 0;
   }
   void print(String *str);
-  unsigned int size_of() { return sizeof(*this);}  
+  unsigned int size_of() { return sizeof(*this);}
 };
 
 
@@ -501,6 +499,7 @@ public:
   String *val_str(String*);
   void make_field(Send_field *field) { item->make_field(field); }
   void copy();
+  bool save_in_field(Field *field, bool no_conversions);
   table_map used_tables() const { return (table_map) 1L; }
   bool const_item() const { return 0; }
   bool is_null() { return null_value; }
