@@ -1,15 +1,15 @@
 /* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
@@ -26,8 +26,8 @@
 
 /*
   Test functions
-  These returns 0LL if false and 1LL if true and null if some arg is null
-  'AND' and 'OR' never return null
+  Most of these  returns 0LL if false and 1LL if true and
+  NULL if some arg is NULL.
 */
 
 longlong Item_func_not::val_int()
@@ -37,18 +37,24 @@ longlong Item_func_not::val_int()
   return !null_value && value == 0 ? 1 : 0;
 }
 
+/*
+  Convert a constant expression or string to an integer.
+  This is done when comparing DATE's of different formats and
+  also when comparing bigint to strings (in which case the string
+  is converted once to a bigint).
+*/
 
 static bool convert_constant_item(Field *field, Item **item)
 {
   if ((*item)->const_item())
   {
-    (*item)->save_in_field(field);
-    if (!((*item)->null_value))
+    if (!(*item)->save_in_field(field, 1) &&
+	!((*item)->null_value))
     {
       Item *tmp=new Item_int_with_ref(field->val_int(), *item);
       if (tmp)
 	*item=tmp;
-      return 1;
+      return 1;					// Item was replaced
     }
   }
   return 0;
@@ -57,7 +63,7 @@ static bool convert_constant_item(Field *field, Item **item)
 
 void Item_bool_func2::fix_length_and_dec()
 {
-  max_length=1;
+  max_length=1;					// Function returns 0 or 1
 
   /*
     As some compare functions are generated after sql_yacc,
@@ -256,7 +262,7 @@ longlong Item_func_strcmp::val_int()
     null_value=1;
     return 0;
   }
-  int value=stringcmp(a,b);
+  int value= binary ? stringcmp(a,b) : sortcmp(a,b);
   null_value=0;
   return !value ? 0 : (value < 0 ? (longlong) -1 : (longlong) 1);
 }
@@ -286,10 +292,13 @@ void Item_func_interval::fix_length_and_dec()
     }
   }
   maybe_null=0; max_length=2;
-  used_tables_cache|=item->used_tables();
+  used_tables_cache|=     item->used_tables();
+  not_null_tables_cache&= item->not_null_tables();
   with_sum_func= with_sum_func || item->with_sum_func;
+  const_item_cache&=item->const_item();
 }
 
+
 void Item_func_interval::split_sum_func(List<Item> &fields)
 {
   if (item->with_sum_func && item->type() != SUM_FUNC_ITEM)
@@ -311,26 +320,25 @@ void Item_func_interval::split_sum_func(List<Item> &fields)
 
 longlong Item_func_interval::val_int()
 {
-  double value=item->val();
+  double value= item->val();
   if (item->null_value)
-    return -1;				// -1 if null /* purecov: inspected */
+    return -1;				// -1 if NULL
   if (intervals)
   {					// Use binary search to find interval
-    uint start,end;
-    start=0; end=arg_count-1;
+    uint start= 0, end= arg_count - 1;
     while (start != end)
     {
-      uint mid=(start+end+1)/2;
+      uint mid= (start + end + 1) / 2;
       if (intervals[mid] <= value)
-	start=mid;
+	start= mid;
       else
-	end=mid-1;
+	end= mid - 1;
     }
-    return (value < intervals[start]) ? 0 : start+1;
+    return (value < intervals[start]) ? 0 : start + 1;
   }
   if (args[0]->val() > value)
     return 0;
-  for (uint i=1 ; i < arg_count ; i++)
+  for (uint i= 1; i < arg_count; i++)
   {
     if (args[i]->val() > value)
       return i;
@@ -357,13 +365,19 @@ void Item_func_between::fix_length_and_dec()
   */
   if (!args[0] || !args[1] || !args[2])
     return;
-  cmp_type=args[0]->result_type();
-  if (args[0]->binary)
+  cmp_type=item_cmp_type(args[0]->result_type(),
+           item_cmp_type(args[1]->result_type(),
+                         args[2]->result_type()));
+  if (args[0]->binary | args[1]->binary | args[2]->binary)
     string_compare=stringcmp;
   else
     string_compare=sortcmp;
 
-  // Make a special case of compare with fields to get nicer DATE comparisons
+  /*
+    Make a special case of compare with date/time and longlong fields.
+    They are compared as integers, so for const item this time-consuming
+    conversion can be done only once, not for every single comparison
+  */
   if (args[0]->type() == FIELD_ITEM)
   {
     Field *field=((Item_field*) args[0])->field;
@@ -445,15 +459,28 @@ longlong Item_func_between::val_int()
   return 0;
 }
 
+static Item_result item_store_type(Item_result a,Item_result b)
+{
+  if (a == STRING_RESULT || b == STRING_RESULT)
+    return STRING_RESULT;
+  if (a == REAL_RESULT || b == REAL_RESULT)
+    return REAL_RESULT;
+  return INT_RESULT;
+}
+
 void
 Item_func_ifnull::fix_length_and_dec()
 {
   maybe_null=args[1]->maybe_null;
   max_length=max(args[0]->max_length,args[1]->max_length);
   decimals=max(args[0]->decimals,args[1]->decimals);
-  cached_result_type=args[0]->result_type();
+  if ((cached_result_type=item_store_type(args[0]->result_type(),
+					  args[1]->result_type())) !=
+      REAL_RESULT)
+    decimals= 0;
 }
 
+
 double
 Item_func_ifnull::val()
 {
@@ -499,6 +526,7 @@ Item_func_ifnull::val_str(String *str)
   return res;
 }
 
+
 void
 Item_func_if::fix_length_and_dec()
 {
@@ -507,17 +535,32 @@ Item_func_if::fix_length_and_dec()
   decimals=max(args[1]->decimals,args[2]->decimals);
   enum Item_result arg1_type=args[1]->result_type();
   enum Item_result arg2_type=args[2]->result_type();
+  bool null1=args[1]->null_value;
+  bool null2=args[2]->null_value;
 
-  binary=1;
-  if (arg1_type == STRING_RESULT || arg2_type == STRING_RESULT)
+  if (null1)
+  {
+    cached_result_type= arg2_type;
+    binary= args[2]->binary;
+  }
+  else if (null2)
+  {
+    cached_result_type= arg1_type;
+    binary= args[1]->binary;
+  }
+  else if (arg1_type == STRING_RESULT || arg2_type == STRING_RESULT)
   {
     cached_result_type = STRING_RESULT;
     binary=args[1]->binary | args[2]->binary;
   }
-  else if (arg1_type == REAL_RESULT || arg2_type == REAL_RESULT)
-    cached_result_type = REAL_RESULT;
   else
-    cached_result_type=arg1_type;		// Should be INT_RESULT
+  {
+    binary=1;					// Number
+    if (arg1_type == REAL_RESULT || arg2_type == REAL_RESULT)
+      cached_result_type = REAL_RESULT;
+    else
+      cached_result_type=arg1_type;		// Should be INT_RESULT
+  }
 }
 
 
@@ -563,7 +606,7 @@ Item_func_nullif::fix_length_and_dec()
 }
 
 /*
-  nullif() returns NULL if arguments are different, else it returns the
+  nullif () returns NULL if arguments are different, else it returns the
   first argument.
   Note that we have to evaluate the first argument twice as the compare
   may have been done with a different type than return value
@@ -573,7 +616,7 @@ double
 Item_func_nullif::val()
 {
   double value;
-  if (!(this->*cmp_func)() || null_value)
+  if (!(this->*cmp_func)())
   {
     null_value=1;
     return 0.0;
@@ -587,7 +630,7 @@ longlong
 Item_func_nullif::val_int()
 {
   longlong value;
-  if (!(this->*cmp_func)() || null_value)
+  if (!(this->*cmp_func)())
   {
     null_value=1;
     return 0;
@@ -601,7 +644,7 @@ String *
 Item_func_nullif::val_str(String *str)
 {
   String *res;
-  if (!(this->*cmp_func)() || null_value)
+  if (!(this->*cmp_func)())
   {
     null_value=1;
     return 0;
@@ -840,8 +883,9 @@ String *Item_func_coalesce::val_str(String *str)
   null_value=0;
   for (uint i=0 ; i < arg_count ; i++)
   {
-    if (args[i]->val_str(str) != NULL)
-      return args[i]->val_str(str);
+    String *res;
+    if ((res=args[i]->val_str(str)))
+      return res;
   }
   null_value=1;
   return 0;
@@ -1001,15 +1045,18 @@ void Item_func_in::fix_length_and_dec()
       array= new in_double(arg_count);
       break;
     }
-    uint j=0;
-    for (uint i=0 ; i < arg_count ; i++)
+    if (array && !(current_thd->fatal_error))		// If not EOM
     {
-      array->set(j,args[i]);
-      if (!args[i]->null_value)			// Skipp NULL values
-	j++;
+      uint j=0;
+      for (uint i=0 ; i < arg_count ; i++)
+      {
+	array->set(j,args[i]);
+	if (!args[i]->null_value)			// Skip NULL values
+	  j++;
+      }
+      if ((array->used_count=j))
+	array->sort();
     }
-    if ((array->used_count=j))
-      array->sort();
   }
   else
   {
@@ -1030,8 +1077,9 @@ void Item_func_in::fix_length_and_dec()
   }
   maybe_null= item->maybe_null;
   max_length=2;
-  used_tables_cache|=item->used_tables();
-  const_item_cache&=item->const_item();
+  used_tables_cache|=     item->used_tables();
+  not_null_tables_cache&= item->not_null_tables();
+  const_item_cache&=      item->const_item();
 }
 
 
@@ -1128,17 +1176,27 @@ Item_cond::fix_fields(THD *thd,TABLE_LIST *tables)
 {
   List_iterator<Item> li(list);
   Item *item;
+#ifndef EMBEDDED_LIBRARY			// Avoid compiler warning
   char buff[sizeof(char*)];			// Max local vars in function
-  used_tables_cache=0;
-  const_item_cache=0;
+#endif
+
+  not_null_tables_cache= used_tables_cache= 0;
+  const_item_cache= 0;
+  /*
+    and_table_cache is the value that Item_cond_or() returns for
+    not_null_tables()
+  */
+  and_tables_cache= ~(table_map) 0;
 
   if (thd && check_stack_overrun(thd,buff))
-    return 0;					// Fatal error flag is set!
+    return 1;					// Fatal error flag is set!
   while ((item=li++))
   {
+    table_map tmp_table_map;
     while (item->type() == Item::COND_ITEM &&
 	   ((Item_cond*) item)->functype() == functype())
     {						// Identical function
+
       li.replace(((Item_cond*) item)->list);
       ((Item_cond*) item)->list.empty();
 #ifdef DELETE_ITEMS
@@ -1146,11 +1204,16 @@ Item_cond::fix_fields(THD *thd,TABLE_LIST *tables)
 #endif
       item= *li.ref();				// new current item
     }
+    if (abort_on_null)
+      item->top_level_item();
     if (item->fix_fields(thd,tables))
       return 1; /* purecov: inspected */
-    used_tables_cache|=item->used_tables();
-    with_sum_func= with_sum_func || item->with_sum_func;
-    const_item_cache&=item->const_item();
+    used_tables_cache|=     item->used_tables();
+    tmp_table_map=	    item->not_null_tables();
+    not_null_tables_cache|= tmp_table_map;
+    and_tables_cache&=      tmp_table_map;
+    const_item_cache&=	    item->const_item();
+    with_sum_func=	    with_sum_func || item->with_sum_func;
     if (item->maybe_null)
       maybe_null=1;
   }
@@ -1189,17 +1252,19 @@ Item_cond::used_tables() const
   return used_tables_cache;
 }
 
+
 void Item_cond::update_used_tables()
 {
+  List_iterator_fast<Item> li(list);
+  Item *item;
+
   used_tables_cache=0;
   const_item_cache=1;
-  List_iterator<Item> li(list);
-  Item *item;
   while ((item=li++))
   {
     item->update_used_tables();
-    used_tables_cache|=item->used_tables();
-    const_item_cache&= item->const_item();
+    used_tables_cache|= item->used_tables();
+    const_item_cache&=  item->const_item();
   }
 }
 
@@ -1207,7 +1272,7 @@ void Item_cond::update_used_tables()
 void Item_cond::print(String *str)
 {
   str->append('(');
-  List_iterator<Item> li(list);
+  List_iterator_fast<Item> li(list);
   Item *item;
   if ((item=li++))
     item->print(str);
@@ -1221,31 +1286,44 @@ void Item_cond::print(String *str)
   str->append(')');
 }
 
+/*
+  Evalution of AND(expr, expr, expr ...)
+
+  NOTES:
+    abort_if_null is set for AND expressions for which we don't care if the
+    result is NULL or 0. This is set for:
+    - WHERE clause
+    - HAVING clause
+    - IF(expression)
+
+  RETURN VALUES
+    1  If all expressions are true
+    0  If all expressions are false or if we find a NULL expression and
+       'abort_on_null' is set.
+    NULL if all expression are either 1 or NULL
+*/
+
 
 longlong Item_cond_and::val_int()
 {
-  List_iterator<Item> li(list);
+  List_iterator_fast<Item> li(list);
   Item *item;
+  null_value= 0;
   while ((item=li++))
   {
     if (item->val_int() == 0)
     {
-      /*
-	TODO: In case of NULL, ANSI would require us to continue evaluation
-	until we get a FALSE value or run out of values; This would
-	require a lot of unnecessary evaluation, which we skip for now
-      */
-      null_value=item->null_value;
-      return 0;
+      if (abort_on_null || !(null_value= item->null_value))
+	return 0;				// return FALSE
     }
   }
-  null_value=0;
-  return 1;
+  return null_value ? 0 : 1;
 }
 
+
 longlong Item_cond_or::val_int()
 {
-  List_iterator<Item> li(list);
+  List_iterator_fast<Item> li(list);
   Item *item;
   null_value=0;
   while ((item=li++))
@@ -1290,12 +1368,16 @@ Item *and_expressions(Item *a, Item *b, Item **org_item)
   {
     Item_cond *res;
     if ((res= new Item_cond_and(a, (Item*) b)))
+    {
       res->used_tables_cache= a->used_tables() | b->used_tables();
+      res->not_null_tables_cache= a->not_null_tables() | b->not_null_tables();
+    }
     return res;
   }
   if (((Item_cond_and*) a)->add((Item*) b))
     return 0;
   ((Item_cond_and*) a)->used_tables_cache|= b->used_tables();
+  ((Item_cond_and*) a)->not_null_tables_cache|= b->not_null_tables();
   return a;
 }
 
@@ -1308,14 +1390,12 @@ longlong Item_func_isnull::val_int()
   */
   if (!used_tables_cache)
     return cached_value;
-  (void) args[0]->val();
-  return (args[0]->null_value) ? 1 : 0;
+  return args[0]->is_null() ? 1: 0;
 }
 
 longlong Item_func_isnotnull::val_int()
 {
-  (void) args[0]->val();
-  return !(args[0]->null_value) ? 1 : 0;
+  return args[0]->is_null() ? 0 : 1;
 }
 
 
@@ -1325,23 +1405,23 @@ void Item_func_like::fix_length_and_dec()
   //  cmp_type=STRING_RESULT;			// For quick select
 }
 
-
 longlong Item_func_like::val_int()
 {
-  String *res,*res2;
-  res=args[0]->val_str(&tmp_value1);
+  String* res = args[0]->val_str(&tmp_value1);
   if (args[0]->null_value)
   {
     null_value=1;
     return 0;
   }
-  res2=args[1]->val_str(&tmp_value2);
+  String* res2 = args[1]->val_str(&tmp_value2);
   if (args[1]->null_value)
   {
     null_value=1;
     return 0;
   }
   null_value=0;
+  if (canDoTurboBM)
+    return turboBM_matches(res->ptr(), res->length()) ? 1 : 0;
   if (binary)
     return wild_compare(*res,*res2,escape) ? 0 : 1;
   else
@@ -1353,18 +1433,75 @@ longlong Item_func_like::val_int()
 
 Item_func::optimize_type Item_func_like::select_optimize() const
 {
-  if (args[1]->type() == STRING_ITEM)
+  if (args[1]->const_item())
   {
-    if (((Item_string *) args[1])->str_value[0] != wild_many)
+    String* res2= args[1]->val_str((String *)&tmp_value2);
+
+    if (!res2)
+      return OPTIMIZE_NONE;
+
+    if (*res2->ptr() != wild_many)
     {
-      if ((args[0]->result_type() != STRING_RESULT) ||
-	  ((Item_string *) args[1])->str_value[0] != wild_one)
+      if (args[0]->result_type() != STRING_RESULT || *res2->ptr() != wild_one)
 	return OPTIMIZE_OP;
     }
   }
   return OPTIMIZE_NONE;
 }
 
+bool Item_func_like::fix_fields(THD *thd,struct st_table_list *tlist)
+{
+  if (Item_bool_func2::fix_fields(thd, tlist))
+    return 1;
+
+  /*
+    TODO--we could do it for non-const, but we'd have to
+    recompute the tables for each row--probably not worth it.
+  */
+  if (args[1]->const_item() && !(specialflag & SPECIAL_NO_NEW_FUNC))
+  {
+    String* res2 = args[1]->val_str(&tmp_value2);
+    if (!res2)
+      return 0;					// Null argument
+
+    const size_t len   = res2->length();
+    const char*  first = res2->ptr();
+    const char*  last  = first + len - 1;
+    /*
+      len must be > 2 ('%pattern%')
+      heuristic: only do TurboBM for pattern_len > 2
+    */
+
+    if (len > MIN_TURBOBM_PATTERN_LEN + 2 &&
+	*first == wild_many &&
+	*last  == wild_many)
+    {
+      const char* tmp = first + 1;
+      for (; *tmp != wild_many && *tmp != wild_one && *tmp != escape; tmp++) ;
+#ifdef USE_MB
+      canDoTurboBM = (tmp == last) && !use_mb(default_charset_info);
+#else
+      canDoTurboBM = tmp == last;
+#endif
+    }
+
+    if (canDoTurboBM)
+    {
+      pattern     = first + 1;
+      pattern_len = len - 2;
+      DBUG_PRINT("info", ("Initializing pattern: '%s'", first));
+      int *suff = (int*) thd->alloc(sizeof(int)*((pattern_len + 1)*2+
+						 alphabet_size));
+      bmGs      = suff + pattern_len + 1;
+      bmBc      = bmGs + pattern_len + 1;
+      turboBM_compute_good_suffix_shifts(suff);
+      turboBM_compute_bad_character_shifts();
+      DBUG_PRINT("info",("done"));
+    }
+  }
+  return 0;
+}
+
 #ifdef USE_REGEX
 
 bool
@@ -1376,6 +1513,8 @@ Item_func_regex::fix_fields(THD *thd,TABLE_LIST *tables)
   max_length=1; decimals=0;
   binary=args[0]->binary || args[1]->binary;
   used_tables_cache=args[0]->used_tables() | args[1]->used_tables();
+  not_null_tables_cache= (args[0]->not_null_tables() |
+			  args[1]->not_null_tables());
   const_item_cache=args[0]->const_item() && args[1]->const_item();
   if (!regex_compiled && args[1]->const_item())
   {
@@ -1404,7 +1543,6 @@ Item_func_regex::fix_fields(THD *thd,TABLE_LIST *tables)
   return 0;
 }
 
-
 longlong Item_func_regex::val_int()
 {
   char buff[MAX_FIELD_WIDTH];
@@ -1461,3 +1599,259 @@ Item_func_regex::~Item_func_regex()
 }
 
 #endif /* USE_REGEX */
+
+
+#ifdef LIKE_CMP_TOUPPER
+#define likeconv(A) (uchar) toupper(A)
+#else
+#define likeconv(A) (uchar) my_sort_order[(uchar) (A)]
+#endif
+
+
+/**********************************************************************
+  turboBM_compute_suffixes()
+  Precomputation dependent only on pattern_len.
+**********************************************************************/
+
+void Item_func_like::turboBM_compute_suffixes(int *suff)
+{
+  const int   plm1 = pattern_len - 1;
+  int            f = 0;
+  int            g = plm1;
+  int *const splm1 = suff + plm1;
+
+  *splm1 = pattern_len;
+
+  if (binary)
+  {
+    int i;
+    for (i = pattern_len - 2; i >= 0; i--)
+    {
+      int tmp = *(splm1 + i - f);
+      if (g < i && tmp < i - g)
+	suff[i] = tmp;
+      else
+      {
+	if (i < g)
+	  g = i; // g = min(i, g)
+	f = i;
+	while (g >= 0 && pattern[g] == pattern[g + plm1 - f])
+	  g--;
+	suff[i] = f - g;
+      }
+    }
+  }
+  else
+  {
+    int i;
+    for (i = pattern_len - 2; 0 <= i; --i)
+    {
+      int tmp = *(splm1 + i - f);
+      if (g < i && tmp < i - g)
+	suff[i] = tmp;
+      else
+      {
+	if (i < g)
+	  g = i; // g = min(i, g)
+	f = i;
+	while (g >= 0 &&
+	       likeconv(pattern[g]) == likeconv(pattern[g + plm1 - f]))
+	  g--;
+	suff[i] = f - g;
+      }
+    }
+  }
+}
+
+
+/**********************************************************************
+   turboBM_compute_good_suffix_shifts()
+   Precomputation dependent only on pattern_len.
+**********************************************************************/
+
+void Item_func_like::turboBM_compute_good_suffix_shifts(int *suff)
+{
+  turboBM_compute_suffixes(suff);
+
+  int *end = bmGs + pattern_len;
+  int *k;
+  for (k = bmGs; k < end; k++)
+    *k = pattern_len;
+
+  int tmp;
+  int i;
+  int j          = 0;
+  const int plm1 = pattern_len - 1;
+  for (i = plm1; i > -1; i--)
+  {
+    if (suff[i] == i + 1)
+    {
+      for (tmp = plm1 - i; j < tmp; j++)
+      {
+	int *tmp2 = bmGs + j;
+	if (*tmp2 == pattern_len)
+	  *tmp2 = tmp;
+      }
+    }
+  }
+
+  int *tmp2;
+  for (tmp = plm1 - i; j < tmp; j++)
+  {
+    tmp2 = bmGs + j;
+    if (*tmp2 == pattern_len)
+      *tmp2 = tmp;
+  }
+
+  tmp2 = bmGs + plm1;
+  for (i = 0; i <= pattern_len - 2; i++)
+    *(tmp2 - suff[i]) = plm1 - i;
+}
+
+
+/**********************************************************************
+   turboBM_compute_bad_character_shifts()
+   Precomputation dependent on pattern_len.
+**********************************************************************/
+
+void Item_func_like::turboBM_compute_bad_character_shifts()
+{
+  int *i;
+  int *end = bmBc + alphabet_size;
+  for (i = bmBc; i < end; i++)
+    *i = pattern_len;
+
+  int j;
+  const int plm1 = pattern_len - 1;
+  if (binary)
+  {
+    for (j = 0; j < plm1; j++)
+      bmBc[(uint) (uchar) pattern[j]] = plm1 - j;
+  }
+  else
+  {
+    for (j = 0; j < plm1; j++)
+      bmBc[(uint) likeconv(pattern[j])] = plm1 - j;
+  }
+}
+
+
+/**********************************************************************
+  turboBM_matches()
+  Search for pattern in text, returns true/false for match/no match
+**********************************************************************/
+
+bool Item_func_like::turboBM_matches(const char* text, int text_len) const
+{
+  register int bcShift;
+  register int turboShift;
+  int shift = pattern_len;
+  int j     = 0;
+  int u     = 0;
+
+  const int plm1=  pattern_len - 1;
+  const int tlmpl= text_len - pattern_len;
+
+  /* Searching */
+  if (binary)
+  {
+    while (j <= tlmpl)
+    {
+      register int i= plm1;
+      while (i >= 0 && pattern[i] == text[i + j])
+      {
+	i--;
+	if (i == plm1 - shift)
+	  i-= u;
+      }
+      if (i < 0)
+	return 1;
+
+      register const int v = plm1 - i;
+      turboShift = u - v;
+      bcShift    = bmBc[(uint) (uchar) text[i + j]] - plm1 + i;
+      shift      = max(turboShift, bcShift);
+      shift      = max(shift, bmGs[i]);
+      if (shift == bmGs[i])
+	u = min(pattern_len - shift, v);
+      else
+      {
+	if (turboShift < bcShift)
+	  shift = max(shift, u + 1);
+	u = 0;
+      }
+      j+= shift;
+    }
+    return 0;
+  }
+  else
+  {
+    while (j <= tlmpl)
+    {
+      register int i = plm1;
+      while (i >= 0 && likeconv(pattern[i]) == likeconv(text[i + j]))
+      {
+	i--;
+	if (i == plm1 - shift)
+	  i-= u;
+      }
+      if (i < 0)
+	return 1;
+
+      register const int v = plm1 - i;
+      turboShift = u - v;
+      bcShift    = bmBc[(uint) likeconv(text[i + j])] - plm1 + i;
+      shift      = max(turboShift, bcShift);
+      shift      = max(shift, bmGs[i]);
+      if (shift == bmGs[i])
+	u = min(pattern_len - shift, v);
+      else
+      {
+	if (turboShift < bcShift)
+	  shift = max(shift, u + 1);
+	u = 0;
+      }
+      j+= shift;
+    }
+    return 0;
+  }
+}
+
+
+/*
+  Make a logical XOR of the arguments.
+
+  SYNOPSIS
+    val_int()
+
+  DESCRIPTION
+  If either operator is NULL, return NULL.
+
+  NOTE
+    As we don't do any index optimization on XOR this is not going to be
+    very fast to use.
+
+  TODO (low priority)
+    Change this to be optimized as:
+      A XOR B   ->  (A) == 1 AND (B) <> 1) OR (A <> 1 AND (B) == 1)
+    To be able to do this, we would however first have to extend the MySQL
+    range optimizer to handle OR better.
+*/
+
+longlong Item_cond_xor::val_int()
+{
+  List_iterator<Item> li(list);
+  Item *item;
+  int result=0;	
+  null_value=0;
+  while ((item=li++))
+  {
+    result^= (item->val_int() != 0);
+    if (item->null_value)
+    {
+      null_value=1;
+      return 0;
+    }
+  }
+  return (longlong) result;
+}
