@@ -1,15 +1,15 @@
 /* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
@@ -17,7 +17,7 @@
 
 /* This file defines all string functions
 ** Warning: Some string functions doesn't always put and end-null on a String
-** (This shouldn't be neaded)
+** (This shouldn't be needed)
 */
 
 #ifdef __GNUC__
@@ -27,11 +27,12 @@
 #include "mysql_priv.h"
 #include "sql_acl.h"
 #include <m_ctype.h>
-#ifdef HAVE_CRYPT_H
-#include <crypt.h>
-#endif
-
+#ifdef HAVE_OPENSSL
+#include <openssl/des.h>
+#endif /* HAVE_OPENSSL */
 #include "md5.h"
+#include "sha1.h"
+#include "my_aes.h"
 
 String empty_string("");
 
@@ -40,7 +41,7 @@ uint nr_of_decimals(const char *str)
   if ((str=strchr(str,'.')))
   {
     const char *start= ++str;
-    for ( ; isdigit(*str) ; str++) ;
+    for (; isdigit(*str) ; str++) ;
     return (uint) (str-start);
   }
   return 0;
@@ -66,14 +67,18 @@ String *Item_func_md5::val_str(String *str)
   String * sptr= args[0]->val_str(str);
   if (sptr)
   {
-    MD5_CTX context;
+    my_MD5_CTX context;
     unsigned char digest[16];
 
     null_value=0;
-    MD5Init (&context);
-    MD5Update (&context,(unsigned char *) sptr->ptr(), sptr->length());
-    MD5Final (digest, &context);
-    str->alloc(32);				// Ensure that memory is free
+    my_MD5Init (&context);
+    my_MD5Update (&context,(unsigned char *) sptr->ptr(), sptr->length());
+    my_MD5Final (digest, &context);
+    if (str->alloc(32))				// Ensure that memory is free
+    {
+      null_value=1;
+      return 0;
+    }
     sprintf((char *) str->ptr(),
 	    "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
 	    digest[0], digest[1], digest[2], digest[3],
@@ -93,10 +98,123 @@ void Item_func_md5::fix_length_and_dec()
    max_length=32;
 }
 
+
+String *Item_func_sha::val_str(String *str)
+{
+  String * sptr= args[0]->val_str(str);
+  if (sptr)  /* If we got value different from NULL */
+  {
+    SHA1_CONTEXT context;  /* Context used to generate SHA1 hash */
+    /* Temporary buffer to store 160bit digest */
+    uint8 digest[SHA1_HASH_SIZE];
+    sha1_reset(&context);  /* We do not have to check for error here */
+    /* No need to check error as the only case would be too long message */
+    sha1_input(&context,(const unsigned char *) sptr->ptr(), sptr->length());
+    /* Ensure that memory is free and we got result */
+    if (!( str->alloc(SHA1_HASH_SIZE*2) || (sha1_result(&context,digest))))
+    {
+      sprintf((char *) str->ptr(),
+      "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x\
+%02x%02x%02x%02x%02x%02x%02x%02x",
+           digest[0], digest[1], digest[2], digest[3],
+           digest[4], digest[5], digest[6], digest[7],
+           digest[8], digest[9], digest[10], digest[11],
+           digest[12], digest[13], digest[14], digest[15],
+           digest[16], digest[17], digest[18], digest[19]);
+	   
+      str->length((uint)  SHA1_HASH_SIZE*2);
+      null_value=0;
+      return str;
+    }
+  }    
+  null_value=1;
+  return 0;
+}
+
+void Item_func_sha::fix_length_and_dec()
+{
+   max_length=SHA1_HASH_SIZE*2; // size of hex representation of hash
+}
+
+
+/* Implementation of AES encryption routines */
+
+String *Item_func_aes_encrypt::val_str(String *str)
+{
+  char key_buff[80];
+  String tmp_key_value(key_buff, sizeof(key_buff));
+  String *sptr= args[0]->val_str(str);			// String to encrypt
+  String *key=  args[1]->val_str(&tmp_key_value);	// key
+  int aes_length;
+  if (sptr && key) // we need both arguments to be not NULL
+  {
+    null_value=0;
+    aes_length=my_aes_get_size(sptr->length()); // Calculate result length
+    
+    if (!str_value.alloc(aes_length))		// Ensure that memory is free
+    {
+      // finally encrypt directly to allocated buffer.
+      if (my_aes_encrypt(sptr->ptr(),sptr->length(), (char*) str_value.ptr(),
+			 key->ptr(), key->length()) == aes_length)
+      {		     
+	// We got the expected result length
+	str_value.length((uint) aes_length);
+	return &str_value;
+      }
+    }
+  }
+  null_value=1;
+  return 0;
+}
+
+
+void Item_func_aes_encrypt::fix_length_and_dec()
+{
+  max_length=my_aes_get_size(args[0]->max_length);
+}
+
+
+String *Item_func_aes_decrypt::val_str(String *str)
+{
+  char key_buff[80];
+  String tmp_key_value(key_buff, sizeof(key_buff)), *sptr, *key;
+  DBUG_ENTER("Item_func_aes_decrypt::val_str");
+
+  sptr= args[0]->val_str(str);			// String to decrypt
+  key=  args[1]->val_str(&tmp_key_value);	// Key
+  if (sptr && key)  			// Need to have both arguments not NULL
+  {
+    null_value=0;
+    if (!str_value.alloc(sptr->length()))  // Ensure that memory is free
+    {
+      // finally decrypt directly to allocated buffer.
+      int length;
+      length=my_aes_decrypt(sptr->ptr(), sptr->length(),
+			    (char*) str_value.ptr(),
+                            key->ptr(), key->length());
+      if (length >= 0)  // if we got correct data data
+      {      
+        str_value.length((uint) length);
+        DBUG_RETURN(&str_value);
+      }
+    }
+  }
+  // Bad parameters. No memory or bad data will all go here
+  null_value=1;
+  DBUG_RETURN(0);
+}
+
+
+void Item_func_aes_decrypt::fix_length_and_dec()
+{
+   max_length=args[0]->max_length;
+}
+
+
 /*
-** Concatinate args with the following premissess
-** If only one arg which is ok, return value of arg
-** Don't reallocate val_str() if not absolute necessary.
+  Concatenate args with the following premises:
+  If only one arg (which is ok), return value of arg
+  Don't reallocate val_str() if not absolute necessary.
 */
 
 String *Item_func_concat::val_str(String *str)
@@ -121,7 +239,8 @@ String *Item_func_concat::val_str(String *str)
 	goto null;
       if (res2->length() == 0)
 	continue;
-      if (res->length()+res2->length() > max_allowed_packet)
+      if (res->length()+res2->length() >
+	  current_thd->variables.max_allowed_packet)
 	goto null;				// Error check
       if (res->alloced_length() >= res->length()+res2->length())
       {						// Use old buffer
@@ -199,11 +318,166 @@ void Item_func_concat::fix_length_and_dec()
   }
 }
 
+/* 
+  Function des_encrypt() by tonu@spam.ee & monty
+  Works only if compiled with OpenSSL library support.
+  This returns a binary string where first character is CHAR(128 | key-number).
+  If one uses a string key key_number is 127.
+  Encryption result is longer than original by formula:
+  new_length= org_length + (8-(org_length % 8))+1
+*/
+
+String *Item_func_des_encrypt::val_str(String *str)
+{
+#ifdef HAVE_OPENSSL
+  DES_cblock ivec;
+  struct st_des_keyblock keyblock;
+  struct st_des_keyschedule keyschedule;
+  const char *append_str="********";
+  uint key_number, res_length, tail;
+  String *res= args[0]->val_str(str);
+
+  if ((null_value=args[0]->null_value))
+    return 0;
+  if ((res_length=res->length()) == 0)
+    return &empty_string;
+
+  if (arg_count == 1)
+  {
+    /* Protect against someone doing FLUSH DES_KEY_FILE */
+    VOID(pthread_mutex_lock(&LOCK_des_key_file));
+    keyschedule= des_keyschedule[key_number=des_default_key];
+    VOID(pthread_mutex_unlock(&LOCK_des_key_file));
+  }
+  else if (args[1]->result_type() == INT_RESULT)
+  {
+    key_number= (uint) args[1]->val_int();
+    if (key_number > 9)
+      goto error;
+    VOID(pthread_mutex_lock(&LOCK_des_key_file));
+    keyschedule= des_keyschedule[key_number];
+    VOID(pthread_mutex_unlock(&LOCK_des_key_file));
+  }
+  else
+  {
+    String *keystr=args[1]->val_str(&tmp_value);
+    if (!keystr)
+      goto error;
+    key_number=127;				// User key string
+
+    /* We make good 24-byte (168 bit) key from given plaintext key with MD5 */
+    bzero((char*) &ivec,sizeof(ivec));
+    EVP_BytesToKey(EVP_des_ede3_cbc(),EVP_md5(),NULL,
+		   (uchar*) keystr->ptr(), (int) keystr->length(),
+		   1, (uchar*) &keyblock,ivec);
+    DES_set_key_unchecked(&keyblock.key1,&keyschedule.ks1);
+    DES_set_key_unchecked(&keyblock.key2,&keyschedule.ks2);
+    DES_set_key_unchecked(&keyblock.key3,&keyschedule.ks3);
+  }
+
+  /* 
+     The problem: DES algorithm requires original data to be in 8-bytes
+     chunks. Missing bytes get filled with '*'s and result of encryption 
+     can be up to 8 bytes longer than original string. When decrypted, 
+     we do not know the size of original string :(
+     We add one byte with value 0x1..0x8 as the last byte of the padded
+     string marking change of string length.
+  */
+
+  tail=  (8-(res_length) % 8);			// 1..8 marking extra length
+  res_length+=tail;
+  if (tail && res->append(append_str, tail) || tmp_value.alloc(res_length+1))
+    goto error;
+  (*res)[res_length-1]=tail;			// save extra length
+  tmp_value.length(res_length+1);
+  tmp_value[0]=(char) (128 | key_number);
+  // Real encryption
+  bzero((char*) &ivec,sizeof(ivec));
+  DES_ede3_cbc_encrypt((const uchar*) (res->ptr()),
+		       (uchar*) (tmp_value.ptr()+1),
+		       res_length,
+		       &keyschedule.ks1,
+		       &keyschedule.ks2,
+		       &keyschedule.ks3,
+		       &ivec, TRUE);
+  return &tmp_value;
+
+error:
+#endif	/* HAVE_OPENSSL */
+  null_value=1;
+  return 0;
+}
+
+
+String *Item_func_des_decrypt::val_str(String *str)
+{
+#ifdef HAVE_OPENSSL
+  DES_key_schedule ks1, ks2, ks3;
+  DES_cblock ivec;
+  struct st_des_keyblock keyblock;
+  struct st_des_keyschedule keyschedule;
+  String *res= args[0]->val_str(str);
+  uint length=res->length(),tail;
+
+  if ((null_value=args[0]->null_value))
+    return 0;
+  length=res->length();
+  if (length < 9 || (length % 8) != 1 || !((*res)[0] & 128))
+    return res;				// Skip decryption if not encrypted
+
+  if (arg_count == 1)			// If automatic uncompression
+  {
+    uint key_number=(uint) (*res)[0] & 127;
+    // Check if automatic key and that we have privilege to uncompress using it
+    if (!(current_thd->master_access & SUPER_ACL) || key_number > 9)
+      goto error;
+    VOID(pthread_mutex_lock(&LOCK_des_key_file));
+    keyschedule= des_keyschedule[key_number];
+    VOID(pthread_mutex_unlock(&LOCK_des_key_file));
+  }
+  else
+  {
+    // We make good 24-byte (168 bit) key from given plaintext key with MD5
+    String *keystr=args[1]->val_str(&tmp_value);
+    if (!keystr)
+      goto error;
+
+    bzero((char*) &ivec,sizeof(ivec));
+    EVP_BytesToKey(EVP_des_ede3_cbc(),EVP_md5(),NULL,
+		   (uchar*) keystr->ptr(),(int) keystr->length(),
+		   1,(uchar*) &keyblock,ivec);
+    // Here we set all 64-bit keys (56 effective) one by one
+    DES_set_key_unchecked(&keyblock.key1,&keyschedule.ks1);
+    DES_set_key_unchecked(&keyblock.key2,&keyschedule.ks2);
+    DES_set_key_unchecked(&keyblock.key3,&keyschedule.ks3); 
+  }
+  if (tmp_value.alloc(length-1))
+    goto error;
+
+  bzero((char*) &ivec,sizeof(ivec));
+  DES_ede3_cbc_encrypt((const uchar*) res->ptr()+1,
+		       (uchar*) (tmp_value.ptr()),
+		       length-1,
+		       &keyschedule.ks1,
+		       &keyschedule.ks2,
+		       &keyschedule.ks3,
+		       &ivec, FALSE);
+  /* Restore old length of key */
+  if ((tail=(uint) (uchar) tmp_value[length-2]) > 8)
+    goto error;					// Wrong key
+  tmp_value.length(length-1-tail);
+  return &tmp_value;
+
+error:
+#endif	/* HAVE_OPENSSL */
+  null_value=1;
+  return 0;
+}
 
 
 /* 
-** concat with separator. First arg is the separator
-** concat_ws takes at least two arguments.
+  concat with separator. First arg is the separator
+  concat_ws takes at least two arguments.
 */
 
 String *Item_func_concat_ws::val_str(String *str)
@@ -221,21 +495,21 @@ String *Item_func_concat_ws::val_str(String *str)
   str->length(0);				// QQ; Should be removed
   res=str;
 
-  // Skip until non-null and non-empty argument is found.
+  // Skip until non-null argument is found.
   // If not, return the empty string
   for (i=0; i < arg_count; i++)
-    if ((res= args[i]->val_str(str)) && res->length())
+    if ((res= args[i]->val_str(str)))
       break;
   if (i ==  arg_count)
     return &empty_string;
 
   for (i++; i < arg_count ; i++)
   {
-    if (!(res2= args[i]->val_str(use_as_buff)) || !res2->length())
-      continue;					// Skipp NULL and empty string
+    if (!(res2= args[i]->val_str(use_as_buff)))
+      continue;					// Skip NULL
 
     if (res->length() + sep_str->length() + res2->length() >
-	max_allowed_packet)
+	current_thd->variables.max_allowed_packet)
       goto null;				// Error check
     if (res->alloced_length() >=
 	res->length() + sep_str->length() + res2->length())
@@ -332,9 +606,10 @@ void Item_func_concat_ws::fix_length_and_dec()
     max_length=MAX_BLOB_WIDTH;
     maybe_null=1;
   }
-  used_tables_cache|=separator->used_tables();
-  const_item_cache&=separator->const_item();
-  with_sum_func= with_sum_func || separator->with_sum_func;
+  used_tables_cache|=     separator->used_tables();
+  not_null_tables_cache&= separator->not_null_tables();
+  const_item_cache&=	  separator->const_item();
+  with_sum_func=	  with_sum_func || separator->with_sum_func;
 }
 
 void Item_func_concat_ws::update_used_tables()
@@ -397,7 +672,7 @@ void Item_func_reverse::fix_length_and_dec()
 
 /*
 ** Replace all occurences of string2 in string1 with string3.
-** Don't reallocate val_str() if not neaded
+** Don't reallocate val_str() if not needed
 */
 
 /* TODO: Fix that this works with binary strings when using USE_MB */
@@ -456,7 +731,8 @@ redo:
           while (j != search_end)
             if (*i++ != *j++) goto skipp;
           offset= (int) (ptr-res->ptr());
-          if (res->length()-from_length + to_length > max_allowed_packet)
+          if (res->length()-from_length + to_length >
+	      current_thd->variables.max_allowed_packet)
             goto null;
           if (!alloced)
           {
@@ -476,7 +752,8 @@ skipp:
 #endif /* USE_MB */
     do
     {
-      if (res->length()-from_length + to_length > max_allowed_packet)
+      if (res->length()-from_length + to_length >
+	  current_thd->variables.max_allowed_packet)
         goto null;
       if (!alloced)
       {
@@ -501,8 +778,8 @@ void Item_func_replace::fix_length_and_dec()
   int diff=(int) (args[2]->max_length - args[1]->max_length);
   if (diff > 0 && args[1]->max_length)
   {						// Calculate of maxreplaces
-    max_length= max_length/args[1]->max_length;
-    max_length= (max_length+1)*(uint) diff;
+    uint max_substrs= max_length/args[1]->max_length;
+    max_length+= max_substrs * (uint)diff;
   }
   if (max_length > MAX_BLOB_WIDTH)
   {
@@ -533,10 +810,11 @@ String *Item_func_insert::val_str(String *str)
   }
 #endif
   if (start > res->length()+1)
-    return res;					// Wrong param; skipp insert
+    return res;					// Wrong param; skip insert
   if (length > res->length()-start)
     length=res->length()-start;
-  if (res->length() - length + res2->length() > max_allowed_packet)
+  if (res->length() - length + res2->length() >
+      current_thd->variables.max_allowed_packet)
     goto null;					// OOM check
   res=copy_if_not_alloced(str,res,res->length());
   res->replace(start,length,*res2);
@@ -619,7 +897,7 @@ void Item_str_func::left_right_max_length()
   max_length=args[0]->max_length;
   if (args[1]->const_item())
   {
-    int length=(int) args[1]->val_int();
+    int length=(int) args[1]->val_int()*default_charset_info->mbmaxlen;
     if (length <= 0)
       max_length=0;
     else
@@ -712,7 +990,7 @@ void Item_func_substr::fix_length_and_dec()
   }
   if (arg_count == 3 && args[2]->const_item())
   {
-    int32 length= (int32) args[2]->val_int();
+    int32 length= (int32) args[2]->val_int() * default_charset_info->mbmaxlen;
     if (length <= 0)
       max_length=0; /* purecov: inspected */
     else
@@ -1093,8 +1371,10 @@ String *Item_func_database::val_str(String *str)
 
 String *Item_func_user::val_str(String *str)
 {
+  // TODO: make USER() replicate properly (currently it is replicated to "")
   THD *thd=current_thd;
-  if (str->copy((const char*) thd->user,(uint) strlen(thd->user)) ||
+  if (!(thd->user) || // for system threads (e.g. replication SQL thread)
+      str->copy((const char*) thd->user,(uint) strlen(thd->user)) ||
       str->append('@') ||
       str->append(thd->host ? thd->host : thd->ip ? thd->ip : ""))
     return &empty_string;
@@ -1110,7 +1390,7 @@ void Item_func_soundex::fix_length_and_dec()
 
   /*
     If alpha, map input letter to soundex code.
-    If not alpha and remove_garbage is set then skipp to next char
+    If not alpha and remove_garbage is set then skip to next char
     else return 0
     */
 
@@ -1137,12 +1417,12 @@ String *Item_func_soundex::val_str(String *str)
   if ((null_value=args[0]->null_value))
     return 0; /* purecov: inspected */
 
-  if (str_value.alloc(max(res->length(),4)))
+  if (tmp_value.alloc(max(res->length(),4)))
     return str; /* purecov: inspected */
-  char *to= (char *) str_value.ptr();
+  char *to= (char *) tmp_value.ptr();
   char *from= (char *) res->ptr(), *end=from+res->length();
 
-  while (from != end && isspace(*from)) // Skipp pre-space
+  while (from != end && !isalpha(*from)) // Skip pre-space
     from++; /* purecov: inspected */
   if (from == end)
     return &empty_string;		// No alpha characters.
@@ -1163,11 +1443,11 @@ String *Item_func_soundex::val_str(String *str)
        last_ch = ch;			// save code of last input letter
     }					// for next double-letter check
   }
-  for (end=(char*) str_value.ptr()+4 ; to < end ; to++)
+  for (end=(char*) tmp_value.ptr()+4 ; to < end ; to++)
     *to = '0';
   *to=0;				// end string
-  str_value.length((uint) (to-str_value.ptr()));
-  return &str_value;
+  tmp_value.length((uint) (to-tmp_value.ptr()));
+  return &tmp_value;
 }
 
 
@@ -1191,18 +1471,24 @@ String *Item_func_format::val_str(String *str)
     return 0; /* purecov: inspected */
   dec= decimals ? decimals+1 : 0;
   str->set(nr,decimals);
+#ifdef HAVE_ISNAN
+  if (isnan(nr))
+    return str;
+#endif
   str_length=str->length();
   if (nr < 0)
     str_length--;				// Don't count sign
-  length=str->length()+(diff=(str_length- dec-1)/3);
-  if (diff)
+
+  /* We need this test to handle 'nan' values */
+  if (str_length >= dec+4)
   {
     char *tmp,*pos;
-    str=copy_if_not_alloced(&tmp_str,str,length);
+    length= str->length()+(diff=(str_length- dec-1)/3);
+    str= copy_if_not_alloced(&tmp_str,str,length);
     str->length(length);
-    tmp=(char*) str->ptr()+length - dec-1;
-    for (pos=(char*) str->ptr()+length ; pos != tmp; pos--)
-      pos[0]=pos[- (int) diff];
+    tmp= (char*) str->ptr()+length - dec-1;
+    for (pos= (char*) str->ptr()+length-1; pos != tmp; pos--)
+      pos[0]= pos[-(int) diff];
     while (diff)
     {
       pos[0]=pos[-(int) diff]; pos--;
@@ -1221,15 +1507,20 @@ void Item_func_elt::fix_length_and_dec()
 {
   max_length=0;
   decimals=0;
-  for (uint i=1 ; i < arg_count ; i++)
+#if MYSQL_VERSION_ID < 40100
+  for (uint i= 0; i < arg_count ; i++)
+#else
+  for (uint i= 1; i < arg_count ; i++)
+#endif
   {
     set_if_bigger(max_length,args[i]->max_length);
     set_if_bigger(decimals,args[i]->decimals);
   }
   maybe_null=1;					// NULL if wrong first arg
   with_sum_func= with_sum_func || item->with_sum_func;
-  used_tables_cache|=item->used_tables();
-  const_item_cache&=item->const_item();
+  used_tables_cache|=	  item->used_tables();
+  not_null_tables_cache&= item->not_null_tables();
+  const_item_cache&=	  item->const_item();
 }
 
 
@@ -1258,37 +1549,39 @@ void Item_func_elt::update_used_tables()
 double Item_func_elt::val()
 {
   uint tmp;
+  null_value=1;
   if ((tmp=(uint) item->val_int()) == 0 || tmp > arg_count)
-  {
-    null_value=1;
     return 0.0;
-  }
-  null_value=0;
-  return args[tmp-1]->val();
+
+  double result= args[tmp-1]->val();
+  null_value= args[tmp-1]->null_value;
+  return result;
 }
 
+
 longlong Item_func_elt::val_int()
 {
   uint tmp;
+  null_value=1;
   if ((tmp=(uint) item->val_int()) == 0 || tmp > arg_count)
-  {
-    null_value=1;
     return 0;
-  }
-  null_value=0;
-  return args[tmp-1]->val_int();
+  
+  longlong result= args[tmp-1]->val_int();
+  null_value= args[tmp-1]->null_value;
+  return result;
 }
 
+
 String *Item_func_elt::val_str(String *str)
 {
   uint tmp;
+  null_value=1;
   if ((tmp=(uint) item->val_int()) == 0 || tmp > arg_count)
-  {
-    null_value=1;
     return NULL;
-  }
-  null_value=0;
-  return args[tmp-1]->val_str(str);
+
+  String *result= args[tmp-1]->val_str(str);
+  null_value= args[tmp-1]->null_value;
+  return result;
 }
 
 
@@ -1310,8 +1603,9 @@ void Item_func_make_set::fix_length_and_dec()
   max_length=arg_count-1;
   for (uint i=1 ; i < arg_count ; i++)
     max_length+=args[i]->max_length;
-  used_tables_cache|=item->used_tables();
-  const_item_cache&=item->const_item();
+  used_tables_cache|=	  item->used_tables();
+  not_null_tables_cache&= item->not_null_tables();
+  const_item_cache&=	  item->const_item();
   with_sum_func= with_sum_func || item->with_sum_func;
 }
 
@@ -1344,7 +1638,7 @@ String *Item_func_make_set::val_str(String *str)
     if (bits & 1)
     {
       String *res= (*ptr)->val_str(str);
-      if (res)					// Skipp nulls
+      if (res)					// Skip nulls
       {
 	if (!first_found)
 	{					// First argument
@@ -1416,14 +1710,11 @@ inline String* alloc_buffer(String *res,String *str,String *tmp_value,
       str->length(length);
       return str;
     }
-    else
-    {
-      if (tmp_value->alloc(length))
-	return 0;
-      (void) tmp_value->copy(*res);
-      tmp_value->length(length);
-      return tmp_value;
-    }
+    if (tmp_value->alloc(length))
+      return 0;
+    (void) tmp_value->copy(*res);
+    tmp_value->length(length);
+    return tmp_value;
   }
   res->length(length);
   return res;
@@ -1468,7 +1759,8 @@ String *Item_func_repeat::val_str(String *str)
   if (count == 1)			// To avoid reallocs
     return res;
   length=res->length();
-  if (length > max_allowed_packet/count)// Safe length check
+  // Safe length check
+  if (length > current_thd->variables.max_allowed_packet/count)
     goto err;				// Probably an error
   tot_length= length*(uint) count;
   if (!(res= alloc_buffer(res,str,&tmp_value,tot_length)))
@@ -1515,9 +1807,9 @@ String *Item_func_rpad::val_str(String *str)
   const char *ptr_pad;
   int32 count= (int32) args[1]->val_int();
   String *res =args[0]->val_str(str);
-  String *rpad = args[2]->val_str(str);
+  String *rpad = args[2]->val_str(&rpad_str);
 
-  if (!res || args[1]->null_value || !rpad)
+  if (!res || args[1]->null_value || !rpad || count < 0)
     goto err;
   null_value=0;
   if (count <= (int32) (res_length=res->length()))
@@ -1526,7 +1818,8 @@ String *Item_func_rpad::val_str(String *str)
     return (res);
   }
   length_pad= rpad->length();
-  if ((ulong) count > max_allowed_packet || args[2]->null_value || !length_pad)
+  if ((ulong) count > current_thd->variables.max_allowed_packet ||
+      args[2]->null_value || !length_pad)
     goto err;
   if (!(res= alloc_buffer(res,str,&tmp_value,count)))
     goto err;
@@ -1574,7 +1867,7 @@ String *Item_func_lpad::val_str(String *str)
   const char *ptr_pad;
   ulong count= (long) args[1]->val_int();
   String *res= args[0]->val_str(str);
-  String *lpad= args[2]->val_str(str);
+  String *lpad= args[2]->val_str(&lpad_str);
 
   if (!res || args[1]->null_value || !lpad)
     goto err;
@@ -1585,7 +1878,8 @@ String *Item_func_lpad::val_str(String *str)
     return (res);
   }
   length_pad= lpad->length();
-  if (count > max_allowed_packet || args[2]->null_value || !length_pad)
+  if (count > current_thd->variables.max_allowed_packet ||
+      args[2]->null_value || !length_pad)
     goto err;
 
   if (res->alloced_length() < count)
@@ -1649,6 +1943,45 @@ String *Item_func_conv::val_str(String *str)
   return str;
 }
 
+
+String *Item_func_hex::val_str(String *str)
+{
+  if (args[0]->result_type() != STRING_RESULT)
+  {
+    /* Return hex of unsigned longlong value */
+    longlong dec= args[0]->val_int();
+    char ans[65],*ptr;
+    if ((null_value= args[0]->null_value))
+      return 0;
+    ptr= longlong2str(dec,ans,16);
+    if (str->copy(ans,(uint32) (ptr-ans)))
+      return &empty_string;			// End of memory
+    return str;
+  }
+
+  /* Convert given string to a hex string, character by character */
+  String *res= args[0]->val_str(str);
+  const char *from, *end;
+  char *to;
+  if (!res || tmp_value.alloc(res->length()*2))
+  {
+    null_value=1;
+    return 0;
+  }
+  null_value=0;
+  tmp_value.length(res->length()*2);
+  for (from=res->ptr(), end=from+res->length(), to= (char*) tmp_value.ptr();
+       from != end ;
+       from++, to+=2)
+  {
+    uint tmp=(uint) (uchar) *from;
+    to[0]=_dig_vec[tmp >> 4];
+    to[1]=_dig_vec[tmp & 15];
+  }
+  return &tmp_value;
+}
+
+
 #include <my_dir.h>				// For my_stat
 
 String *Item_load_file::val_str(String *str)
@@ -1667,7 +2000,7 @@ String *Item_load_file::val_str(String *str)
     /* my_error(ER_TEXTFILE_NOT_READABLE, MYF(0), file_name->c_ptr()); */
     goto err;
   }
-  if (stat_info.st_size > (long) max_allowed_packet)
+  if (stat_info.st_size > (long) current_thd->variables.max_allowed_packet)
   {
     /* my_error(ER_TOO_LONG_STRING, MYF(0), file_name->c_ptr()); */
     goto err;
@@ -1711,6 +2044,10 @@ String* Item_func_export_set::val_str(String* str)
     null_value=1;
     return 0;
   }
+  /*
+    Arg count can only be 3, 4 or 5 here. This is guaranteed from the
+    grammar for EXPORT_SET()
+  */
   switch(arg_count) {
   case 5:
     num_set_values = (uint) args[4]->val_int();
@@ -1741,7 +2078,7 @@ String* Item_func_export_set::val_str(String* str)
       str->append(*yes);
     else
       str->append(*no);
-    if(i != num_set_values - 1)
+    if (i != num_set_values - 1)
       str->append(*sep);
   }
   return str;
@@ -1759,24 +2096,23 @@ String* Item_func_inet_ntoa::val_str(String* str)
   uchar buf[8], *p;
   ulonglong n = (ulonglong) args[0]->val_int();
   char num[4];
+
   /*
-    we do not know if args[0] is NULL until we have called
+    We do not know if args[0] is NULL until we have called
     some val function on it if args[0] is not a constant!
+
+    Also return null if n > 255.255.255.255
   */
-  if ((null_value=args[0]->null_value))
+  if ((null_value= (args[0]->null_value || n > (ulonglong) LL(4294967295))))
     return 0;					// Null value
 
   str->length(0);
-  int8store(buf,n);
+  int4store(buf,n);
 
-  /*
-    Now we can assume little endian. 
-    We handle the possibility of an 8-byte IP address however, we do
-    not want to confuse those who are just using 4 byte ones
-  */
-  for (p= buf + 8; p > buf+4 && p[-1] == 0 ; p-- ) ;
+  /* Now we can assume little endian. */
+  
   num[3]='.';
-  while (p-- > buf)
+  for (p=buf+4 ; p-- > buf ; )
   {
     uint c = *p;
     uint n1,n2;					// Try to avoid divisions
@@ -1794,3 +2130,94 @@ String* Item_func_inet_ntoa::val_str(String* str)
   str->length(str->length()-1);			// Remove last '.';
   return str;
 }
+
+/*
+  QUOTE() function returns argument string in single quotes suitable for
+  using in a SQL statement.
+
+  DESCRIPTION
+    Adds a \ before all characters that needs to be escaped in a SQL string.
+    We also escape '^Z' (END-OF-FILE in windows) to avoid probelms when
+    running commands from a file in windows. 
+
+    This function is very useful when you want to generate SQL statements
+
+    RETURN VALUES
+    str		Quoted string
+    NULL	Argument to QUOTE() was NULL or out of memory.
+*/
+
+#define get_esc_bit(mask, num) (1 & (*((mask) + ((num) >> 3))) >> ((num) & 7))
+
+String *Item_func_quote::val_str(String *str)
+{
+  /*
+    Bit mask that has 1 for set for the position of the following characters:
+    0, \, ' and ^Z
+  */ 
+
+  static uchar escmask[32]=
+  {
+    0x01, 0x00, 0x00, 0x04, 0x80, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+  };
+
+  char *from, *to, *end, *start;
+  String *arg= args[0]->val_str(str);
+  uint arg_length, new_length;
+  if (!arg)					// Null argument
+    goto null;
+  arg_length= arg->length();
+  new_length= arg_length+2; /* for beginning and ending ' signs */
+
+  for (from= (char*) arg->ptr(), end= from + arg_length; from < end; from++)
+    new_length+= get_esc_bit(escmask, (uchar) *from);
+
+  /*
+    We have to use realloc() instead of alloc() as we want to keep the
+    old result in str
+  */
+  if (str->realloc(new_length))
+    goto null;
+
+  /*
+    As 'arg' and 'str' may be the same string, we must replace characters
+    from the end to the beginning
+  */
+  to= (char*) str->ptr() + new_length - 1;
+  *to--= '\'';
+  for (start= (char*) arg->ptr(),end= start + arg_length; end-- != start; to--)
+  {
+    /*
+      We can't use the bitmask here as we want to replace \O and ^Z with 0
+      and Z
+    */
+    switch (*end)  {
+    case 0:
+      *to--= '0';
+      *to=   '\\';
+      break;
+    case '\032':
+      *to--= 'Z';
+      *to=   '\\';
+      break;
+    case '\'':
+    case '\\':
+      *to--= *end;
+      *to=   '\\';
+      break;
+    default:
+      *to= *end;
+      break;
+    }
+  }
+  *to= '\'';
+  str->length(new_length);
+  return str;
+
+null:
+  null_value= 1;
+  return 0;
+}
