@@ -70,7 +70,7 @@ public:
   {}
   inline bool is_same(SEL_ARG *arg)
   {
-    if (type != arg->type)
+    if (type != arg->type || part != arg->part)
       return 0;
     if (type != KEY_RANGE)
       return 1;
@@ -280,6 +280,7 @@ public:
 
 
 typedef struct st_qsel_param {
+  THD	*thd;
   TABLE *table;
   KEY_PART *key_parts,*key_parts_end,*key[MAX_KEY];
   MEM_ROOT *mem_root;
@@ -289,6 +290,7 @@ typedef struct st_qsel_param {
   char min_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH],
     max_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
   bool quick;				// Don't calulate possible keys
+  COND *cond;
 } PARAM;
 
 static SEL_TREE * get_mm_parts(PARAM *param,Field *field,
@@ -382,13 +384,14 @@ SQL_SELECT::~SQL_SELECT()
 
 #undef index					// Fix for Unixware 7
 
-QUICK_SELECT::QUICK_SELECT(TABLE *table,uint key_nr,bool no_alloc)
+QUICK_SELECT::QUICK_SELECT(THD *thd, TABLE *table, uint key_nr, bool no_alloc)
   :dont_free(0),error(0),index(key_nr),max_used_key_length(0),head(table),
    it(ranges),range(0)
 {
   if (!no_alloc)
   {
-    init_sql_alloc(&alloc,1024,0);		// Allocates everything here
+    // Allocates everything through the internal memroot
+    init_sql_alloc(&alloc, thd->variables.range_alloc_block_size, 0);
     my_pthread_setspecific_ptr(THR_MALLOC,&alloc);
   }
   else
@@ -460,17 +463,17 @@ SEL_ARG *SEL_ARG::clone(SEL_ARG *new_parent,SEL_ARG **next_arg)
   SEL_ARG *tmp;
   if (type != KEY_RANGE)
   {
-    if(!(tmp=new SEL_ARG(type)))
-      return 0;			// out of memory
+    if (!(tmp= new SEL_ARG(type)))
+      return 0;					// out of memory
     tmp->prev= *next_arg;			// Link into next/prev chain
     (*next_arg)->next=tmp;
     (*next_arg)= tmp;
   }
   else
   {
-    if(!(tmp=new SEL_ARG(field,part, min_value,max_value,
-			 min_flag, max_flag, maybe_flag)))
-      return 0;			// out of memory
+    if (!(tmp= new SEL_ARG(field,part, min_value,max_value,
+			   min_flag, max_flag, maybe_flag)))
+      return 0;					// OOM
     tmp->parent=new_parent;
     tmp->next_key_part=next_key_part;
     if (left != &null_element)
@@ -481,9 +484,11 @@ SEL_ARG *SEL_ARG::clone(SEL_ARG *new_parent,SEL_ARG **next_arg)
     (*next_arg)= tmp;
 
     if (right != &null_element)
-      tmp->right=right->clone(tmp,next_arg);
+      if (!(tmp->right= right->clone(tmp,next_arg)))
+	return 0;				// OOM
   }
   increment_use_count(1);
+  tmp->color= color;
   return tmp;
 }
 
@@ -560,10 +565,11 @@ SEL_ARG *SEL_ARG::clone_tree()
 {
   SEL_ARG tmp_link,*next_arg,*root;
   next_arg= &tmp_link;
-  root=clone((SEL_ARG *) 0, &next_arg);
+  root= clone((SEL_ARG *) 0, &next_arg);
   next_arg->next=0;				// Fix last link
   tmp_link.next->prev=0;			// Fix first link
-  root->use_count=0;
+  if (root)					// If not OOM
+    root->use_count= 0;
   return root;
 }
 
@@ -581,7 +587,8 @@ SEL_ARG *SEL_ARG::clone_tree()
 **	quick_rows ; How many rows the key matches
 *****************************************************************************/
 
-int SQL_SELECT::test_quick_select(key_map keys_to_use, table_map prev_tables,
+int SQL_SELECT::test_quick_select(THD *thd, key_map keys_to_use,
+				  table_map prev_tables,
 				  ha_rows limit, bool force_quick_range)
 {
   uint basflag;
@@ -624,6 +631,7 @@ int SQL_SELECT::test_quick_select(key_map keys_to_use, table_map prev_tables,
     PARAM param;
 
     /* set up parameter that is passed to all functions */
+    param.thd= thd;
     param.baseflag=basflag;
     param.prev_tables=prev_tables | const_tables;
     param.read_tables=read_tables;
@@ -631,14 +639,13 @@ int SQL_SELECT::test_quick_select(key_map keys_to_use, table_map prev_tables,
     param.table=head;
     param.keys=0;
     param.mem_root= &alloc;
-
-    current_thd->no_errors=1;			// Don't warn about NULL
-    init_sql_alloc(&alloc,2048,0);
+    param.thd->no_errors=1;			// Don't warn about NULL
+    init_sql_alloc(&alloc, param.thd->variables.range_alloc_block_size, 0);
     if (!(param.key_parts = (KEY_PART*) alloc_root(&alloc,
 						   sizeof(KEY_PART)*
 						   head->key_parts)))
     {
-      current_thd->no_errors=0;
+      param.thd->no_errors=0;
       free_root(&alloc,MYF(0));			// Return memory & allocator
       DBUG_RETURN(0);				// Can't use range
     }
@@ -737,7 +744,7 @@ int SQL_SELECT::test_quick_select(key_map keys_to_use, table_map prev_tables,
     }
     free_root(&alloc,MYF(0));			// Return memory & allocator
     my_pthread_setspecific_ptr(THR_MALLOC,old_root);
-    current_thd->no_errors=0;
+    param.thd->no_errors=0;
   }
   DBUG_EXECUTE("info",print_quick(quick,needed_reg););
   /*
@@ -765,7 +772,7 @@ static SEL_TREE *get_mm_tree(PARAM *param,COND *cond)
       while ((item=li++))
       {
 	SEL_TREE *new_tree=get_mm_tree(param,item);
-	if(current_thd->fatal_error)
+	if (param->thd->fatal_error)
 	  DBUG_RETURN(0);	// out of memory
 	tree=tree_and(param,tree,new_tree);
 	if (tree && tree->type == SEL_TREE::IMPOSSIBLE)
@@ -812,6 +819,8 @@ static SEL_TREE *get_mm_tree(PARAM *param,COND *cond)
   if (cond_func->select_optimize() == Item_func::OPTIMIZE_NONE)
     DBUG_RETURN(0);				// Can't be calculated
 
+  param->cond= cond;
+
   if (cond_func->functype() == Item_func::BETWEEN)
   {
     if (cond_func->arguments()[0]->type() == Item::FIELD_ITEM)
@@ -906,7 +915,7 @@ get_mm_parts(PARAM *param, Field *field, Item_func::Functype type,
     {
       SEL_ARG *sel_arg=0;
       if (!tree && !(tree=new SEL_TREE()))
-	DBUG_RETURN(0);		// out of memory
+	DBUG_RETURN(0);				// OOM
       if (!value || !(value->used_tables() & ~param->read_tables))
       {
 	sel_arg=get_mm_leaf(param,key_part->field,key_part,type,value);
@@ -918,10 +927,11 @@ get_mm_parts(PARAM *param, Field *field, Item_func::Functype type,
 	  DBUG_RETURN(tree);
 	}
       }
-      else {
+      else
+      {
 	// This key may be used later
-	if(!(sel_arg=new SEL_ARG(SEL_ARG::MAYBE_KEY))) 
-	  DBUG_RETURN(0);	// out of memory
+	if (!(sel_arg= new SEL_ARG(SEL_ARG::MAYBE_KEY))) 
+	  DBUG_RETURN(0);			// OOM
       }
       sel_arg->part=(uchar) key_part->part;
       tree->keys[key_part->key]=sel_add(tree->keys[key_part->key],sel_arg);
@@ -935,9 +945,10 @@ static SEL_ARG *
 get_mm_leaf(PARAM *param, Field *field, KEY_PART *key_part,
 	    Item_func::Functype type,Item *value)
 {
-  uint maybe_null=(uint) field->real_maybe_null();
+  uint maybe_null=(uint) field->real_maybe_null(), copies;
   uint field_length=field->pack_length()+maybe_null;
   SEL_ARG *tree;
+  char *str, *str2;
   DBUG_ENTER("get_mm_leaf");
 
   if (type == Item_func::LIKE_FUNC)
@@ -986,23 +997,26 @@ get_mm_leaf(PARAM *param, Field *field, KEY_PART *key_part,
     if (maybe_null)
       max_str[0]= min_str[0]=0;
     if (field->binary())
-      like_error=like_range(res->ptr(),res->length(),wild_prefix,field_length,
-			    min_str+offset,max_str+offset,(char) 255,
-			    &min_length,&max_length);
+      like_error=like_range(res->ptr(), res->length(),
+			    ((Item_func_like*)(param->cond))->escape,
+			    field_length, min_str + offset, max_str + offset,
+			    (char) 255, &min_length, &max_length);
     else
     {
 #ifdef USE_STRCOLL
       if (use_strcoll(default_charset_info))
-        like_error= my_like_range(default_charset_info,
-                                  res->ptr(),res->length(),wild_prefix,
-                                  field_length, min_str+maybe_null,
-                                  max_str+maybe_null,&min_length,&max_length);
+        like_error= my_like_range(default_charset_info, res->ptr(),
+				  res->length(),
+				  ((Item_func_like*)(param->cond))->escape,
+                                  field_length, min_str + maybe_null,
+                                  max_str + maybe_null, &min_length,
+				  &max_length);
       else
 #endif
-        like_error=like_range(res->ptr(),res->length(),wild_prefix,
-			      field_length,
-                              min_str+offset,max_str+offset,
-                              max_sort_char,&min_length,&max_length);
+        like_error=like_range(res->ptr(), res->length(),
+			      ((Item_func_like*)(param->cond))->escape,
+			      field_length, min_str + offset, max_str + offset,
+                              max_sort_char, &min_length, &max_length);
     }
     if (like_error)				// Can't optimize with LIKE
       DBUG_RETURN(0);
@@ -1049,15 +1063,37 @@ get_mm_leaf(PARAM *param, Field *field, KEY_PART *key_part,
     /* This happens when we try to insert a NULL field in a not null column */
     DBUG_RETURN(&null_element);			// cmp with NULL is never true
   }
-  // Get local copy of key
-  char *str= (char*) alloc_root(param->mem_root,
-				key_part->part_length+maybe_null);
+  /* Get local copy of key */
+  copies= 1;
+  if (field->key_type() == HA_KEYTYPE_VARTEXT)
+    copies= 2;
+  str= str2= (char*) alloc_root(param->mem_root,
+				(key_part->part_length+maybe_null)*copies+1);
   if (!str)
     DBUG_RETURN(0);
   if (maybe_null)
     *str= (char) field->is_real_null();		// Set to 1 if null
   field->get_key_image(str+maybe_null,key_part->part_length);
-  if (!(tree=new SEL_ARG(field,str,str)))
+  if (copies == 2)
+  {
+    /*
+      The key is stored as 2 byte length + key
+      key doesn't match end space. In other words, a key 'X ' should match
+      all rows between 'X' and 'X           ...'
+    */
+    uint length= uint2korr(str+maybe_null);
+    str2= str+ key_part->part_length + maybe_null;
+    /* remove end space */
+    while (length > 0 && str[length+HA_KEY_BLOB_LENGTH+maybe_null-1] == ' ')
+      length--;
+    int2store(str+maybe_null, length);
+    /* Create key that is space filled */
+    memcpy(str2, str, length + HA_KEY_BLOB_LENGTH + maybe_null);
+    bfill(str2+ length+ HA_KEY_BLOB_LENGTH +maybe_null,
+	  key_part->part_length-length - HA_KEY_BLOB_LENGTH, ' ');
+    int2store(str2+maybe_null, key_part->part_length - HA_KEY_BLOB_LENGTH);
+  }
+  if (!(tree=new SEL_ARG(field,str,str2)))
     DBUG_RETURN(0);		// out of memory
 
   switch (type) {
@@ -1164,8 +1200,8 @@ static bool like_range(const char *ptr,uint ptr_length,char escape,
 ******************************************************************************/
 
 /*
-** Add a new key test to a key when scanning through all keys
-** This will never be called for same key parts.
+  Add a new key test to a key when scanning through all keys
+  This will never be called for same key parts.
 */
 
 static SEL_ARG *
@@ -1349,7 +1385,8 @@ key_and(SEL_ARG *key1,SEL_ARG *key2,uint clone_flag)
     // key1->part < key2->part
     key1->use_count--;
     if (key1->use_count > 0)
-      key1=key1->clone_tree();
+      if (!(key1= key1->clone_tree()))
+	return 0;				// OOM
     return and_all_keys(key1,key2,clone_flag);
   }
 
@@ -1368,7 +1405,8 @@ key_and(SEL_ARG *key1,SEL_ARG *key2,uint clone_flag)
     if (key1->use_count > 1)
     {
       key1->use_count--;
-      key1=key1->clone_tree();
+      if (!(key1=key1->clone_tree()))
+	return 0;				// OOM
       key1->use_count++;
     }
     if (key1->type == SEL_ARG::MAYBE_KEY)
@@ -1412,6 +1450,8 @@ key_and(SEL_ARG *key1,SEL_ARG *key2,uint clone_flag)
     if (!next || next->type != SEL_ARG::IMPOSSIBLE)
     {
       SEL_ARG *new_arg= e1->clone_and(e2);
+      if (!new_arg)
+	return &null_element;			// End of memory
       new_arg->next_key_part=next;
       if (!new_tree)
       {
@@ -1463,7 +1503,7 @@ key_or(SEL_ARG *key1,SEL_ARG *key2)
     }
     return 0;
   }
-  else if (!key2)
+  if (!key2)
   {
     key1->use_count--;
     key1->free_tree();
@@ -1499,8 +1539,8 @@ key_or(SEL_ARG *key1,SEL_ARG *key2)
     {
       swap(SEL_ARG *,key1,key2);
     }
-    else
-      key1=key1->clone_tree();
+    else if (!(key1=key1->clone_tree()))
+      return 0;					// OOM
   }
 
   // Add tree at key2 to tree at key1
@@ -1526,7 +1566,7 @@ key_or(SEL_ARG *key1,SEL_ARG *key2)
 	SEL_ARG *key2_next=key2->next;
 	if (key2_shared)
 	{
-	  if(!(key2=new SEL_ARG(*key2)))
+	  if (!(key2=new SEL_ARG(*key2)))
 	    return 0;		// out of memory
 	  key2->increment_use_count(key1->use_count+1);
 	  key2->next=key2_next;			// New copy of key2
@@ -1568,7 +1608,10 @@ key_or(SEL_ARG *key1,SEL_ARG *key2)
 	  SEL_ARG *next=key2->next;		// Keys are not overlapping
 	  if (key2_shared)
 	  {
-	    key1=key1->insert(new SEL_ARG(*key2)); // Must make copy
+	    SEL_ARG *tmp= new SEL_ARG(*key2);	// Must make copy
+	    if (!tmp)
+	      return 0;				// OOM
+	    key1=key1->insert(tmp);
 	    key2->increment_use_count(key1->use_count+1);
 	  }
 	  else
@@ -1614,6 +1657,8 @@ key_or(SEL_ARG *key1,SEL_ARG *key2)
     if (cmp >= 0 && tmp->cmp_min_to_min(key2) < 0)
     {						// tmp.min <= x < key2.min
       SEL_ARG *new_arg=tmp->clone_first(key2);
+      if (!new_arg)
+	return 0;				// OOM
       if ((new_arg->next_key_part= key1->next_key_part))
 	new_arg->increment_use_count(key1->use_count+1);
       tmp->copy_min_to_min(key2);
@@ -1627,6 +1672,8 @@ key_or(SEL_ARG *key1,SEL_ARG *key2)
       if (tmp->cmp_min_to_min(&key) > 0)
       {						// key.min <= x < tmp.min
 	SEL_ARG *new_arg=key.clone_first(tmp);
+	if (!new_arg)
+	  return 0;				// OOM
 	if ((new_arg->next_key_part=key.next_key_part))
 	  new_arg->increment_use_count(key1->use_count+1);
 	key1=key1->insert(new_arg);
@@ -1641,21 +1688,31 @@ key_or(SEL_ARG *key1,SEL_ARG *key2)
 	key.copy_max_to_min(tmp);
 	if (!(tmp=tmp->next))
 	{
-	  key1=key1->insert(new SEL_ARG(key));
+	  SEL_ARG *tmp2= new SEL_ARG(key);
+	  if (!tmp2)
+	    return 0;				// OOM
+	  key1=key1->insert(tmp2);
 	  key2=key2->next;
 	  goto end;
 	}
 	if (tmp->cmp_min_to_max(&key) > 0)
 	{
-	  key1=key1->insert(new SEL_ARG(key));
+	  SEL_ARG *tmp2= new SEL_ARG(key);
+	  if (!tmp2)
+	    return 0;				// OOM
+	  key1=key1->insert(tmp2);
 	  break;
 	}
       }
       else
       {
 	SEL_ARG *new_arg=tmp->clone_last(&key); // tmp.min <= x <= key.max
+	if (!new_arg)
+	  return 0;				// OOM
 	tmp->copy_max_to_min(&key);
 	tmp->increment_use_count(key1->use_count+1);
+	/* Increment key count as it may be used for next loop */
+	key.increment_use_count(1);
 	new_arg->next_key_part=key_or(tmp->next_key_part,key.next_key_part);
 	key1=key1->insert(new_arg);
 	break;
@@ -1670,8 +1727,11 @@ end:
     SEL_ARG *next=key2->next;
     if (key2_shared)
     {
+      SEL_ARG *tmp=new SEL_ARG(*key2);		// Must make copy
+      if (!tmp)
+	return 0;
       key2->increment_use_count(key1->use_count+1);
-      key1=key1->insert(new SEL_ARG(*key2));	// Must make copy
+      key1=key1->insert(tmp);
     }
     else
       key1=key1->insert(key2);			// Will destroy key2_root
@@ -2084,6 +2144,7 @@ static ulong count_key_part_usage(SEL_ARG *root, SEL_ARG *key)
 
 void SEL_ARG::test_use_count(SEL_ARG *root)
 {
+  uint e_count=0;
   if (this == root && use_count != 1)
   {
     sql_print_error("Note: Use_count: Wrong count %lu for root",use_count);
@@ -2091,7 +2152,6 @@ void SEL_ARG::test_use_count(SEL_ARG *root)
   }
   if (this->type != SEL_ARG::KEY_RANGE)
     return;
-  uint e_count=0;
   for (SEL_ARG *pos=first(); pos ; pos=pos->next)
   {
     e_count++;
@@ -2108,8 +2168,8 @@ void SEL_ARG::test_use_count(SEL_ARG *root)
     }
   }
   if (e_count != elements)
-    sql_print_error("Warning: Wrong use count: %u for tree at %lx", e_count,
-		    (gptr) this);
+    sql_print_error("Warning: Wrong use count: %u (should be %u) for tree at %lx",
+		    e_count, elements, (gptr) this);
 }
 
 #endif
@@ -2204,7 +2264,8 @@ check_quick_keys(PARAM *param,uint idx,SEL_ARG *key_tree,
   param->range_count++;
   if (!tmp_min_flag && ! tmp_max_flag &&
       (uint) key_tree->part+1 == param->table->key_info[keynr].key_parts &&
-      (param->table->key_info[keynr].flags & HA_NOSAME) &&
+      (param->table->key_info[keynr].flags & (HA_NOSAME | HA_END_SPACE_KEY)) ==
+      HA_NOSAME &&
       min_key_length == max_key_length &&
       !memcmp(param->min_key,param->max_key,min_key_length))
     tmp=1;					// Max one record
@@ -2247,7 +2308,8 @@ get_quick_select(PARAM *param,uint idx,SEL_ARG *key_tree)
 {
   QUICK_SELECT *quick;
   DBUG_ENTER("get_quick_select");
-  if ((quick=new QUICK_SELECT(param->table,param->real_keynr[idx])))
+  if ((quick=new QUICK_SELECT(param->thd, param->table,
+			      param->real_keynr[idx])))
   {
     if (quick->error ||
 	get_quick_keys(param,quick,param->key[idx],key_tree,param->min_key,0,
@@ -2337,7 +2399,8 @@ get_quick_keys(PARAM *param,QUICK_SELECT *quick,KEY_PART *key,
     {
       KEY *table_key=quick->head->key_info+quick->index;
       flag=EQ_RANGE;
-      if (table_key->flags & HA_NOSAME && key->part == table_key->key_parts-1)
+      if ((table_key->flags & (HA_NOSAME | HA_END_SPACE_KEY)) == HA_NOSAME &&
+	  key->part == table_key->key_parts-1)
       {
 	if (!(table_key->flags & HA_NULL_PART_KEY) ||
 	    !null_part_in_key(key,
@@ -2351,11 +2414,11 @@ get_quick_keys(PARAM *param,QUICK_SELECT *quick,KEY_PART *key,
   }
 
   /* Get range for retrieving rows in QUICK_SELECT::get_next */
-  if(!(range= new QUICK_RANGE(param->min_key,
-			      (uint) (tmp_min_key - param->min_key),
-			      param->max_key,
-			      (uint) (tmp_max_key - param->max_key),
-			      flag)))
+  if (!(range= new QUICK_RANGE((const char *) param->min_key,
+			       (uint) (tmp_min_key - param->min_key),
+			       (const char *) param->max_key,
+			       (uint) (tmp_max_key - param->max_key),
+			       flag)))
     return 1;			// out of memory
 
   set_if_bigger(quick->max_used_key_length,range->min_length);
@@ -2382,7 +2445,7 @@ bool QUICK_SELECT::unique_key_range()
     if (((tmp=ranges.head())->flag & (EQ_RANGE | NULL_RANGE)) == EQ_RANGE)
     {
       KEY *key=head->key_info+index;
-      return ((key->flags & HA_NOSAME) &&
+      return ((key->flags & (HA_NOSAME | HA_END_SPACE_KEY)) == HA_NOSAME &&
 	      key->key_length == tmp->min_length);
     }
   }
@@ -2411,31 +2474,32 @@ static bool null_part_in_key(KEY_PART *key_part, const char *key, uint length)
 ** Create a QUICK RANGE based on a key
 ****************************************************************************/
 
-QUICK_SELECT *get_quick_select_for_ref(TABLE *table, TABLE_REF *ref)
+QUICK_SELECT *get_quick_select_for_ref(THD *thd, TABLE *table, TABLE_REF *ref)
 {
   table->file->index_end();			// Remove old cursor
-  QUICK_SELECT *quick=new QUICK_SELECT(table, ref->key, 1);
+  QUICK_SELECT *quick=new QUICK_SELECT(thd, table, ref->key, 1);
   KEY *key_info = &table->key_info[ref->key];
   KEY_PART *key_part;
+  QUICK_RANGE *range;
   uint part;
 
   if (!quick)
     return 0;			/* no ranges found */
   if (cp_buffer_from_ref(ref))
   {
-    if (current_thd->fatal_error)
-      return 0;					// out of memory
+    if (thd->fatal_error)
+      goto err;					// out of memory
     return quick;				// empty range
   }
 
-  QUICK_RANGE *range= new QUICK_RANGE();
-  if (!range)
+  if (!(range= new QUICK_RANGE()))
     goto err;			// out of memory
 
   range->min_key=range->max_key=(char*) ref->key_buff;
   range->min_length=range->max_length=ref->key_length;
   range->flag= ((ref->key_length == key_info->key_length &&
-		 (key_info->flags & HA_NOSAME)) ? EQ_RANGE : 0);
+		 (key_info->flags & (HA_NOSAME | HA_END_SPACE_KEY)) ==
+		 HA_NOSAME) ? EQ_RANGE : 0);
 
   if (!(quick->key_parts=key_part=(KEY_PART *)
 	alloc_root(&quick->alloc,sizeof(KEY_PART)*ref->key_parts)))
@@ -2647,15 +2711,18 @@ int QUICK_SELECT_DESC::get_next()
     }
     else
     {
-      /* Heikki changed Sept 11, 2002: since InnoDB does not store the cursor
-	 position if READ_KEY_EXACT is used to a primary key with all
-	 key columns specified, we must use below HA_READ_KEY_OR_NEXT,
-	 so that InnoDB stores the cursor position and is able to move
-	 the cursor one step backward after the search. */
+      /*
+	Heikki changed Sept 11, 2002: since InnoDB does not store the cursor
+	position if READ_KEY_EXACT is used to a primary key with all
+	key columns specified, we must use below HA_READ_KEY_OR_NEXT,
+	so that InnoDB stores the cursor position and is able to move
+	the cursor one step backward after the search. */
 
       DBUG_ASSERT(range->flag & NEAR_MAX || range_reads_after_key(range));
-      /* Note: even if max_key is only a prefix, HA_READ_AFTER_KEY will
-       * do the right thing - go past all keys which match the prefix */
+      /*
+	Note: even if max_key is only a prefix, HA_READ_AFTER_KEY will
+	do the right thing - go past all keys which match the prefix
+      */
       result=file->index_read(record, (byte*) range->max_key,
 			      range->max_length,
 			      ((range->flag & NEAR_MAX) ?
