@@ -52,6 +52,13 @@ static Slave_log_event* find_slave_event(IO_CACHE* log,
 					 const char* log_file_name,
 					 char* errmsg);
 
+/*
+  All of the functions defined in this file which are not used (the ones to
+  handle failsafe) are not used; their code has not been updated for more than
+  one year now so should be considered as BADLY BROKEN. Do not enable it.
+  The used functions (to handle LOAD DATA FROM MASTER, plus some small
+  functions like register_slave()) are working.
+*/
 
 static int init_failsafe_rpl_thread(THD* thd)
 {
@@ -419,7 +426,6 @@ static Slave_log_event* find_slave_event(IO_CACHE* log,
     my_snprintf(errmsg, SLAVE_ERRMSG_SIZE,
 		"Could not find slave event in log '%s'",
 		(char*)log_file_name);
-    delete ev;
     return 0;
   }
 
@@ -717,7 +723,8 @@ static int fetch_db_tables(THD *thd, MYSQL *mysql, const char *db,
       if (!tables_ok(thd, &table))
 	continue;
     }
-    if ((error= fetch_master_table(thd, db, table_name, mi, mysql)))
+    /* download master's table and overwrite slave's table */
+    if ((error= fetch_master_table(thd, db, table_name, mi, mysql, 1)))
       return error;
   }
   return 0;
@@ -743,7 +750,7 @@ int load_master_data(THD* thd)
     We do not want anyone messing with the slave at all for the entire
     duration of the data load.
   */
-  LOCK_ACTIVE_MI;
+  pthread_mutex_lock(&LOCK_active_mi);
   lock_slave_threads(active_mi);
   init_thread_mask(&restart_thread_mask,active_mi,0 /*not inverse*/);
   if (restart_thread_mask &&
@@ -752,7 +759,7 @@ int load_master_data(THD* thd)
   {
     send_error(&thd->net,error);
     unlock_slave_threads(active_mi);
-    UNLOCK_ACTIVE_MI;
+    pthread_mutex_unlock(&LOCK_active_mi);
     return 1;
   }
   
@@ -819,8 +826,11 @@ int load_master_data(THD* thd)
       char* db = row[0];
 
       /*
-	Do not replicate databases excluded by rules
-	also skip mysql database - in most cases the user will
+	Do not replicate databases excluded by rules. We also test
+	replicate_wild_*_table rules (replicate_wild_ignore_table='db1.%' will
+	be considered as "ignore the 'db1' database as a whole, as it already
+	works for CREATE DATABASE and DROP DATABASE).
+	Also skip 'mysql' database - in most cases the user will
 	mess up and not exclude mysql database with the rules when
 	he actually means to - in this case, he is up for a surprise if
 	his priv tables get dropped and downloaded from master
@@ -830,14 +840,14 @@ int load_master_data(THD* thd)
       */
 
       if (!db_ok(db, replicate_do_db, replicate_ignore_db) ||
+          !db_ok_with_wild_table(db) ||
 	  !strcmp(db,"mysql"))
       {
 	*cur_table_res = 0;
 	continue;
       }
 
-      if (mysql_rm_db(thd, db, 1,1) ||
-	  mysql_create_db(thd, db, 0, 1))
+      if (mysql_create_db(thd, db, HA_LEX_CREATE_IF_NOT_EXISTS, 1))
       {
 	send_error(&thd->net, 0, 0);
 	cleanup_mysql_results(db_res, cur_table_res - 1, table_res);
@@ -864,7 +874,7 @@ int load_master_data(THD* thd)
 
     cleanup_mysql_results(db_res, cur_table_res - 1, table_res);
 
-    // adjust position in the master
+    // adjust replication coordinates from the master
     if (master_status_res)
     {
       MYSQL_ROW row = mc_mysql_fetch_row(master_status_res);
@@ -877,14 +887,22 @@ int load_master_data(THD* thd)
       */
       if (row && row[0] && row[1])
       {
+        /*
+          If the slave's master info is not inited, we init it, then we write
+          the new coordinates to it. Must call init_master_info() *before*
+          setting active_mi, because init_master_info() sets active_mi with
+          defaults.
+        */
+        if (init_master_info(active_mi, master_info_file, relay_log_info_file, 0))
+          send_error(&thd->net, ER_MASTER_INFO);
 	strmake(active_mi->master_log_name, row[0],
 		sizeof(active_mi->master_log_name));
 	active_mi->master_log_pos = strtoull(row[1], (char**) 0, 10);
-	// don't hit the magic number
+        // at least in recent versions, the condition below should be false
 	if (active_mi->master_log_pos < BIN_LOG_HEADER_SIZE)
 	  active_mi->master_log_pos = BIN_LOG_HEADER_SIZE;
 	active_mi->rli.pending = 0;
-	flush_master_info(active_mi);
+	flush_master_info(active_mi, 0);
       }
       mc_mysql_free_result(master_status_res);
     }
@@ -903,7 +921,7 @@ int load_master_data(THD* thd)
   {
     send_error(&thd->net, 0, "Failed purging old relay logs");
     unlock_slave_threads(active_mi);
-    UNLOCK_ACTIVE_MI;
+    pthread_mutex_unlock(&LOCK_active_mi);
     return 1;
   }
   pthread_mutex_lock(&active_mi->rli.data_lock);
@@ -924,7 +942,7 @@ int load_master_data(THD* thd)
 
 err:
   unlock_slave_threads(active_mi);
-  UNLOCK_ACTIVE_MI;
+  pthread_mutex_unlock(&LOCK_active_mi);
   thd->proc_info = 0;
 
   mc_mysql_close(&mysql); // safe to call since we always do mc_mysql_init()
