@@ -1,15 +1,15 @@
 /* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
@@ -34,8 +34,9 @@
 #ifdef	__WIN__
 #include <io.h>
 #endif
+#include <mysys_err.h>
+#include <assert.h>
 
-extern struct rand_struct sql_rand;
 
 /*****************************************************************************
 ** Instansiate templates
@@ -57,14 +58,14 @@ template class List_iterator<Alter_column>;
 ** User variables
 ****************************************************************************/
 
-static byte* get_var_key(user_var_entry *entry, uint *length,
-			 my_bool not_used __attribute__((unused)))
+extern "C" byte *get_var_key(user_var_entry *entry, uint *length,
+			     my_bool not_used __attribute__((unused)))
 {
   *length=(uint) entry->name.length;
   return (byte*) entry->name.str;
 }
 
-static void free_var(user_var_entry *entry)
+extern "C" void free_user_var(user_var_entry *entry)
 {
   char *pos= (char*) entry+ALIGN_SIZE(sizeof(*entry));
   if (entry->value && entry->value != pos)
@@ -78,66 +79,65 @@ static void free_var(user_var_entry *entry)
 ****************************************************************************/
 
 THD::THD():user_time(0),fatal_error(0),last_insert_id_used(0),
-	   insert_id_used(0),in_lock_tables(0),
+	   insert_id_used(0),rand_used(0),in_lock_tables(0),
 	   global_read_lock(0),bootstrap(0)
 {
   host=user=priv_user=db=query=ip=0;
+  host_or_ip= "connecting host";
   locked=killed=count_cuted_fields=some_tables_deleted=no_errors=password=
-    query_start_used=0;
-  query_length=col_access=0;
+    query_start_used=safe_to_cache_query=0;
+  db_length=query_length=col_access=0;
   query_error=0;
   next_insert_id=last_insert_id=0;
-  open_tables=temporary_tables=0;
+  open_tables=temporary_tables=handler_tables=0;
+  current_tablenr=0;
+  handler_items=0;
   tmp_table=0;
   lock=locked_tables=0;
   used_tables=0;
-  gemini_spin_retries=0;
   cuted_fields=sent_row_count=0L;
   start_time=(time_t) 0;
   current_linfo =  0;
   slave_thread = 0;
   slave_proxy_id = 0;
-  last_nx_table = last_nx_db = 0;
+  file_id = 0;
   cond_count=0;
-  convert_set=0;
   mysys_var=0;
+#ifndef DBUG_OFF
+  dbug_sentry=THD_SENTRY_MAGIC;
+#endif  
   net.vio=0;
+  net.last_error[0]=0;				// If error on boot
   ull=0;
   system_thread=cleanup_done=0;
+  peer_port= 0;					// For SHOW PROCESSLIST
+  transaction.changed_tables = 0;
 #ifdef	__WIN__
   real_id = 0;
 #endif
-#ifdef HAVE_GEMINI_DB
-  bzero((char *)&gemini, sizeof(gemini));
-#endif
 #ifdef SIGNAL_WITH_VIO_CLOSE
   active_vio = 0;
-  pthread_mutex_init(&active_vio_lock, MY_MUTEX_INIT_FAST);
 #endif  
+  pthread_mutex_init(&LOCK_delete, MY_MUTEX_INIT_FAST);
 
   /* Variables with default values */
   proc_info="login";
   where="field list";
   server_id = ::server_id;
-  server_status=SERVER_STATUS_AUTOCOMMIT;
-  update_lock_default= low_priority_updates ? TL_WRITE_LOW_PRIORITY : TL_WRITE;
-  options=thd_startup_options;
-  sql_mode=(uint) opt_sql_mode;
-  inactive_timeout=net_wait_timeout;
-  open_options=ha_open_options;
-  tx_isolation=session_tx_isolation=default_tx_isolation;
+  slave_net = 0;
   command=COM_CONNECT;
   set_query_id=1;
-  default_select_limit= HA_POS_ERROR;
-  max_join_size=  ((::max_join_size != ~ (ulong) 0L) ? ::max_join_size :
-		   HA_POS_ERROR);
   db_access=NO_ACCESS;
+  version=refresh_version;			// For boot
 
+  init();
   /* Initialize sub structures */
   bzero((char*) &mem_root,sizeof(mem_root));
+  bzero((char*) &transaction.mem_root,sizeof(transaction.mem_root));
+  user_connect=(USER_CONN *)0;
   hash_init(&user_vars, USER_VARS_HASH_SIZE, 0, 0,
 	    (hash_get_key) get_var_key,
-	    (void (*)(void*)) free_var,0);
+	    (hash_free_key) free_user_var,0);
 #ifdef USING_TRANSACTIONS
   bzero((char*) &transaction,sizeof(transaction));
   if (opt_using_transactions)
@@ -150,19 +150,60 @@ THD::THD():user_time(0),fatal_error(0),last_insert_id_used(0),
   }
 #endif
 
-  /* 
-     We need good random number initialization for new thread
-     Just coping global one will not work 
+  /*
+    We need good random number initialization for new thread
+    Just coping global one will not work
   */
   {
     pthread_mutex_lock(&LOCK_thread_count);
-    ulong tmp=(ulong) (rnd(&sql_rand) * ((ulong)0xffffffff));
-    randominit(&rand, tmp + (ulong) start_time,
-	       tmp + (ulong) thread_id);
+    ulong tmp=(ulong) (my_rnd(&sql_rand) * 0xffffffff); /* make all bits random */
     pthread_mutex_unlock(&LOCK_thread_count);
+    randominit(&rand, tmp + (ulong) &rand, tmp + (ulong) ::query_id);
   }
 }
 
+
+/*
+  Init common variables that has to be reset on start and on change_user
+*/
+
+void THD::init(void)
+{
+  pthread_mutex_lock(&LOCK_global_system_variables);
+  variables= global_system_variables;
+  pthread_mutex_unlock(&LOCK_global_system_variables);
+  server_status= SERVER_STATUS_AUTOCOMMIT;
+  options= thd_startup_options;
+  sql_mode=(uint) opt_sql_mode;
+  open_options=ha_open_options;
+  update_lock_default= (variables.low_priority_updates ?
+			TL_WRITE_LOW_PRIORITY :
+			TL_WRITE);
+  session_tx_isolation= (enum_tx_isolation) variables.tx_isolation;
+}
+
+/*
+  Do what's needed when one invokes change user
+
+  SYNOPSIS
+    change_user()
+
+  IMPLEMENTATION
+    Reset all resources that are connection specific
+*/
+
+
+void THD::change_user(void)
+{
+  cleanup();
+  cleanup_done=0;
+  init();
+  hash_init(&user_vars, USER_VARS_HASH_SIZE, 0, 0,
+	    (hash_get_key) get_var_key,
+	    (hash_free_key) free_user_var,0);
+}
+
+
 /* Do operations that may take a long time */
 
 void THD::cleanup(void)
@@ -174,21 +215,35 @@ void THD::cleanup(void)
     lock=locked_tables; locked_tables=0;
     close_thread_tables(this);
   }
-  close_temporary_tables(this);
-#ifdef USING_TRANSACTIONS
-  if (opt_using_transactions)
+  if (handler_tables)
   {
-    close_cached_file(&transaction.trans_log);
-    ha_close_connection(this);
+    open_tables=handler_tables; handler_tables=0;
+    close_thread_tables(this);
+  }
+  close_temporary_tables(this);
+  hash_free(&user_vars);
+  if (global_read_lock)
+    unlock_global_read_lock(this);
+  if (ull)
+  {
+    pthread_mutex_lock(&LOCK_user_locks);
+    item_user_lock_release(ull);
+    pthread_mutex_unlock(&LOCK_user_locks);
+    ull= 0;
   }
-#endif
   cleanup_done=1;
   DBUG_VOID_RETURN;
 }
 
+
 THD::~THD()
 {
+  THD_CHECK_SENTRY(this);
   DBUG_ENTER("~THD()");
+  /* Ensure that no one is using THD */
+  pthread_mutex_lock(&LOCK_delete);
+  pthread_mutex_unlock(&LOCK_delete);
+
   /* Close connection */
   if (net.vio)
   {
@@ -197,23 +252,15 @@ THD::~THD()
   }
   if (!cleanup_done)
     cleanup();
-  if (global_read_lock)
+#ifdef USING_TRANSACTIONS
+  if (opt_using_transactions)
   {
-    pthread_mutex_lock(&LOCK_open);
-    ::global_read_lock--;
-    pthread_cond_broadcast(&COND_refresh);
-    pthread_mutex_unlock(&LOCK_open);
+    close_cached_file(&transaction.trans_log);
+    ha_close_connection(this);
   }
-  if (ull)
-  {
-    pthread_mutex_lock(&LOCK_user_locks);
-    item_user_lock_release(ull);
-    pthread_mutex_unlock(&LOCK_user_locks);
-  }
-  hash_free(&user_vars);
+#endif
 
   DBUG_PRINT("info", ("freeing host"));
-  
   if (host != localhost)			// If not pointer to constant
     safeFree(host);
   if (user != delayed_user)
@@ -221,45 +268,173 @@ THD::~THD()
   safeFree(db);
   safeFree(ip);
   free_root(&mem_root,MYF(0));
+  free_root(&transaction.mem_root,MYF(0));
   mysys_var=0;					// Safety (shouldn't be needed)
-#ifdef SIGNAL_WITH_VIO_CLOSE
-  pthread_mutex_destroy(&active_vio_lock);
+  pthread_mutex_destroy(&LOCK_delete);
+#ifndef DBUG_OFF
+  dbug_sentry = THD_SENTRY_GONE;
 #endif  
   DBUG_VOID_RETURN;
 }
 
-void THD::prepare_to_die()
+
+void THD::awake(bool prepare_to_die)
 {
+  THD_CHECK_SENTRY(this);
+  safe_mutex_assert_owner(&LOCK_delete); 
+
+  if (prepare_to_die)
+    killed = 1;
   thr_alarm_kill(real_id);
-  killed = 1;
 #ifdef SIGNAL_WITH_VIO_CLOSE
   close_active_vio();
 #endif    
   if (mysys_var)
+  {
+    pthread_mutex_lock(&mysys_var->mutex);
+    if (!system_thread)		// Don't abort locks
+      mysys_var->abort=1;
+    /*
+      This broadcast could be up in the air if the victim thread
+      exits the cond in the time between read and broadcast, but that is
+      ok since all we want to do is to make the victim thread get out
+      of waiting on current_cond.
+    */
+    if (mysys_var->current_cond)
     {
-      pthread_mutex_lock(&mysys_var->mutex);
-      if (!system_thread)		// Don't abort locks
-	mysys_var->abort=1;
-      if (mysys_var->current_cond)
-      {
-	pthread_mutex_lock(mysys_var->current_mutex);
-	pthread_cond_broadcast(mysys_var->current_cond);
-	pthread_mutex_unlock(mysys_var->current_mutex);
-      }
-      pthread_mutex_unlock(&mysys_var->mutex);
+      pthread_mutex_lock(mysys_var->current_mutex);
+      pthread_cond_broadcast(mysys_var->current_cond);
+      pthread_mutex_unlock(mysys_var->current_mutex);
     }
+    pthread_mutex_unlock(&mysys_var->mutex);
+  }
 }
 
-// remember the location of thread info, the structure needed for
-// sql_alloc() and the structure for the net buffer
+/*
+  Remember the location of thread info, the structure needed for
+  sql_alloc() and the structure for the net buffer
+*/
 
 bool THD::store_globals()
 {
-  return (my_pthread_setspecific_ptr(THR_THD,  this) ||
-	  my_pthread_setspecific_ptr(THR_MALLOC, &mem_root) ||
-	  my_pthread_setspecific_ptr(THR_NET,  &net));
+  if (my_pthread_setspecific_ptr(THR_THD,  this) ||
+      my_pthread_setspecific_ptr(THR_MALLOC, &mem_root) ||
+      my_pthread_setspecific_ptr(THR_NET,  &net))
+    return 1;
+  mysys_var=my_thread_var;
+  dbug_thread_id=my_thread_id();
+  /*
+    By default 'slave_proxy_id' is 'thread_id'. They may later become different
+    if this is the slave SQL thread.
+  */
+  slave_proxy_id= thread_id;
+  return 0;
 }
 
+
+/* routings to adding tables to list of changed in transaction tables */
+
+inline static void list_include(CHANGED_TABLE_LIST** prev,
+				CHANGED_TABLE_LIST* curr,
+				CHANGED_TABLE_LIST* new_table)
+{
+  if (new_table)
+  {
+    *prev = new_table;
+    (*prev)->next = curr;
+  }
+}
+
+/* add table to list of changed in transaction tables */
+
+void THD::add_changed_table(TABLE *table)
+{
+  DBUG_ENTER("THD::add_changed_table(table)");
+
+  DBUG_ASSERT((options & (OPTION_NOT_AUTOCOMMIT | OPTION_BEGIN)) &&
+	      table->file->has_transactions());
+  add_changed_table(table->table_cache_key, table->key_length);
+  DBUG_VOID_RETURN;
+}
+
+
+void THD::add_changed_table(const char *key, long key_length)
+{
+  DBUG_ENTER("THD::add_changed_table(key)");
+  CHANGED_TABLE_LIST **prev_changed = &transaction.changed_tables;
+  CHANGED_TABLE_LIST *curr = transaction.changed_tables;
+
+  for (; curr; prev_changed = &(curr->next), curr = curr->next)
+  {
+    int cmp =  (long)curr->key_length - (long)key_length;
+    if (cmp < 0)
+    {
+      list_include(prev_changed, curr, changed_table_dup(key, key_length));
+      DBUG_PRINT("info", 
+		 ("key_length %u %u", key_length, (*prev_changed)->key_length));
+      DBUG_VOID_RETURN;
+    }
+    else if (cmp == 0)
+    {
+      cmp = memcmp(curr->key, key, curr->key_length);
+      if (cmp < 0)
+      {
+	list_include(prev_changed, curr, changed_table_dup(key, key_length));
+	DBUG_PRINT("info", 
+		   ("key_length %u %u", key_length,
+		    (*prev_changed)->key_length));
+	DBUG_VOID_RETURN;
+      }
+      else if (cmp == 0)
+      {
+	DBUG_PRINT("info", ("already in list"));
+	DBUG_VOID_RETURN;
+      }
+    }
+  }
+  *prev_changed = changed_table_dup(key, key_length);
+  DBUG_PRINT("info", ("key_length %u %u", key_length,
+		      (*prev_changed)->key_length));
+  DBUG_VOID_RETURN;
+}
+
+
+CHANGED_TABLE_LIST* THD::changed_table_dup(const char *key, long key_length)
+{
+  CHANGED_TABLE_LIST* new_table = 
+    (CHANGED_TABLE_LIST*) trans_alloc(ALIGN_SIZE(sizeof(CHANGED_TABLE_LIST))+
+				      key_length + 1);
+  if (!new_table)
+  {
+    my_error(EE_OUTOFMEMORY, MYF(ME_BELL),
+	     ALIGN_SIZE(sizeof(TABLE_LIST)) + key_length + 1);
+    killed= 1;
+    return 0;
+  }
+
+  new_table->key = (char *) (((byte*)new_table)+
+			     ALIGN_SIZE(sizeof(CHANGED_TABLE_LIST)));
+  new_table->next = 0;
+  new_table->key_length = key_length;
+  ::memcpy(new_table->key, key, key_length);
+  return new_table;
+}
+
+
+#ifdef SIGNAL_WITH_VIO_CLOSE
+void THD::close_active_vio()
+{
+  DBUG_ENTER("close_active_vio");
+  safe_mutex_assert_owner(&LOCK_delete); 
+  if (active_vio)
+  {
+    vio_close(active_vio);
+    active_vio = 0;
+  }
+  DBUG_VOID_RETURN;
+}
+#endif
+
 /*****************************************************************************
 ** Functions to provide a interface to select results
 *****************************************************************************/
@@ -291,10 +466,18 @@ bool select_send::send_fields(List<Item> &list,uint flag)
 
 bool select_send::send_data(List<Item> &items)
 {
-  List_iterator<Item> li(items);
+  List_iterator_fast<Item> li(items);
   String *packet= &thd->packet;
   DBUG_ENTER("send_data");
 
+#ifdef HAVE_INNOBASE_DB
+  /* We may be passing the control from mysqld to the client: release the
+     InnoDB adaptive hash S-latch to avoid thread deadlocks if it was reserved
+     by thd */
+  if (thd->transaction.all.innobase_tid)
+    ha_release_temporary_latches(thd);
+#endif
+
   if (thd->offset_limit)
   {						// using limit offset,count
     thd->offset_limit--;
@@ -304,7 +487,7 @@ bool select_send::send_data(List<Item> &items)
   Item *item;
   while ((item=li++))
   {
-    if (item->send(packet))
+    if (item->send(thd, packet))
     {
       packet->free();				// Free used
       my_error(ER_OUT_OF_RESOURCES,MYF(0));
@@ -312,18 +495,22 @@ bool select_send::send_data(List<Item> &items)
     }
   }
   thd->sent_row_count++;
+  if (!thd->net.vio)
+    DBUG_RETURN(0);
   bool error=my_net_write(&thd->net,(char*) packet->ptr(),packet->length());
   DBUG_RETURN(error);
 }
 
-
-void select_send::send_error(uint errcode,const char *err)
-{
-  ::send_error(&thd->net,errcode,err);
-}
-
 bool select_send::send_eof()
 {
+#ifdef HAVE_INNOBASE_DB
+  /* We may be passing the control from mysqld to the client: release the
+     InnoDB adaptive hash S-latch to avoid thread deadlocks if it was reserved
+     by thd */
+  if (thd->transaction.all.innobase_tid)
+    ha_release_temporary_latches(thd);
+#endif
+
   /* Unlock tables before sending packet to gain some speed */
   if (thd->lock)
   {
@@ -353,7 +540,6 @@ select_export::~select_export()
 int
 select_export::prepare(List<Item> &list)
 {
-  char path[FN_REFLEN];
   uint option=4;
   bool blob_flag=0;
 #ifdef DONT_ALLOW_FULL_LOAD_DATA_PATHS
@@ -384,7 +570,7 @@ select_export::prepare(List<Item> &list)
   }
   /* Check if there is any blobs in data */
   {
-    List_iterator<Item> li(list);
+    List_iterator_fast<Item> li(list);
     Item *item;
     while ((item=li++))
     {
@@ -431,7 +617,7 @@ bool select_export::send_data(List<Item> &items)
   Item *item;
   char *buff_ptr=buff;
   uint used_length=0,items_left=items.elements;
-  List_iterator<Item> li(items);
+  List_iterator_fast<Item> li(items);
 
   if (my_b_write(&cache,(byte*) exchange->line_start->ptr(),
 		 exchange->line_start->length()))
@@ -519,7 +705,7 @@ bool select_export::send_data(List<Item> &items)
 	  bfill(space,sizeof(space),' ');
 	}
 	uint length=item->max_length-used_length;
-	for ( ; length > sizeof(space) ; length-=sizeof(space))
+	for (; length > sizeof(space) ; length-=sizeof(space))
 	{
 	  if (my_b_write(&cache,(byte*) space,sizeof(space)))
 	    goto err;
@@ -554,9 +740,13 @@ err:
 void select_export::send_error(uint errcode,const char *err)
 {
   ::send_error(&thd->net,errcode,err);
-  (void) end_io_cache(&cache);
-  (void) my_close(file,MYF(0));
-  file= -1;
+  if (file > 0)
+  {
+    (void) end_io_cache(&cache);
+    (void) my_close(file,MYF(0));
+    (void) my_delete(path,MYF(0));		// Delete file on error
+    file= -1;
+  }
 }
 
 
@@ -624,7 +814,7 @@ select_dump::prepare(List<Item> &list __attribute__((unused)))
 
 bool select_dump::send_data(List<Item> &items)
 {
-  List_iterator<Item> li(items);
+  List_iterator_fast<Item> li(items);
   char buff[MAX_FIELD_WIDTH];
   String tmp(buff,sizeof(buff)),*res;
   tmp.length(0);
@@ -664,10 +854,13 @@ err:
 void select_dump::send_error(uint errcode,const char *err)
 {
   ::send_error(&thd->net,errcode,err);
-  (void) end_io_cache(&cache);
-  (void) my_close(file,MYF(0));
-  (void) my_delete(path,MYF(0));		// Delete file on error
-  file= -1;
+  if (file > 0)
+  {
+    (void) end_io_cache(&cache);
+    (void) my_close(file,MYF(0));
+    (void) my_delete(path,MYF(0));		// Delete file on error
+    file= -1;
+  }
 }
 
 
