@@ -1,15 +1,15 @@
 /* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
@@ -46,19 +46,20 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   reg2 uchar *strpos;
   int	 j,error;
   uint	 rec_buff_length,n_length,int_length,records,key_parts,keys,
-    interval_count,interval_parts,read_length,db_create_options;
+         interval_count,interval_parts,read_length,db_create_options;
+  uint	 key_info_length, com_length;
   ulong  pos;
-  char	 index_file[FN_REFLEN], *names,*keynames;
+  char	 index_file[FN_REFLEN], *names, *keynames;
   uchar  head[288],*disk_buff,new_field_pack_flag;
   my_string record;
   const char **int_array;
-  bool	 new_frm_ver,use_hash, null_field_first;
+  bool	 use_hash, null_field_first;
   File	 file;
   Field  **field_ptr,*reg_field;
   KEY	 *keyinfo;
   KEY_PART_INFO *key_part;
   uchar *null_pos;
-  uint null_bit;
+  uint null_bit, new_frm_ver, field_pack_length;
   SQL_CRYPT *crypted=0;
   DBUG_ENTER("openfrm");
   DBUG_PRINT("enter",("name: '%s'  form: %lx",name,outparam));
@@ -69,7 +70,7 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   outparam->db_stat = db_stat;
   error=1;
 
-  init_sql_alloc(&outparam->mem_root,1024,0);
+  init_sql_alloc(&outparam->mem_root, TABLE_ALLOC_BLOCK_SIZE, 0);
   MEM_ROOT *old_root=my_pthread_getspecific_ptr(MEM_ROOT*,THR_MALLOC);
   my_pthread_setspecific_ptr(THR_MALLOC,&outparam->mem_root);
 
@@ -94,14 +95,15 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 
   if (my_read(file,(byte*) head,64,MYF(MY_NABP))) goto err_not_open;
   if (head[0] != (uchar) 254 || head[1] != 1 ||
-      (head[2] != FRM_VER && head[2] != FRM_VER+1))
-    goto err_not_open; /* purecov: inspected */
+      (head[2] != FRM_VER && head[2] != FRM_VER+1 && head[2] != FRM_VER+3))
+    goto err_not_open;				/* purecov: inspected */
   new_field_pack_flag=head[27];
-  new_frm_ver= (head[2] == FRM_VER+1);
+  new_frm_ver= (head[2] - FRM_VER);
+  field_pack_length= new_frm_ver < 2 ? 11 : 17;
 
   error=3;
   if (!(pos=get_form_pos(file,head,(TYPELIB*) 0)))
-    goto err_not_open; /* purecov: inspected */
+    goto err_not_open;				/* purecov: inspected */
   *fn_ext(index_file)='\0';			// Remove .frm extension
 
   outparam->db_type=ha_checktype((enum db_type) (uint) *(head+3));
@@ -127,11 +129,12 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   outparam->min_rows=uint4korr(head+22);
 
   /* Read keyinformation */
+  key_info_length= (uint) uint2korr(head+28);
   VOID(my_seek(file,(ulong) uint2korr(head+6),MY_SEEK_SET,MYF(0)));
-  if (read_string(file,(gptr*) &disk_buff,(uint) uint2korr(head+28)))
+  if (read_string(file,(gptr*) &disk_buff,key_info_length))
     goto err_not_open; /* purecov: inspected */
   outparam->keys=keys=   disk_buff[0];
-  outparam->keys_in_use= set_bits(key_map, keys);
+  outparam->keys_for_keyread= outparam->keys_in_use= set_bits(key_map, keys);
 
   outparam->key_parts=key_parts=disk_buff[1];
   n_length=keys*sizeof(KEY)+key_parts*sizeof(KEY_PART_INFO);
@@ -140,8 +143,8 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
     goto err_not_open; /* purecov: inspected */
   bzero((char*) keyinfo,n_length);
   outparam->key_info=keyinfo;
-  outparam->max_key_length=0;
-  key_part= (KEY_PART_INFO*) (keyinfo+keys);
+  outparam->max_key_length= outparam->total_key_length= 0;
+  key_part= my_reinterpret_cast(KEY_PART_INFO*) (keyinfo+keys);
   strpos=disk_buff+6;
 
   ulong *rec_per_key;
@@ -151,9 +154,23 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 
   for (i=0 ; i < keys ; i++, keyinfo++)
   {
-    keyinfo->flags=	 ((uint) strpos[0]) ^ HA_NOSAME;
-    keyinfo->key_length= (uint) uint2korr(strpos+1);
-    keyinfo->key_parts=  (uint) strpos[3];  strpos+=4;
+    if (new_frm_ver == 3)
+    {
+      keyinfo->flags=	   (uint) uint2korr(strpos) ^ HA_NOSAME;
+      keyinfo->key_length= (uint) uint2korr(strpos+2);
+      keyinfo->key_parts=  (uint) strpos[4];
+      keyinfo->algorithm=  (enum ha_key_alg) strpos[5];
+      strpos+=8;
+    }
+    else
+    {
+      keyinfo->flags=	 ((uint) strpos[0]) ^ HA_NOSAME;
+      keyinfo->key_length= (uint) uint2korr(strpos+1);
+      keyinfo->key_parts=  (uint) strpos[3];
+      keyinfo->algorithm= HA_KEY_ALG_UNDEF;
+      strpos+=4;
+    }
+
     keyinfo->key_part=	 key_part;
     keyinfo->rec_per_key= rec_per_key;
     for (j=keyinfo->key_parts ; j-- ; key_part++)
@@ -163,7 +180,7 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
       key_part->offset= (uint) uint2korr(strpos+2)-1;
       key_part->key_type=	(uint) uint2korr(strpos+5);
       // key_part->field=	(Field*) 0;	// Will be fixed later
-      if (new_frm_ver)
+      if (new_frm_ver >= 1)
       {
 	key_part->key_part_flag= *(strpos+4);
 	key_part->length=	(uint) uint2korr(strpos+7);
@@ -184,11 +201,13 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
     }
     set_if_bigger(outparam->max_key_length,keyinfo->key_length+
 		  keyinfo->key_parts);
+    outparam->total_key_length+= keyinfo->key_length;
     if (keyinfo->flags & HA_NOSAME)
       set_if_bigger(outparam->max_unique_length,keyinfo->key_length);
   }
+  keynames=(char*) key_part;
+  strpos+= (strmov(keynames, (char *) strpos) - keynames)+1;
 
-  (void) strmov(keynames= (char *) key_part,(char *) strpos);
   outparam->reclength = uint2korr((head+16));
   if (*(head+26) == 1)
     outparam->system=1;				/* one-record-database */
@@ -256,10 +275,11 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   interval_parts=uint2korr(head+272);
   int_length=uint2korr(head+274);
   outparam->null_fields=uint2korr(head+282);
+  com_length=uint2korr(head+284);
   outparam->comment=strdup_root(&outparam->mem_root,
 				(char*) head+47);
 
-  DBUG_PRINT("form",("i_count: %d  i_parts: %d  index: %d  n_length: %d  int_length: %d", interval_count,interval_parts, outparam->keys,n_length,int_length));
+  DBUG_PRINT("info",("i_count: %d  i_parts: %d  index: %d  n_length: %d  int_length: %d", interval_count,interval_parts, outparam->keys,n_length,int_length));
 
   if (!(field_ptr = (Field **)
 	alloc_root(&outparam->mem_root,
@@ -267,12 +287,12 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 			   interval_count*sizeof(TYPELIB)+
 			   (outparam->fields+interval_parts+
 			    keys+3)*sizeof(my_string)+
-			   (n_length+int_length)))))
+			   (n_length+int_length+com_length)))))
     goto err_not_open; /* purecov: inspected */
 
   outparam->field=field_ptr;
-  read_length=((uint) (outparam->fields*11)+pos+
-	       (uint) (n_length+int_length));
+  read_length=(uint) (outparam->fields * field_pack_length +
+		      pos+ (uint) (n_length+int_length+com_length));
   if (read_string(file,(gptr*) &disk_buff,read_length))
     goto err_not_open; /* purecov: inspected */
   if (crypted)
@@ -288,7 +308,7 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   names= (char*) (int_array+outparam->fields+interval_parts+keys+3);
   if (!interval_count)
     outparam->intervals=0;			// For better debugging
-  memcpy((char*) names, strpos+(outparam->fields*11),
+  memcpy((char*) names, strpos+(outparam->fields*field_pack_length),
 	 (uint) (n_length+int_length));
 
   fix_type_pointers(&int_array,&outparam->fieldnames,1,&names);
@@ -321,22 +341,49 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 			 (hash_get_key) get_field_name,0,
 			 HASH_CASE_INSENSITIVE);
 
-  for (i=0 ; i < outparam->fields; i++, strpos+= 11, field_ptr++)
+  for (i=0 ; i < outparam->fields; i++, strpos+=field_pack_length, field_ptr++)
   {
-    uint pack_flag= uint2korr(strpos+6);
-    uint interval_nr= (uint) strpos[10];
+    uint pack_flag, interval_nr, unireg_type, recpos, field_length;
+    enum_field_types field_type;
+
+    if (new_frm_ver == 3)
+    {
+      /* new frm file in 4.1 */
+      field_length= uint2korr(strpos+3);
+      recpos=	    uint3korr(strpos+5);
+      pack_flag=    uint2korr(strpos+8);
+      unireg_type=  (uint) strpos[10];
+      interval_nr=  (uint) strpos[12];
+      field_type= (enum_field_types) (uint) strpos[13];
+    }
+    else
+    {
+      /* old frm file */
+      field_length= (uint) strpos[3];
+      recpos=	    uint2korr(strpos+4),
+      pack_flag=    uint2korr(strpos+6);
+      unireg_type=  (uint) strpos[8];
+      interval_nr=  (uint) strpos[10];
+      field_type= (enum_field_types) f_packtype(pack_flag);
+    }
 
     *field_ptr=reg_field=
-      make_field(record+uint2korr(strpos+4),
-		 (uint32) strpos[3],		// field_length
+      make_field(record+recpos,
+		 (uint32) field_length,
 		 null_pos,null_bit,
 		 pack_flag,
-		 (Field::utype) MTYP_TYPENR((uint) strpos[8]),
+		 field_type,
+		 (Field::utype) MTYP_TYPENR(unireg_type),
 		 (interval_nr ?
 		  outparam->intervals+interval_nr-1 :
 		  (TYPELIB*) 0),
 		 outparam->fieldnames.type_names[i],
 		 outparam);
+    if (!*field_ptr)				// Field in 4.1
+    {
+      error= 4;
+      goto err_not_open;			/* purecov: inspected */
+    }
     if (!(reg_field->flags & NOT_NULL_FLAG))
     {
       if ((null_bit<<=1) == 256)
@@ -359,25 +406,41 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   {
     uint primary_key=(uint) (find_type((char*) "PRIMARY",&outparam->keynames,
 				       3)-1);
-    uint ha_option=outparam->file->option_flag();
+    uint ha_option=outparam->file->table_flags();
     keyinfo=outparam->key_info;
     key_part=keyinfo->key_part;
 
     for (uint key=0 ; key < outparam->keys ; key++,keyinfo++)
     {
       uint usable_parts=0;
+      ulong index_flags;
       keyinfo->name=(char*) outparam->keynames.type_names[key];
+      /* Fix fulltext keys for old .frm files */
+      if (outparam->key_info[key].flags & HA_FULLTEXT)
+	outparam->key_info[key].algorithm= HA_KEY_ALG_FULLTEXT;
+
+      /* This has to be done after the above fulltext correction */
+      index_flags=outparam->file->index_flags(key);
+      if (!(index_flags & HA_KEY_READ_ONLY))
+      {
+	outparam->read_only_keys|=    ((key_map) 1 << key);
+	outparam->keys_for_keyread&= ~((key_map) 1 << key);
+      }
+
       if (primary_key >= MAX_KEY && (keyinfo->flags & HA_NOSAME))
       {
 	/*
-	  If the UNIQUE key don't have NULL columns, declare this as
-	  a primary key.
+	  If the UNIQUE key doesn't have NULL columns and is not a part key
+	  declare this as a primary key.
 	*/
 	primary_key=key;
 	for (i=0 ; i < keyinfo->key_parts ;i++)
 	{
-	  if (!key_part[i].fieldnr ||
-	      outparam->field[key_part[i].fieldnr-1]->null_ptr)
+	  uint fieldnr= key_part[i].fieldnr;
+	  if (!fieldnr ||
+	      outparam->field[fieldnr-1]->null_ptr ||
+	      outparam->field[fieldnr-1]->key_length() !=
+	      key_part[i].length)
 	  {
 	    primary_key=MAX_KEY;		// Can't be used
 	    break;
@@ -416,6 +479,12 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	    keyinfo->extra_length+=HA_KEY_BLOB_LENGTH;
 	    key_part->store_length+=HA_KEY_BLOB_LENGTH;
 	    keyinfo->key_length+= HA_KEY_BLOB_LENGTH;
+	    /*
+	      Mark that there may be many matching values for one key
+	      combination ('a', 'a ', 'a  '...)
+	    */
+	    if (!(field->flags & BINARY_FLAG))
+	      keyinfo->flags|= HA_END_SPACE_KEY;
 	  }
 	  if (i == 0 && key != primary_key)
 	    field->flags |=
@@ -424,16 +493,18 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	       keyinfo->key_length ? UNIQUE_KEY_FLAG : MULTIPLE_KEY_FLAG);
 	  if (i == 0)
 	    field->key_start|= ((key_map) 1 << key);
-	  if ((ha_option & HA_HAVE_KEY_READ_ONLY) &&
+	  if ((index_flags & HA_KEY_READ_ONLY) &&
 	      field->key_length() == key_part->length &&
 	      field->type() != FIELD_TYPE_BLOB)
 	  {
 	    if (field->key_type() != HA_KEYTYPE_TEXT ||
-		(!(ha_option & HA_KEY_READ_WRONG_STR) &&
+		((!(ha_option & HA_KEY_READ_WRONG_STR) ||
+		  field->flags & BINARY_FLAG) &&
 		 !(keyinfo->flags & HA_FULLTEXT)))
 	      field->part_of_key|= ((key_map) 1 << key);
-	    if (field->key_type() != HA_KEYTYPE_TEXT ||
-		!(keyinfo->flags & HA_FULLTEXT))
+	    if ((field->key_type() != HA_KEYTYPE_TEXT ||
+		 !(keyinfo->flags & HA_FULLTEXT)) &&
+		!(index_flags & HA_WRONG_ASCII_ORDER))
 	      field->part_of_sortkey|= ((key_map) 1 << key);
 	  }
 	  if (!(key_part->key_part_flag & HA_REVERSE_SORT) &&
@@ -443,15 +514,20 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	  if (key == primary_key)
 	  {
 	    field->flags|= PRI_KEY_FLAG;
+	    /*
+	      If this field is part of the primary key and all keys contains
+	      the primary key, then we can use any key to find this column
+	    */
 	    if (ha_option & HA_PRIMARY_KEY_IN_READ_INDEX)
-	      field->part_of_key|= ((key_map) 1 << primary_key);
+	      field->part_of_key= outparam->keys_in_use;
 	  }
 	  if (field->key_length() != key_part->length)
 	  {
-	    key_part->key_part_flag|= HA_PART_KEY;
+	    key_part->key_part_flag|= HA_PART_KEY_SEG;
 	    if (field->type() != FIELD_TYPE_BLOB)
 	    {					// Create a new field
-	      field=key_part->field=field->new_field(outparam);
+	      field=key_part->field=field->new_field(&outparam->mem_root,
+						     outparam);
 	      field->field_length=key_part->length;
 	    }
 	  }
@@ -461,7 +537,7 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	    as we need to test for NULL = NULL.
 	  */
 	  if (field->real_maybe_null())
-	    key_part->key_part_flag|= HA_PART_KEY;
+	    key_part->key_part_flag|= HA_PART_KEY_SEG;
 	}
 	else
 	{					// Error: shorten key
@@ -475,8 +551,6 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	(outparam->keys_in_use & ((key_map) 1 << primary_key)))
     {
       outparam->primary_key=primary_key;
-      if (outparam->file->option_flag() & HA_PRIMARY_KEY_IN_READ_INDEX)
-	outparam->ref_primary_key= (key_map) 1 << primary_key;
       /*
 	If we are using an integer as the primary key then allow the user to
 	refer to it as '_rowid'
@@ -504,6 +578,7 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
       bfill(outparam->null_flags+outparam->rec_buff_length*2,null_length,255);
   }
 
+
   if ((reg_field=outparam->found_next_number_field))
   {
     if ((int) (outparam->next_number_index= (uint)
@@ -543,8 +618,9 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   if (db_stat)
   {
     int err;
+    unpack_filename(index_file,index_file);
     if ((err=(outparam->file->
-	      ha_open(unpack_filename(index_file,index_file),
+	      ha_open(index_file,
 		      (db_stat & HA_READ_ONLY ? O_RDONLY : O_RDWR),
 		      (db_stat & HA_OPEN_TEMPORARY ? HA_OPEN_TMP_TABLE :
 		       ((db_stat & HA_WAIT_IF_LOCKED) ||
@@ -757,7 +833,7 @@ ulong make_new_entry(File file, uchar *fileinfo, TYPELIB *formnames,
 
   int2store(fileinfo+8,names+1);
   int2store(fileinfo+4,n_length+length);
-  VOID(my_chsize(file,newpos,MYF(MY_WME)));	/* Append file with '\0' */
+  VOID(my_chsize(file, newpos, 0, MYF(MY_WME)));/* Append file with '\0' */
   DBUG_RETURN(newpos);
 } /* make_new_entry */
 
@@ -831,7 +907,7 @@ fix_type_pointers(const char ***array, TYPELIB *point_to_type, uint types,
 	*type_name= '\0';		/* End string */
 	ptr=type_name;
       }
-      ptr+=2;				/* Skipp end mark and last 0 */
+      ptr+=2;				/* Skip end mark and last 0 */
     }
     else
       ptr++;
@@ -917,7 +993,7 @@ ulong next_io_size(register ulong pos)
 
 void append_unescaped(String *res,const char *pos)
 {
-  for ( ; *pos ; pos++)
+  for (; *pos ; pos++)
   {
     switch (*pos) {
     case 0:				/* Must be escaped for 'mysql' */
@@ -964,6 +1040,11 @@ File create_frm(register my_string name, uint reclength, uchar *fileinfo,
   if (create_info->min_rows > ~(ulong) 0)
     create_info->min_rows= ~(ulong) 0;
 #endif
+  /*
+    Ensure that raid_chunks can't be larger than 255, as this would cause
+    problems with drop database
+  */
+  set_if_smaller(create_info->raid_chunks, 255);
 
   if ((file=my_create(name,CREATE_MODE,O_RDWR | O_TRUNC,MYF(MY_WME))) >= 0)
   {
@@ -1005,6 +1086,7 @@ File create_frm(register my_string name, uint reclength, uchar *fileinfo,
 
 void update_create_info_from_table(HA_CREATE_INFO *create_info, TABLE *table)
 {
+  DBUG_ENTER("update_create_info_from_table");
   create_info->max_rows=table->max_rows;
   create_info->min_rows=table->min_rows;
   create_info->table_options=table->db_create_options;
@@ -1013,7 +1095,8 @@ void update_create_info_from_table(HA_CREATE_INFO *create_info, TABLE *table)
   create_info->raid_type=table->raid_type;
   create_info->raid_chunks=table->raid_chunks;
   create_info->raid_chunksize=table->raid_chunksize;
-}  
+  DBUG_VOID_RETURN;
+}
 
 int
 rename_file_ext(const char * from,const char * to,const char * ext)
@@ -1033,13 +1116,12 @@ rename_file_ext(const char * from,const char * to,const char * ext)
 char *get_field(MEM_ROOT *mem, TABLE *table, uint fieldnr)
 {
   Field *field=table->field[fieldnr];
-  char buff[MAX_FIELD_WIDTH];
+  char buff[MAX_FIELD_WIDTH], *to;
   String str(buff,sizeof(buff));
   field->val_str(&str,&str);
   uint length=str.length();
-  if (!length)
+  if (!length || !(to= (char*) alloc_root(mem,length+1)))
     return NullS;
-  char *to= (char*) alloc_root(mem,length+1);
   memcpy(to,str.ptr(),(uint) length);
   to[length]=0;
   return to;
@@ -1063,7 +1145,8 @@ char *get_field(MEM_ROOT *mem, TABLE *table, uint fieldnr)
 
 bool check_db_name(char *name)
 {
-   char *start=name;
+  char *start=name;
+  bool last_char_is_space= FALSE;
 
   if (lower_case_table_names)
     casedn_str(name);
@@ -1071,6 +1154,7 @@ bool check_db_name(char *name)
   while (*name)
   {
 #if defined(USE_MB) && defined(USE_MB_IDENT)
+    last_char_is_space= my_isspace(default_charset_info, *name);
     if (use_mb(default_charset_info))
     {
       int len=my_ismbchar(default_charset_info, name, name+MBMAXLEN);
@@ -1080,18 +1164,22 @@ bool check_db_name(char *name)
         continue;
       }
     }
+#else
+    last_char_is_space= *name==' ';
 #endif
-    if (*name == '/' || *name == FN_LIBCHAR || *name == FN_EXTCHAR)
+    if (*name == '/' || *name == '\\' || *name == FN_LIBCHAR ||
+	*name == FN_EXTCHAR)
       return 1;
     name++;
   }
-  return (uint) (name - start) > NAME_LEN;
+  return last_char_is_space || (uint) (name - start) > NAME_LEN;
 }
 
 
 /*
   Allow anything as a table name, as long as it doesn't contain an
   a '/', or a '.' character
+  or ' ' at the end
   returns 1 on error
 */
 
@@ -1099,10 +1187,19 @@ bool check_db_name(char *name)
 bool check_table_name(const char *name, uint length)
 {
   const char *end= name+length;
+  if (!length || length > NAME_LEN)
+    return 1;
+#if defined(USE_MB) && defined(USE_MB_IDENT)
+  bool last_char_is_space= FALSE;
+#else
+  if (name[length-1]==' ')
+    return 1;
+#endif
 
   while (name != end)
   {
 #if defined(USE_MB) && defined(USE_MB_IDENT)
+    last_char_is_space= my_isspace(default_charset_info, *name);
     if (use_mb(default_charset_info))
     {
       int len=my_ismbchar(default_charset_info, name, end);
@@ -1113,18 +1210,27 @@ bool check_table_name(const char *name, uint length)
       }
     }
 #endif
-    if (*name == '/' || *name == FN_LIBCHAR || *name == FN_EXTCHAR)
+    if (*name == '/' || *name == '\\' || *name == FN_EXTCHAR)
       return 1;
     name++;
   }
+#if defined(USE_MB) && defined(USE_MB_IDENT)
+  return last_char_is_space;
+#else
   return 0;
+#endif
 }
 
+
 bool check_column_name(const char *name)
 {
+  const char *start= name;
+  bool last_char_is_space= FALSE;
+  
   while (*name)
   {
 #if defined(USE_MB) && defined(USE_MB_IDENT)
+    last_char_is_space= my_isspace(default_charset_info, *name);
     if (use_mb(default_charset_info))
     {
       int len=my_ismbchar(default_charset_info, name, name+MBMAXLEN);
@@ -1134,12 +1240,15 @@ bool check_column_name(const char *name)
         continue;
       }
     }
+#else
+    last_char_is_space= *name==' ';
 #endif
     if (*name == NAMES_SEP_CHAR)
       return 1;
     name++;
   }
-  return 0;
+  /* Error if empty or too long column name */
+  return last_char_is_space || (name == start || (uint) (name - start) > NAME_LEN);
 }
 
 /*
@@ -1159,7 +1268,7 @@ db_type get_table_type(const char *name)
   error=my_read(file,(byte*) head,4,MYF(MY_NABP));
   my_close(file,MYF(0));
   if (error || head[0] != (uchar) 254 || head[1] != 1 ||
-      (head[2] != FRM_VER && head[2] != FRM_VER+1))
+      (head[2] != FRM_VER && head[2] != FRM_VER+1 && head[2] != FRM_VER+3))
     DBUG_RETURN(DB_TYPE_UNKNOWN);
   DBUG_RETURN(ha_checktype((enum db_type) (uint) *(head+3)));
 }
