@@ -16,8 +16,9 @@
 
 /* This is defines strtoll() if neaded */
 
+#define strtoll glob_strtoll		/* Fix for True64 */
+
 #include <my_global.h>
-#include <m_string.h>
 #if !defined(HAVE_STRTOLL) && defined(HAVE_LONG_LONG)
 #define USE_LONGLONG
 #include "strto.c"
